#include <mios32.h>
#include <seq_bpm.h>
#include <seq_midi_out.h>
#include "seq.h"
#include "app.h"

static s32 SEQ_Tick(u32 bpm_tick);

static u16 main_step_pos = 1; // the Main - Song position (you dont hear that, that is only for SYNC.
   
static u8 seq_pause = 0; // pause mode (will be controlled from user interface)

static s16 TicCounterNorm   = 0; //To count Normal/NotSwinged Steps Delay
        s16 StepSend = 0; //Memorized StepNr
static s16 TicCounter8      = 0; //To Count 8th swinged Steps Delay
static s16 TicCounter16     = 0; //To Count 8th swinged Steps Delay
static u8 FirstTicFlag      = 1;//to initalize the device to midiclock

static u16 setPpqnDivider = 4; //to be shure that the 16th-PPQM-divider is changed in main-sync
       s16 TicSend = 0;
  u8 StepOffset = 0; //Number off steps to delayd (in Order to get better in sync with other hardware)
  u8 intro = 0;


s32 SEQ_Init(u32 mode){
    SEQ_Reset();  // reset sequencer
    SEQ_BPM_Init(0);  // init BPM generator
    SEQ_BPM_PPQN_Set(Tic);
    SEQ_BPM_Set(BPM);
    SEQ_BPM_TickSet(0); 
  return 0; // no error
}

s32 SEQ_Handler(void){ // this sequencer handler is called periodically to check for new requests from BPM generator
  // handle requests

  u8 num_loops = 0;
  u8 again = 0;
  do {
    ++num_loops;

    // note: don't remove any request check - clocks won't be propagated
    // so long any Stop/Cont/Start/SongPos event hasn't been flagged to the sequencer
    if( SEQ_BPM_ChkReqStop() ) {
     // SEQ_PlayOffEvents();
    }

    if( SEQ_BPM_ChkReqCont() ) {
      // release pause mode
      seq_pause = 0;
    }

    if( SEQ_BPM_ChkReqStart() ) {
      SEQ_Reset();
    }

    u16 new_song_pos;
    if( SEQ_BPM_ChkReqSongPos(&new_song_pos) ) {
   //   SEQ_PlayOffEvents();
    }

    u32 bpm_tick;
    if( SEQ_BPM_ChkReqClk(&bpm_tick) > 0 ) {
      again = 1; // check all requests again after execution of this part

      SEQ_Tick(bpm_tick);
    }
  } while( again && num_loops < 10 );

  return 0; // no error
}

s32 SEQ_Reset(void){
  seq_pause = 0;   // release pause mode
  SEQ_BPM_TickSet(0);   // reset BPM tick
  return 0; // no error
}

s32 SEQ_Loop(void){return 0;}


static s32 SEQ_Tick(u32 bpm_tick){  // performs a single bpm tick
  //MAIN RESET ON FIRST TIC EVER... (PLAY-Button)
  if( bpm_tick == 0 ) { main_step_pos = -1; TicCounterNorm = 1; TicCounter16 = 1; TicCounter8 = 1; FirstTicFlag = 1; SyncFlag = 1;
                        loop_step_pos[0] = StepOffset;   // start immediatly @ Loop Point
                        loop_step_pos[1] = StepOffset;  // start immediatly @ Loop Point
                        loop_step_pos[2] = StepOffset;  // start immediatly @ Loop Point
                        loop_step_pos[3] = StepOffset;  // start immediatly @ Loop Point
                        loop_step_pos[4] = StepOffset;  // start immediatly @ Loop Point
                        loop_step_pos[5] = StepOffset;  // start immediatly @ Loop Point
                        loop_step_pos[6] = StepOffset;  // start immediatly @ Loop Point
                        loop_step_pos[7] = StepOffset;}


  //96tic loop
  TicSend = (bpm_tick % (SEQ_BPM_PPQN_Get()/setPpqnDivider));

  //Step-Count --Running Light
  if(TicSend == 0 ) { // whenever we reach a new 16th note (e.g 96 tics @384 ppqn):
         //MAIN LOOP COUNTER
         if( ++main_step_pos >= StepPerPage ) main_step_pos = 0; // increment step number until it reaches the mainloop length of a page /tact system

         //Set new Loop (in Main Sync)static u8 FirstTicFlag      = 1;//to initalize the device to midiclock
         if(main_step_pos == 0) {
                if(SyncFlag  == 1){//Something has changed, Set new Loop point when MainLoop is finished
                   SyncFlag = 3; //New SyncFlag to avoid double triggering events.
                   //set new step position 
                    loop_step_pos[0] = StepOffset;
                    loop_step_pos[1] = StepOffset;
                    loop_step_pos[2] = StepOffset;
                    loop_step_pos[3] = StepOffset;
                    loop_step_pos[4] = StepOffset;
                    loop_step_pos[5] = StepOffset;
                    loop_step_pos[6] = StepOffset;
                    loop_step_pos[7] = StepOffset;}                 

              if(SyncFlag == 4) {
                 SEQ_BPM_PPQN_Set(Tic); //set TICs (normally 384)
                 SyncFlag = 3;} //New SyncFlag to avoid double triggering events.

              if(SyncFlag == 5) {
                 setPpqnDivider = PpqnDivider; //set PPQN Divider in time
                 SyncFlag = 3;} //New SyncFlag to avoid double triggering events.


              if(SyncFlag == 4) {
                 SEQ_BPM_PPQN_Set(Tic); //set TICs (normally 384)
                 SyncFlag = 3;} //New SyncFlag to avoid double triggering events.

              if(SyncFlag == 5) {
                 setPpqnDivider = PpqnDivider; //set PPQN Divider in time
                 SyncFlag = 3;} //New SyncFlag to avoid double triggering events.
             }


           //Avoid Additional Counter ++ after giving new Loop Step Positions
	       if(SyncFlag != 3) { if( ++loop_step_pos[0] >= ((Loop_Length[0]+1) * StepPerPage))    loop_step_pos[0] = StepOffset;
	                           if( ++loop_step_pos[1] >= ((Loop_Length[1]+1) * StepPerPage))    loop_step_pos[1] = StepOffset;
	                           if( ++loop_step_pos[2] >= ((Loop_Length[2]+1) * StepPerPage))    loop_step_pos[2] = StepOffset;
	                           if( ++loop_step_pos[3] >= ((Loop_Length[3]+1) * StepPerPage))    loop_step_pos[3] = StepOffset;
	                           if( ++loop_step_pos[4] >= ((Loop_Length[4]+1) * StepPerPage))    loop_step_pos[4] = StepOffset;
	                           if( ++loop_step_pos[5] >= ((Loop_Length[5]+1) * StepPerPage))    loop_step_pos[5] = StepOffset;
	                           if( ++loop_step_pos[6] >= ((Loop_Length[6]+1) * StepPerPage))    loop_step_pos[6] = StepOffset;
	                           if( ++loop_step_pos[7] >= ((Loop_Length[7]+1) * StepPerPage))    loop_step_pos[7] = StepOffset;}
           //Reset Syncflac
	       if(SyncFlag == 3) { SyncFlag = 0;}
        
         //LEDRING Loop Position Indication
            if( ( MelOmute[0] == 1) || (REC[0] == 1) ) {
                   IND[0] = ((10000*loop_step_pos[0]/(StepPerPage*(Loop_Length[0]+1))) * 16)/10000;} //show Loop Position on a LED-RING 

            
            if( ( MelOmute[1] == 1) || (REC[1] == 1) ) {
                   IND[1] = ((10000*loop_step_pos[1]/(StepPerPage*(Loop_Length[1]+1))) * 16)/10000;} //show Loop Position on a LED-RING 

            
            if( ( MelOmute[2] == 1) || (REC[2] == 1) ) {
                   IND[2] = ((10000*loop_step_pos[2]/(StepPerPage*(Loop_Length[2]+1))) * 16)/10000;} //show Loop Position on a LED-RING 

            
            if( ( MelOmute[3] == 1) || (REC[3] == 1) ) {
                   IND[3] = ((10000*loop_step_pos[3]/(StepPerPage*(Loop_Length[3]+1))) * 16)/10000;} //show Loop Position on a LED-RING 

            
            if( ( MelOmute[4] == 1) || (REC[4] == 1) ) {
                   IND[4] = ((10000*loop_step_pos[4]/(StepPerPage*(Loop_Length[4]+1))) * 16)/10000;} //show Loop Position on a LED-RING 
            
            if( ( MelOmute[5] == 1) || (REC[5] == 1) ) {
                   IND[5] = ((10000*loop_step_pos[5]/(StepPerPage*(Loop_Length[5]+1))) * 16)/10000;} //show Loop Position on a LED-RING 

            
            if( ( MelOmute[6] == 1) || (REC[6] == 1) ) {
                   IND[6] = ((10000*loop_step_pos[6]/(StepPerPage*(Loop_Length[6]+1))) * 16)/10000;} //show Loop Position on a LED-RING 

            
            if( ( MelOmute[7] == 1) || (REC[7] == 1) ) {
                   IND[7] = ((10000*loop_step_pos[7]/(StepPerPage*(Loop_Length[7]+1))) * 16)/10000;} //show Loop Position on a LED-RING 

    
      StepSend = loop_step_pos[0];
           
    } //END "If Tic=0"...
     
//send 96th Tics to Triggermatrix to trigger Notes between 16th notes (human feel)
APP_NOTEPROCESS(0, 127, 0, StepSend, TicSend); //(u8 port, u8 note, u16 velocity, u8 step) Back to APP.c

  return 0; // no error
}

