//16th fixed MelodySequencer, =========================================================
//connect to a Triggermatrix with melody retrigger - to get melody-beatstructure=======
//============2017 by Michael Sigl aka "Phatline,GreatFullTekk,GuteSigl,Technomiliz..."
//non-commercial use only. All other rights reserved. Mios32 Based > www.ucapps.de=====
//thx @ TK 4 MIOS==============//
//=====================================================================================
#include <mios32.h>
#include "app.h"
#include <FreeRTOS.h>
#include <portmacro.h>
#include <task.h>
#include <queue.h>
#include <semphr.h>
#include "tasks.h"
#include "file.h"
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <seq_bpm.h>
#include <seq_midi_out.h>
#include "seq.h"
#include <app_lcd.h>


static void TASK_SEQ(void *pvParameters);
#define PRIORITY_TASK_SEQ		( tskIDLE_PRIORITY + 4 ) // higher priority than MIDI receive task!
static s32 NOTIFY_MIDI_Rx(mios32_midi_port_t port, u8 byte);
static s32 APP_MIDI_TimeOut(mios32_midi_port_t port);
static void TASK_Period1mS(void *pvParameters); // periodic task
#define PRIORITY_TASK_PERIOD1MS          ( tskIDLE_PRIORITY + 2 ) // lower than MIDI

xSemaphoreHandle xSDCardSemaphore; 	// take and give access to SD-Card
xSemaphoreHandle xLCDSemaphore;		// take and give access to LCD

static file_t midifile_fi; //used to read files from SD-Card

#define MUTEX_LCD_TAKE { while( xSemaphoreTakeRecursive(xLCDSemaphore, (portTickType)0) != pdTRUE ); }  //a Mutex reserve a Resoure (LCD or SD-Card) for a task, until it is given away...
#define MUTEX_LCD_GIVE { xSemaphoreGiveRecursive(xLCDSemaphore); }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//System Variables >>> 
////////////////////////////////////////////////
  u8 ClockPortIN= 32;    //Midiclock Input Port (Midiclock Slave)
  u8 MelPortOut= 33;     //MeloTrigger Midi-Port, [32=UART0=Midi-A, 33=UART1=Midi-B]...
  u8 MeloOutCh[16]   ={0,0,1,1,2,2, 3,3,4,4,5,5, 6,6,7,7};  //Midi-Output-Channel Set...0-5 Poly-Melody && 6-15 is not used.

  //ProgramChange //Send Receive  		
  u8 PCPortIN = 32; //Receives ProgramChange Messages on Port?
  u8 PCReceive = 1; //Activate Programchange Receive
  u8 PCChIN = 15;   //ProgramChange Input Channel

//Melody - Keyboard input
  u8 MelPortIn = 33;	//MidiPort - Melody-Line-Input
//MidiChannel is set above "MeloOutCh[0-5]"
  u8 PitchbendPWR[16] = {1,1,1,1,1,1, 0,0,0,0,0,0, 0,0,0,0}; //only Activate Channels you really need! otherwhise midioutput is messed up...



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Patch Variables >>> Saved on SDCards (511 Patches)
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  u8 CCEnc[4] = {64,64,64,64};
  
  u8 CC_SD[128] = {}; //Save all other Variables into this Array - for SD-Card...
  u8  RecNoteBuff[6][8]=         //NoteBuffer: Save Incoming 8xPoly-Notes into a temporar BUFFER...waiting to be recordet INTO Sequencer-Matrix  
    {{0,48,60,0,0,0,0,0},       //NoteBuffer MelodyRetrigger-Track 0
     {0,48,60,0,0,0,0,0},       //NoteBuffer MelodyRetrigger-Track 1
     {0,48,60,0,0,0,0,0},       //NoteBuffer MelodyRetrigger-Track 2
     {0,48,60,0,0,0,0,0},       //NoteBuffer MelodyRetrigger-Track 3
     {0,48,60,0,0,0,0,0},       //NoteBuffer MelodyRetrigger-Track 4
     {0,48,60,0,0,0,0,0}};      //NoteBuffer MelodyRetrigger-Track 5 
     
  u8 MelImute[6]  = {0,0,0,0,0,0};
  u8 MelIshift[6] = {2,2,2,2,2,2};
  u8 MelIroute[6][6] = {{1,1,1,1,1,1}, //Melody Input Channel Routing to the 6 sequences
                        {1,1,1,1,1,1},
                        {1,1,1,1,1,1},
                        {1,1,1,1,1,1},
                        {1,1,1,1,1,1},
                        {1,1,1,1,1,1}};
  u8 MelOmute[8] = {1,1,1,1,1,1,1,1}; //ACTIVATE Button
  u8 RandomActive[6] = {1,1,1,1,1,1}; // Random Gate Activation ... 1 mean: RandomGateEnc has effect
  u8 RandomGateEnc = 127; //0 Gate let nothing thru, 127 Let all thru, default 127 because no Enocder for this @ Moment
  u8 SetDecayTime[6] = {60,60,60,60,60,60}; //20ms... could be set with Encoder, but I dont need a variable, since the melody out will be used by the Triggermatrix...

   //Sequencer Stuff
   u8  Rythm = 4;       //4x4=16, 5x3=15 usw... to calculate maximal duration per matrix page
   u8  RythmRepeat = 4; //4x4=16 === to get to a maximal loop length

   u8  REC[8] = {};
   u8  PLAY[8]= {};

       //Loop Stuff
   u8  SyncFlag = 0;    //to indicate that PageFirst/Last has changed, or the DIRECTION, or Loop LengthBtn....
   u8  Page = 0;        // Step Page, in 4/4 Mode one Page has 16 Step     
   u16 StepPerPage = 16;//calculate from 0 on....so 0-15 are 16 Steps that are visible on the BLM with a 4/4Beat    

   u8  LoopSetCount = 0;   //only to count the number of pressed buttons (first button last button thing....)
   
   s16 MainDelay = 0; //the main TIC-Delay
   u16 loop_step_pos[8] = {1,1,1,1, 1,1,1,1}; // straight step position bevore swing

  //Tact-Related Sequencer Stuff @ 384 PPQN
  s8  BaseTicCount = 1; //count from 0-3 ---equals 8 12 18
  s16 BaseTic = 12;  // 128th    12    Dotted 128th     18   128th Triplet     8   //32x12=384 tics
  s16 Tic = 384;  //32x12=384 tics between 2 Notes -- not to be saved ... just to calculate
  s8  PpqnDivider = 4; //4=16th speed, 2:8th speed
  s8  PpqnDividerCount = 1; //only to calculate....
  u8  BPM = 100;

   int test = 0;
   u8 Sequence[8][256][8]={{{}}};     //8Tracks, 256Steps, 8NotePolyphony
   u8 TrackFocus = 0;               //Currently edited Track
   u8 StepFocus[6] = {0,0,0,0,0,0}; //Currently edited Step of Track
   u8 TrackClipboard[256][8]  = {{}}; //Copy actual Track into ClipboardBuffer
   u8 StepClipBoard[6][8] = {};   //Just a memory container to copy paste Sequencer Steps

   u8 RecMode = 0; //0:Realtime, 1:StepEnter
   u8 StepLCD = 0; //Which Step and its Notes is Displayed in LCD

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Coding only Variables >>>No Settings here<<<
////should not be saved in Patchfile on SD
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Retrigger
    u16 MeloDecayTIMER[16][128] ={{}};//MidiDecayCounter --- Note OFF Handling
    u16 Decay = 400; //only initial... later processed via CC
    u8  NoteCount[16]={};
    s8  MelOctShiftFactor[16]={};
    u8  NoteCalc[16]={};           //incoming note + MelOctShiftFactor[0]
    u8  MeloOutNote[16] ={};
    u16 pitchbend[16] = {8192};    // inital No Pitchbend!

//Random Trigger Kill
    int random1 = 0;                    //1st Kill Function (Operator only -used 4 coding)
    int random_seed_l = 0;              //depend on the CController and its range
    int random_seed_h = 127;            //depens on the CController and its range...
    int random_tmp = 64;				
    int random_nr = 127;                //depends on the Controller and its range...
    u8 MeloRandGate = 1;                //if that flag is 0 - no MeloTrigger will come thru....
    int TMR0L = 0;                      //TMR0L = Timer - Random -Roll Variable
    u8 velocity = 0;

//LEDRING
    u8 LED_RING_Common = 0;     //only to calculate
    u8 IND[15] = {};
    u8 Loop_Length[8] = {0,3,0,3, 0,3,0,3}; //0-15 = Value of LED of a ring
    u8 OFF = 0; //0 or 1 depends on Led-Ring-Type (common anode, common catode)
    u8 ON =  1; //0 or 1 depends on Led-Ring-Type (common anode, common catode)

//V-LEDs-Switches...
u8 VLED[2][4][16] = {{{}}};   //VLED[ActualPage][Page][LED] to write directly into register
u8 PAGE[2] = {3,0}; //I have 2 PAGE Radio-Switches...
u8 ActualPage = 0;


//MENUE Store-Load Variables
   u16 SongNrLoad = 0;				
   u16 SongNrStor = 0;
    u8 PCBank[17]={0,1,2,3,0,0,0,0,0,0,0,0,0,0,0,0,0};   //Program-Change-Bank, Bank16 is for Trigger Channel!
   char file_typeBank[4] = "MQ01";	//should always be 4, and not longer then 4 chars!!! //definie the Filetype which is later saved via "FILE_WriteBuffer" into the file header (use Text/Hex-Editor to proove)
   char filepathS[8];				//STORE-path - Number of Pathsymbols t/512.tm >>> 8! >> again: dont change the length, or you will have to change the code	
   char filepathL[8];				//LOAD -path - Number of Pathsymbols t/512.tm >>> 8! >> again: dont change the length, or you will have to change the code	
   u8  MenueLoad = 0;				//LOAD FILES
   u8  MenueStore =0 ;			//STORE FILES
   u8  PCLoad;                //if 0 then load the Programchange that is currently changed (Load Button)
   s32 statusSD;					//Status of the SDCard--connected? fat?
   s32 statusDir;					//Status "is" your direction "sdcard/t" there?
   u16 SDCardCount = 0;
   u8  i = 0;						//only to count some things in for loops...
   u8  MenueUpdateFlag = 1;         //Flag to force the MenueUpdateCounter to count a periode of time, after the time "update the menue"
   u8  MenueUpdateCount = 0;
   //"MENUE"-Function Variable - This is the main UI Function, i do it there because i get Midi-In and DIN-IN... and both remote the same Code... so i need a common  Function... let say "Router"
   u32 menu_cc = 0;
   s32 menue_value = 0;
   s32 menue_incrementer = 0;
   s16 LCD0Page = 0;
   u8 PCLoadFlag = 0;          //Activate Timer to Load ProgramChanges from external source after a time (2seconds)
   u8 PCLoadFlagCount = 0;     //and a timer for that

//Task Prioritys	
#define PRIORITY_TASK_Midi_Decay0	              ( tskIDLE_PRIORITY + 2 ) //3:Mios standart ---- 5highest, 1lowest Priority
#define PRIORITY_TASK_Midi_Decay1	              ( tskIDLE_PRIORITY + 2 ) //3:Mios standart
#define PRIORITY_TASK_Midi_Decay2	              ( tskIDLE_PRIORITY + 2 ) //3:Mios standart
#define PRIORITY_TASK_Midi_Decay3	              ( tskIDLE_PRIORITY + 2 ) //3:Mios standart
    
#define PRIORITY_StoreLoad                        ( tskIDLE_PRIORITY + 2 ) //3:Mios standart
#define PRIORITY_MELODY_BUFFER0 	              ( tskIDLE_PRIORITY + 2 ) //3:Mios standart
#define PRIORITY_MELODY_BUFFER1 	              ( tskIDLE_PRIORITY + 2 ) //3:Mios standart
#define PRIORITY_MELODY_BUFFER2 	              ( tskIDLE_PRIORITY + 2 ) //3:Mios standart
#define PRIORITY_MELODY_BUFFER3 	              ( tskIDLE_PRIORITY + 2 ) //3:Mios standart

#define PRIORITY_APP_NOTEPROCESS	              ( tskIDLE_PRIORITY + 3 ) //4:high

// local prototype of the task function
static void TASK_Midi_Decay0(void *pvParameters);
static void TASK_Midi_Decay1(void *pvParameters);
static void TASK_Midi_Decay2(void *pvParameters);
static void TASK_Midi_Decay3(void *pvParameters);

static void MENUE(s32 menue_cc, s32 menue_incrementer);

static void StoreLoad(u8 tick, u16 order);
static void MELODY_BUFFER0(u8 velocity,  u8 note);
static void MELODY_BUFFER1(u8 velocity,  u8 note);
static void MELODY_BUFFER2(u8 velocity,  u8 note);
static void MELODY_BUFFER3(u8 velocity,  u8 note);


void APP_Init(void){
/////////////////////////////////////////////////////////////////////////
//Init TriggerMatrix UI
Tic = 32 * BaseTic; //32x12=384

  MIOS32_BOARD_LED_Init(0xffffffff);   // initialize all LEDs
  MIOS32_MIDI_TimeOutCallback_Init(APP_MIDI_TimeOut);  // install timeout callback function
  xTaskCreate(TASK_Period1mS, (signed portCHAR *)"Period1mS", configMINIMAL_STACK_SIZE, NULL, 1, NULL);   // start tasks
  	xLCDSemaphore 		= xSemaphoreCreateRecursiveMutex();														// create Mutex for LCD access
	xSDCardSemaphore 	= xSemaphoreCreateRecursiveMutex();														// create semaphores
	FILE_Init(0);// initialize file functions

  xTaskCreate(TASK_Midi_Decay0,  (signed portCHAR *)"Midi_Decay0",     64, NULL, 2, NULL);   // start task
  xTaskCreate(TASK_Midi_Decay1,  (signed portCHAR *)"Midi_Decay1",     64, NULL, 2, NULL);   // start task
  xTaskCreate(TASK_Midi_Decay2,  (signed portCHAR *)"Midi_Decay2",     64, NULL, 2, NULL);   // start task
  xTaskCreate(TASK_Midi_Decay3,  (signed portCHAR *)"Midi_Decay3",     64, NULL, 2, NULL);   // start task
  
  //Get and Set Multiple LCDs
  MIOS32_LCD_DeviceSet(0);
  MIOS32_LCD_Init(0);
  MIOS32_LCD_DeviceSet(1);
  MIOS32_LCD_Init(0);
  MIOS32_LCD_DeviceSet(2);
  MIOS32_LCD_Init(0);

  // SEQUENCER TASKS
  MIOS32_BOARD_LED_Set(1, 0);
  SEQ_MIDI_OUT_Init(0);   // initialize MIDI handler
  SEQ_Init(0);   // initialize sequencer
  MIOS32_MIDI_DirectRxCallback_Init(NOTIFY_MIDI_Rx);
  xTaskCreate(TASK_SEQ, (signed portCHAR *)"SEQ", configMINIMAL_STACK_SIZE, NULL, PRIORITY_TASK_SEQ, NULL);   // install sequencer task 
}

void APP_Background(void){}

//MENUE - PAGES
static void MENUE(s32 menue_cc, s32 menue_incrementer){
////////////////////////////////////////////////////////////////
//                                                            //
//  M       E     EEEEEEE    N     N     U     U     EEEEEEE  //                                                                
//  M E   U E     E          N N   N     U     U     E        //                                                          
//  M   N   E     EEEE       N  N  N     U     U     EEEE     //                                                          
//  M       E     E          N   N N     U     U     E        //                                                          
//  M       E     EEEEEEE    N    NN     UUUUUUU     EEEEEEE  //                                                          
//                                                            //
//Lower-MENU PAGE: 0
        /*
        //HARDWARE-TEST-OUT
        MUTEX_LCD_TAKE
                MIOS32_LCD_DeviceSet  (1);          MIOS32_LCD_Clear();  
                MIOS32_LCD_PrintFormattedString("%d %d", menue_cc, menue_incrementer);
        MUTEX_LCD_GIVE;
        */
        
    if( ((PAGE[1] == 0)  &&  menue_cc < 17) || ((PAGE[1] == 0) && (menue_cc == 126)) ) { //Lower Pages, REC-PLAY-MODE
        if( (menue_cc == 0) && (menue_incrementer == 0) ) {  MelOmute[0] =! MelOmute[0];    if(MelOmute[0] > 1) {MelOmute[0] = 1;}}
        if( (menue_cc == 1) && (menue_incrementer == 0) ) {  MelOmute[1] =! MelOmute[1];    if(MelOmute[1] > 1) {MelOmute[1] = 1;}}
        if( (menue_cc == 2) && (menue_incrementer == 0) ) {  MelOmute[2] =! MelOmute[2];    if(MelOmute[2] > 1) {MelOmute[2] = 1;}}
        if( (menue_cc == 3) && (menue_incrementer == 0) ) {  MelOmute[3] =! MelOmute[3];    if(MelOmute[3] > 1) {MelOmute[3] = 1;}}
        if( (menue_cc == 4) && (menue_incrementer == 0) ) {  MelOmute[4] =! MelOmute[4];    if(MelOmute[4] > 1) {MelOmute[4] = 1;}}
        if( (menue_cc == 5) && (menue_incrementer == 0) ) {  MelOmute[5] =! MelOmute[5];    if(MelOmute[5] > 1) {MelOmute[5] = 1;}}
        if( (menue_cc == 6) && (menue_incrementer == 0) ) {  MelOmute[6] =! MelOmute[6];    if(MelOmute[6] > 1) {MelOmute[6] = 1;}}
        if( (menue_cc == 7) && (menue_incrementer == 0) ) {  MelOmute[7] =! MelOmute[7];    if(MelOmute[7] > 1) {MelOmute[7] = 1;}}
        if( (menue_cc == 8) && (menue_incrementer == 0) ) {  REC[0]  =! REC[0];     if(REC[0] > 1)  {REC[0]  = 1;}}
        if( (menue_cc == 9) && (menue_incrementer == 0) ) {  REC[1]  =! REC[1];     if(REC[1] > 1)  {REC[1]  = 1;}}
        if( (menue_cc ==10) && (menue_incrementer == 0) ) {  REC[2]  =! REC[2];     if(REC[2] > 1)  {REC[2]  = 1;}}
        if( (menue_cc ==11) && (menue_incrementer == 0) ) {  REC[3]  =! REC[3];     if(REC[3] > 1)  {REC[3]  = 1;}}
        if( (menue_cc ==12) && (menue_incrementer == 0) ) {  REC[4]  =! REC[4];     if(REC[4] > 1)  {REC[4]  = 1;}}
        if( (menue_cc ==13) && (menue_incrementer == 0) ) {  REC[5]  =! REC[5];     if(REC[5] > 1)  {REC[5]  = 1;}}
        if( (menue_cc ==14) && (menue_incrementer == 0) ) {  REC[6]  =! REC[6];     if(REC[6] > 1)  {REC[6]  = 1;}}
        if( (menue_cc ==15) && (menue_incrementer == 0) ) {  REC[7]  =! REC[7];     if(REC[7] > 1)  {REC[7]  = 1;}}     
      
  //LCD-LABELING 
        MUTEX_LCD_TAKE; //request LCD Access    
                //LCD 0
                MIOS32_LCD_DeviceSet  (0);          MIOS32_LCD_Clear();  
                    MIOS32_LCD_CursorSet(0, 0);     MIOS32_LCD_PrintFormattedString("PLAYa            PLAYb             PLAYa");
                    MIOS32_LCD_CursorSet(0, 1);     MIOS32_LCD_PrintFormattedString("REC-a            REC-b             REC-a");
                //LCD 1
                MIOS32_LCD_DeviceSet  (1);          MIOS32_LCD_Clear();  
                    MIOS32_LCD_CursorSet(0, 0);     MIOS32_LCD_PrintFormattedString("PLAYb            PLAYa             PLAYb"); 
                    MIOS32_LCD_CursorSet(0, 1);     MIOS32_LCD_PrintFormattedString("REC-b            REC-a             REC-b"); 
                //LCD 2
                MIOS32_LCD_DeviceSet  (2);          MIOS32_LCD_Clear();  
                    MIOS32_LCD_CursorSet(0, 0);     MIOS32_LCD_PrintFormattedString("PLAYa            PLAYb         PLAY-MODE");
                    MIOS32_LCD_CursorSet(0, 1);     MIOS32_LCD_PrintFormattedString("REC-a            REC-b          REC-MODE");
        MUTEX_LCD_GIVE;					// release LCD access for other tasks  	
    }
           // MIOS32_LCD_CursorSet(35, 0);MIOS32_LCD_PrintFormattedString("%d %d", Sequence[0] [StepLCD] [7], LCD0Page+1);        

/*
  //LCD-LABELING
  if(pin == 1)   {//Page0-Btn pressed?
      //CLEAR ALL SCREENS
	  MIOS32_LCD_DeviceSet  (0);   MIOS32_LCD_Clear();  
	  MIOS32_LCD_DeviceSet  (1);   MIOS32_LCD_Clear();  
	  MIOS32_LCD_DeviceSet  (2);   MIOS32_LCD_Clear();  
      //update  Display
	  if (PAGE[ActualPage] == 0) {    
         //1st Row
         MIOS32_LCD_DeviceSet  (0);   MIOS32_LCD_CursorSet(0, 0);   MIOS32_LCD_PrintFormattedString("-----            -----             -----");
         MIOS32_LCD_DeviceSet  (1);   MIOS32_LCD_CursorSet(0, 0);   MIOS32_LCD_PrintFormattedString("-----            -----             -----"); 
         MIOS32_LCD_DeviceSet  (2);   MIOS32_LCD_CursorSet(0, 0);   MIOS32_LCD_PrintFormattedString("-----            -----           TM-BANK");
         //2nd Row
         MIOS32_LCD_DeviceSet  (0);   MIOS32_LCD_CursorSet(0, 1);   MIOS32_LCD_PrintFormattedString("-----            -----             -----");
         MIOS32_LCD_DeviceSet  (1);   MIOS32_LCD_CursorSet(0, 1);   MIOS32_LCD_PrintFormattedString("-----            -----             -----"); 
         MIOS32_LCD_DeviceSet  (2);   MIOS32_LCD_CursorSet(0, 1);   MIOS32_LCD_PrintFormattedString("-----            -----           SQ-BANK");}
}
*/

/*
  //Take 2x8 Buttons and Make more Virtual Buttons && SEND them to Triggermatrix
  if(ActualPage == 0  && pin >= 8) { //1st Page-Btn
     pin = pin - 8 + (PAGE[ActualPage]*16);} //Offset Pin by PAGE

  if(ActualPage == 1 && pin >= 8) {
     pin = pin - 8 + (PAGE[ActualPage]*16);}
*/

/*
MIOS32_DOUT_PinSet(8, Rec[0]);
MIOS32_DOUT_PinSet(23,Rec[1]);
MIOS32_DOUT_PinSet(22,Rec[2]);
MIOS32_DOUT_PinSet(21,Rec[3]);

if(direction == 0) {MIOS32_DOUT_PinSet(13,1); 
                    MIOS32_DOUT_PinSet(12,0);
                    MIOS32_DOUT_PinSet(11,0);}
if(direction == 1) {MIOS32_DOUT_PinSet(13,0); 
                    MIOS32_DOUT_PinSet(12,1);
                    MIOS32_DOUT_PinSet(11,0);}
if(direction == 2) {MIOS32_DOUT_PinSet(13,0); 
                    MIOS32_DOUT_PinSet(12,0);
                    MIOS32_DOUT_PinSet(11,1);}

MIOS32_DOUT_PinSet(6, RecMode);
*/

/*
		

//MENU PAGE 1////////////////////////////////////////////////////////////////////////////////////
  if( (LCD0Page == 1  &&  menue_cc < 9 && RecMode == 1) || (LCD0Page == 1 && menue_cc == 126)) { //Show-Edit Sequence 1 @ Current Step
			if(menue_cc == 0){
				menue_value = Sequence[1][StepLCD][0] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[1][StepLCD][0] = menue_value;}  
 			if(menue_cc == 1){
				menue_value = Sequence[1][StepLCD][1] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[1][StepLCD][1] = menue_value;}  
 			if(menue_cc == 2){
				menue_value = Sequence[1][StepLCD][2] + menue_incrementer;    
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[1][StepLCD][2] = menue_value;}  
 			if(menue_cc == 3){
				menue_value = Sequence[1][StepLCD][3] + menue_incrementer;    
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[1][StepLCD][3] = menue_value;}  
 			if(menue_cc == 4){
				menue_value = Sequence[1][StepLCD][4] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[1][StepLCD][4] = menue_value;}   
 			if(menue_cc == 5){
				menue_value = Sequence[1][StepLCD][5] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[1][StepLCD][5] = menue_value;}  
 			if(menue_cc == 6){
				menue_value = Sequence[1][StepLCD][6] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[1][StepLCD][6] = menue_value;}    
 			if(menue_cc == 7){
				menue_value = Sequence[1][StepLCD][7] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[1][StepLCD][7] = menue_value;}  

            if(menue_cc == 8){ // 0:Setting-Menue, 1:Edit Note Date of Step
                menue_value = StepLCD + menue_incrementer;    // increment to virtual position and ensure that the menue_value is in range 0..127
                if(menue_value < 0) {menue_value = 255;}        // 
                if(menue_value > 255) {menue_value = 0;}        // 
                StepLCD = menue_value;
                StepFocus[TrackFocus] = StepLCD;
                MENUE(126, 0);}
//LCD
         MUTEX_LCD_TAKE;        //request LCD aCCes0s
            MIOS32_LCD_DeviceSet  (0); //=1st 2x40 LCD-screen
            MIOS32_LCD_Clear(); //clear screen 
          //1st Line = Encoder Variables....change something!
            MIOS32_LCD_CursorSet(0,  0);MIOS32_LCD_PrintFormattedString("%d", Sequence[1] [StepLCD] [0]);
            MIOS32_LCD_CursorSet(5,  0);MIOS32_LCD_PrintFormattedString("%d", Sequence[1] [StepLCD] [1]);			
	        MIOS32_LCD_CursorSet(10, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[1] [StepLCD] [2]);
            MIOS32_LCD_CursorSet(15, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[1] [StepLCD] [3]);			
			MIOS32_LCD_CursorSet(20, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[1] [StepLCD] [4]);
            MIOS32_LCD_CursorSet(25, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[1] [StepLCD] [5]);			
			MIOS32_LCD_CursorSet(30, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[1] [StepLCD] [6]);
            MIOS32_LCD_CursorSet(35, 0);MIOS32_LCD_PrintFormattedString("%d %d", Sequence[1] [StepLCD] [7], LCD0Page+1);

          //2nd Line = Menue Describtion
            MIOS32_LCD_CursorSet(0,  1);MIOS32_LCD_PrintFormattedString("%s", "1");
            MIOS32_LCD_CursorSet(6,  1);MIOS32_LCD_PrintFormattedString("%s", "2");
            MIOS32_LCD_CursorSet(11, 1);MIOS32_LCD_PrintFormattedString("%s", "3");
            MIOS32_LCD_CursorSet(16, 1);MIOS32_LCD_PrintFormattedString("%s", "4");
            MIOS32_LCD_CursorSet(21, 1);MIOS32_LCD_PrintFormattedString("%s", "5");
            MIOS32_LCD_CursorSet(26, 1);MIOS32_LCD_PrintFormattedString("%s", "6");
            MIOS32_LCD_CursorSet(31, 1);MIOS32_LCD_PrintFormattedString("%s", "7");
            MIOS32_LCD_CursorSet(35, 1);MIOS32_LCD_PrintFormattedString("%s %d", "8",StepLCD);
          MUTEX_LCD_GIVE;}					// release LCD access for other tasks  			

//MENU PAGE 2////////////////////////////////////////////////////////////////////////////////////
  if( (LCD0Page == 2  &&  menue_cc < 9 && RecMode == 1) || (LCD0Page == 2 && menue_cc == 126)) { //Show-Edit Sequence 2 @ Current Step
			if(menue_cc == 0){
				menue_value = Sequence[2][StepLCD][0] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[2][StepLCD][0] = menue_value;}  
 			if(menue_cc == 1){
				menue_value = Sequence[2][StepLCD][1] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[2][StepLCD][1] = menue_value;}  
 			if(menue_cc == 2){
				menue_value = Sequence[2][StepLCD][2] + menue_incrementer;    
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[2][StepLCD][2] = menue_value;}  
 			if(menue_cc == 3){
				menue_value = Sequence[2][StepLCD][3] + menue_incrementer;    
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[2][StepLCD][3] = menue_value;}  
 			if(menue_cc == 4){
				menue_value = Sequence[2][StepLCD][4] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[2][StepLCD][4] = menue_value;}   
 			if(menue_cc == 5){
				menue_value = Sequence[2][StepLCD][5] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[2][StepLCD][5] = menue_value;}  
 			if(menue_cc == 6){
				menue_value = Sequence[2][StepLCD][6] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[2][StepLCD][6] = menue_value;}    
 			if(menue_cc == 7){
				menue_value = Sequence[2][StepLCD][7] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[2][StepLCD][7] = menue_value;}  

            if(menue_cc == 8){ // 0:Setting-Menue, 1:Edit Note Date of Step
                menue_value = StepLCD + menue_incrementer;    // increment to virtual position and ensure that the menue_value is in range 0..127
                if(menue_value < 0) {menue_value = 255;}        // 
                if(menue_value > 255) {menue_value = 0;}        // 
                StepLCD = menue_value;
                StepFocus[TrackFocus] = StepLCD;
                MENUE(126, 0);}
//LCD
         MUTEX_LCD_TAKE;        //request LCD aCCes0s
            MIOS32_LCD_DeviceSet  (0); //=1st 2x40 LCD-screen
            MIOS32_LCD_Clear(); //clear screen 
          //1st Line = Encoder Variables....change something!
            MIOS32_LCD_CursorSet(0,  0);MIOS32_LCD_PrintFormattedString("%d", Sequence[2] [StepLCD] [0]);
            MIOS32_LCD_CursorSet(5,  0);MIOS32_LCD_PrintFormattedString("%d", Sequence[2] [StepLCD] [1]);			
	        MIOS32_LCD_CursorSet(10, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[2] [StepLCD] [2]);
            MIOS32_LCD_CursorSet(15, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[2] [StepLCD] [3]);			
			MIOS32_LCD_CursorSet(20, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[2] [StepLCD] [4]);
            MIOS32_LCD_CursorSet(25, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[2] [StepLCD] [5]);			
			MIOS32_LCD_CursorSet(30, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[2] [StepLCD] [6]);
            MIOS32_LCD_CursorSet(35, 0);MIOS32_LCD_PrintFormattedString("%d %d", Sequence[2] [StepLCD] [7], LCD0Page+1);

          //2nd Line = Menue Describtion
            MIOS32_LCD_CursorSet(0,  1);MIOS32_LCD_PrintFormattedString("%s", "1");
            MIOS32_LCD_CursorSet(6,  1);MIOS32_LCD_PrintFormattedString("%s", "2");
            MIOS32_LCD_CursorSet(11, 1);MIOS32_LCD_PrintFormattedString("%s", "3");
            MIOS32_LCD_CursorSet(16, 1);MIOS32_LCD_PrintFormattedString("%s", "4");
            MIOS32_LCD_CursorSet(21, 1);MIOS32_LCD_PrintFormattedString("%s", "5");
            MIOS32_LCD_CursorSet(26, 1);MIOS32_LCD_PrintFormattedString("%s", "6");
            MIOS32_LCD_CursorSet(31, 1);MIOS32_LCD_PrintFormattedString("%s", "7");
            MIOS32_LCD_CursorSet(35, 1);MIOS32_LCD_PrintFormattedString("%s %d", "8",StepLCD);
          MUTEX_LCD_GIVE;}					// release LCD access for other tasks  			

//MENU PAGE 3////////////////////////////////////////////////////////////////////////////////////
  if( (LCD0Page == 3  &&  menue_cc < 9 && RecMode == 1) || (LCD0Page == 3 && menue_cc == 126)) { //Show-Edit Sequence 3 @ Current Step
			if(menue_cc == 0){
				menue_value = Sequence[3][StepLCD][0] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[3][StepLCD][0] = menue_value;}  
 			if(menue_cc == 1){
				menue_value = Sequence[3][StepLCD][1] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[3][StepLCD][1] = menue_value;}  
 			if(menue_cc == 2){
				menue_value = Sequence[3][StepLCD][2] + menue_incrementer;    
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[3][StepLCD][2] = menue_value;}  
 			if(menue_cc == 3){
				menue_value = Sequence[3][StepLCD][3] + menue_incrementer;    
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[3][StepLCD][3] = menue_value;}  
 			if(menue_cc == 4){
				menue_value = Sequence[3][StepLCD][4] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[3][StepLCD][4] = menue_value;}   
 			if(menue_cc == 5){
				menue_value = Sequence[3][StepLCD][5] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[3][StepLCD][5] = menue_value;}  
 			if(menue_cc == 6){
				menue_value = Sequence[3][StepLCD][6] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[3][StepLCD][6] = menue_value;}    
 			if(menue_cc == 7){
				menue_value = Sequence[3][StepLCD][7] + menue_incrementer;   
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				Sequence[3][StepLCD][7] = menue_value;}  

            if(menue_cc == 8){ // 0:Setting-Menue, 1:Edit Note Date of Step
                menue_value = StepLCD + menue_incrementer;    // increment to virtual position and ensure that the menue_value is in range 0..127
                if(menue_value < 0) {menue_value = 255;}        // 
                if(menue_value > 255) {menue_value = 0;}        // 
                StepLCD = menue_value;
                StepFocus[TrackFocus] = StepLCD;
                MENUE(126, 0);}
//LCD
         MUTEX_LCD_TAKE;        //request LCD aCCes0s
            MIOS32_LCD_DeviceSet  (0); //=1st 2x40 LCD-screen
            MIOS32_LCD_Clear(); //clear screen 
          //1st Line = Encoder Variables....change something!
            MIOS32_LCD_CursorSet(0,  0);MIOS32_LCD_PrintFormattedString("%d", Sequence[3] [StepLCD] [0]);
            MIOS32_LCD_CursorSet(5,  0);MIOS32_LCD_PrintFormattedString("%d", Sequence[3] [StepLCD] [1]);			
	        MIOS32_LCD_CursorSet(10, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[3] [StepLCD] [2]);
            MIOS32_LCD_CursorSet(15, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[3] [StepLCD] [3]);			
			MIOS32_LCD_CursorSet(20, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[3] [StepLCD] [4]);
            MIOS32_LCD_CursorSet(25, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[3] [StepLCD] [5]);			
			MIOS32_LCD_CursorSet(30, 0);MIOS32_LCD_PrintFormattedString("%d", Sequence[3] [StepLCD] [6]);
            MIOS32_LCD_CursorSet(35, 0);MIOS32_LCD_PrintFormattedString("%d %d", Sequence[3] [StepLCD] [7], LCD0Page+1);

          //2nd Line = Menue Describtion
            MIOS32_LCD_CursorSet(0,  1);MIOS32_LCD_PrintFormattedString("%s", "1");
            MIOS32_LCD_CursorSet(6,  1);MIOS32_LCD_PrintFormattedString("%s", "2");
            MIOS32_LCD_CursorSet(11, 1);MIOS32_LCD_PrintFormattedString("%s", "3");
            MIOS32_LCD_CursorSet(16, 1);MIOS32_LCD_PrintFormattedString("%s", "4");
            MIOS32_LCD_CursorSet(21, 1);MIOS32_LCD_PrintFormattedString("%s", "5");
            MIOS32_LCD_CursorSet(26, 1);MIOS32_LCD_PrintFormattedString("%s", "6");
            MIOS32_LCD_CursorSet(31, 1);MIOS32_LCD_PrintFormattedString("%s", "7");
            MIOS32_LCD_CursorSet(35, 1);MIOS32_LCD_PrintFormattedString("%s %d", "8",StepLCD);
          MUTEX_LCD_GIVE;}					// release LCD access for other tasks  			

//MENU PAGE 4/////////////////////////////////////////////////////////////////////////////////////
  if( (LCD0Page == 4  &&  menue_cc < 9) || (LCD0Page == 4 && menue_cc == 126)) { //Timing Related (BPM, StepPerPage, Divider...)
			if(menue_cc == 0){{ //RYTHM + CALC Step Per Page
				menue_value = Rythm + menue_incrementer;   
				if(menue_value  < 2)  {menue_value = 2; StepPerPage = 16;}
				if(menue_value == 3)  {                 StepPerPage = 12;}
				if(menue_value == 4)  {                 StepPerPage = 16;}
				if(menue_value == 5)  {                 StepPerPage = 10;}
				if(menue_value == 6)  {                 StepPerPage = 12;}
				if(menue_value == 7)  {                 StepPerPage = 14;}
				if(menue_value == 8)  {                 StepPerPage = 16;}
				if(menue_value == 9)  {                 StepPerPage =  9;}
				if(menue_value == 10) {                 StepPerPage = 10;}
				if(menue_value == 11) {                 StepPerPage = 11;}
				if(menue_value == 12) {                 StepPerPage = 12;}
				if(menue_value == 13) {                 StepPerPage = 13;}
				if(menue_value == 14) {                 StepPerPage = 14;}
				if(menue_value >= 15) {menue_value = 15;StepPerPage = 15;}
				Rythm = menue_value;}}
 
			if(menue_cc == 2){ //Rythm repeats...5x5? or 5x50?
				menue_value = RythmRepeat + menue_incrementer;  
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				RythmRepeat = menue_value;}

                SyncFlag = 1;    //to give the order to change the loop...after some time it will change.

             //BPM 
			 if(menue_cc == 4){
				menue_value = BPM + menue_incrementer;  
				if(menue_value < 20){menue_value = 20;}
				if(menue_value > 255){menue_value = 255;}
				BPM = menue_value;
                SEQ_BPM_Set(BPM);}

             //BPM Divider
			 if(menue_cc == 5){
				menue_value = PpqnDividerCount + menue_incrementer;  
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 2){menue_value = 2;}
				PpqnDividerCount = menue_value;
             if(PpqnDividerCount == 0){PpqnDivider = 2;}
             if(PpqnDividerCount == 1){PpqnDivider = 4;}
             if(PpqnDividerCount == 2){PpqnDivider = 8;}
             SyncFlag = 5;} //update ClockCounter when ready...

            //Calculate TICs
			 if(menue_cc == 6){
				menue_value = BaseTicCount + menue_incrementer;  
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 2){menue_value = 2;}
				BaseTicCount = menue_value;
            if(BaseTicCount == 0){BaseTic =  8;}
       else if(BaseTicCount == 1){BaseTic = 12;}
       else if(BaseTicCount == 2){BaseTic = 18;}
       Tic = 32 * BaseTic; //32x12=384
       SyncFlag = 4;} //update ClockCounter when ready...

//2.Menu - PC CH9-16
         MUTEX_LCD_TAKE;        //request LCD aCCess
            MIOS32_LCD_DeviceSet  (0); //=1st 2x40 LCD-screen
            MIOS32_LCD_Clear(); //clear screen 
          //1st Line = Encoder Variables....change something!
            MIOS32_LCD_CursorSet(0,  0);MIOS32_LCD_PrintFormattedString("%d", Rythm);
            MIOS32_LCD_CursorSet(5,  0);MIOS32_LCD_PrintFormattedString("%d", StepPerPage);				
	        MIOS32_LCD_CursorSet(10, 0);MIOS32_LCD_PrintFormattedString("%d", RythmRepeat);
            MIOS32_LCD_CursorSet(15, 0);MIOS32_LCD_PrintFormattedString("%d", MainLoop);				
            MIOS32_LCD_CursorSet(21, 0);MIOS32_LCD_PrintFormattedString("%d", BPM);	
            MIOS32_LCD_CursorSet(28, 0);MIOS32_LCD_PrintFormattedString("%d", PpqnDivider);
	        MIOS32_LCD_CursorSet(30, 0);MIOS32_LCD_PrintFormattedString("%s", ">");		
			MIOS32_LCD_CursorSet(33, 0);MIOS32_LCD_PrintFormattedString("%d", BaseTic);
	        MIOS32_LCD_CursorSet(36, 0);MIOS32_LCD_PrintFormattedString("%s", "=");	
            MIOS32_LCD_CursorSet(37, 0);MIOS32_LCD_PrintFormattedString("%d", Tic);

          //2nd Line = Menue Describtion
            MIOS32_LCD_CursorSet(0,  1);MIOS32_LCD_PrintFormattedString("%s", "Beat");
            MIOS32_LCD_CursorSet(5,  1);MIOS32_LCD_PrintFormattedString("%s", "Step");
            MIOS32_LCD_CursorSet(10, 1);MIOS32_LCD_PrintFormattedString("%s", "Rept");
            MIOS32_LCD_CursorSet(15, 1);MIOS32_LCD_PrintFormattedString("%s", "Loop");
            MIOS32_LCD_CursorSet(21, 1);MIOS32_LCD_PrintFormattedString("%s", "BPM");
            MIOS32_LCD_CursorSet(26, 1);MIOS32_LCD_PrintFormattedString("%s", "Tempo");
            MIOS32_LCD_CursorSet(32, 1);MIOS32_LCD_PrintFormattedString("%s", "Base");
            MIOS32_LCD_CursorSet(37, 1);MIOS32_LCD_PrintFormattedString("%s", "Tic");
          MUTEX_LCD_GIVE;					// release LCD access for other tasks  			
}      

//MENU PAGE 5/////////////////////////////////////////////////////////////////////////////////////	  	      
  if( (LCD0Page == 5  &&  menue_cc < 9) || (LCD0Page == 5 && menue_cc == 126)) { //LOAD - STORE RENAME Bank
			if(menue_cc == 0){ //Load-NR
				menue_value = SongNrLoad + menue_incrementer;    
				if(menue_value < 0){menue_value = 511;}
				if(menue_value > 511){menue_value = 0;}
				SongNrLoad = menue_value;
				//ReadName From File	
				MUTEX_SDCARD_GIVE;
		          sprintf(filepathL, "mq/%d.mq", SongNrLoad);
                  FILE_ReadOpen	(&midifile_fi, filepathL);
                    FILE_ReadBuffer	((u8 *)file_typeBank,	4); 	 
                  FILE_ReadClose	(&midifile_fi);
                MUTEX_SDCARD_GIVE;}
			if(menue_cc == 1){ //Store-Nr
				menue_value = SongNrStor + menue_incrementer;    
				if(menue_value < 0){menue_value = 511;}
				if(menue_value > 511){menue_value = 0;}
				SongNrStor= menue_value;}
			if(menue_cc == 2){ //Start-Page (looplength)
				    menue_value = PageFirst + menue_incrementer;    
				    if(menue_value < 0){menue_value = 0;}
				    if(menue_value > 8){menue_value = 8;}
				    PageFirst= menue_value;
                    SEQ_Loop();//Send update CALL to seq.c > s32 SEQ_Loop - go make new loop length, and just do it...
                    SyncFlag = 1;}   //see "seq.c." Call BPM counter that something has to be change
                          
			if(menue_cc == 3){ //End-Page (looplength)
			    	menue_value = PageLast + menue_incrementer;    
				    if(menue_value < 0){menue_value = 0;}
				    if(menue_value > 8){menue_value = 8;}
                    PageLast = menue_value;
                    SEQ_Loop();//Send update CALL to seq.c > s32 SEQ_Loop - go make new loop length, and just do it...
                    SyncFlag = 1;}   //see "seq.c." Call BPM counter that something has to be change


//0.Menu - STORE+LOAD+RENAME
         MUTEX_LCD_TAKE;        //request LCD aCCess
            MIOS32_LCD_DeviceSet  (0); //=1st 2x40 LCD-screen 
            MIOS32_LCD_Clear(); //clear screen
          //1st Line = Encoder Variables....change something!
            MIOS32_LCD_CursorSet(0,  0);  MIOS32_LCD_PrintFormattedString("%d", SongNrLoad);
            MIOS32_LCD_CursorSet(5,  0);  MIOS32_LCD_PrintFormattedString("%d", SongNrStor);
            MIOS32_LCD_CursorSet(10, 0); MIOS32_LCD_PrintFormattedString("%d", PageFirst);
            MIOS32_LCD_CursorSet(15, 0); MIOS32_LCD_PrintFormattedString("%d", PageLast);           		
          //2nd Line = Menue Describtion
            MIOS32_LCD_CursorSet(0,  1);  MIOS32_LCD_PrintFormattedString("%s", "Load");
            MIOS32_LCD_CursorSet(5,  1);  MIOS32_LCD_PrintFormattedString("%s", "Save");
            MIOS32_LCD_CursorSet(10, 1);  MIOS32_LCD_PrintFormattedString("%s", "Strt");
            MIOS32_LCD_CursorSet(15, 1);  MIOS32_LCD_PrintFormattedString("%s", "End");
          MUTEX_LCD_GIVE;					// release LCD aCCess for other tasks   
}

//MENU PAGE 6/////////////////////////////////////////////////////////////////////////////////////	  	      
  if( (menue_cc >= 9  &&  menue_cc <=13) || (LCD0Page == 6 && menue_cc == 126)) { //Additonal CC-Encoder-Functions...

			if(menue_cc ==  9){
				menue_value = RandomGateEnc + menue_incrementer;    
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				RandomGateEnc = menue_value; LCD0Page=6;}
                
			if(menue_cc == 10){ 
				menue_value = SetDecayTime[0] + menue_incrementer;    
				if(menue_value < 10){menue_value = 10;}
				if(menue_value > 127){menue_value = 127;}
				SetDecayTime[0] = menue_value;
                SetDecayTime[1] = menue_value;                 
                SetDecayTime[2] = menue_value;                 
                SetDecayTime[3] = menue_value;; LCD0Page=6;}                 
                
			if(menue_cc == 11){ 
				menue_value = MainDelay + menue_incrementer;    
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 95){menue_value = 95;}
				MainDelay = menue_value; LCD0Page=6;}
                
			if(menue_cc == 12){ 
				menue_value = CCEnc[3] + menue_incrementer;    
				if(menue_value < 0){menue_value = 0;}
				if(menue_value > 127){menue_value = 127;}
				CCEnc[3] = menue_value; LCD0Page=6;}

//0.Menu - STORE+LOAD+RENAME
         MUTEX_LCD_TAKE;        //request LCD aCCess
            MIOS32_LCD_DeviceSet  (0); //=1st 2x40 LCD-screen 
            MIOS32_LCD_Clear(); //clear screen
          //1st Line = Encoder Variables....change something!
            MIOS32_LCD_CursorSet(0,  0);  MIOS32_LCD_PrintFormattedString("%d", RandomGateEnc);
            MIOS32_LCD_CursorSet(12, 0);  MIOS32_LCD_PrintFormattedString("%d", SetDecayTime[0]);
            MIOS32_LCD_CursorSet(24, 0);  MIOS32_LCD_PrintFormattedString("%d", MainDelay);
            MIOS32_LCD_CursorSet(36, 0);  MIOS32_LCD_PrintFormattedString("%d", CCEnc[3]);           		
          //2nd Line = Menue Describtion
            MIOS32_LCD_CursorSet(0,  1);  MIOS32_LCD_PrintFormattedString("%s", "Kill");
            MIOS32_LCD_CursorSet(11, 1);  MIOS32_LCD_PrintFormattedString("%s", "Decay");
            MIOS32_LCD_CursorSet(23, 1);  MIOS32_LCD_PrintFormattedString("%s", "Delay");
            MIOS32_LCD_CursorSet(36, 1);  MIOS32_LCD_PrintFormattedString("%s", "ENC4");
          MUTEX_LCD_GIVE;					// release LCD aCCess for other tasks   
}


*/
}//END TASK///////////////////////////////////////////////////////////////////////////////////////////////////

/*
static void Router(u32 port, u32 channel, u32 control, s32 value) { //DIN, ENCs, MidiCC all together, More Displays

MUTEX_LCD_TAKE;        //request LCD access
MIOS32_LCD_DeviceSet  (0); //=1st 2x40 LCD-screen 
MIOS32_LCD_Clear();    //clear screen
MIOS32_LCD_CursorSet(0, 0);  MIOS32_LCD_PrintFormattedString("%d %d %d %d", port, channel, control, value);
MUTEX_LCD_GIVE;        //request LCD access



if(port == 0)  {// >>DIN<<
  if(channel == 0 && value == 0) {//Sequencer Stuff
   if(control == 0)  {SEQ_BPM_Stop(); SEQ_Init(0);}
   if(control == 1)  {SEQ_BPM_Start();}
   if(control == 16) {Rec[0] =! Rec[0];} //Record Track0
   if(control == 17) {Rec[1] =! Rec[1];} //Record Track0
   if(control == 18) {Rec[2] =! Rec[2];} //Record Track0   
   if(control == 19) {Rec[3] =! Rec[3];} //Record Track0  
   
   if(control == 12){direction = 0; SyncFlag = 1;}   //Play Direction & Update FLAG see "seq.c." Call BPM counter that something has to be change
   if(control == 13){direction = 1; SyncFlag = 1;}   //Play Direction & Update FLAG see "seq.c." Call BPM counter that something has to be change
   if(control == 14){direction = 2; SyncFlag = 1;}   //Play Direction & Update FLAG see "seq.c." Call BPM counter that something has to be change

   if(control == 10){SyncType =! SyncType;}
   if(control == 2) {RecMode =! RecMode;}  


//MENUE-PAGES-SELECT/////////////////////////////////////////
//                   PageNumber    Dump-Out-2-LCD   Focus Track1-4 - 2 Edit NoteData
   if(control == 3) {LCD0Page = 0; MENUE(126, 0);   TrackFocus = 0;} //Focus Track for Deep Edit  -  edit 8 poly-NoteNumbers @ 2x40SignsDisplays
   if(control == 4) {LCD0Page = 1; MENUE(126, 0);   TrackFocus = 1;} //Focus Track for Deep Edit     
   if(control == 5) {LCD0Page = 2; MENUE(126, 0);   TrackFocus = 2;} //Focus Track for Deep Edit     
   if(control == 6) {LCD0Page = 3; MENUE(126, 0);   TrackFocus = 3;} //Focus Track for Deep Edit  
   if(control == 7) {LCD0Page = 4; MENUE(126, 0);} //Save,Load,LoopLength...and Perform Controlls 4 Live- while loading Sets...         
   if(control == 8) {LCD0Page = 5; MENUE(126, 0);} //All about Timing:BPM,Tics,3/4,5/4,6/4,8/4...   
   if(control == 9) {LCD0Page = 6; MENUE(126, 0);} //not used@the moment of writing this lines...         
  
//COPY PASTE DELETE of Melody Sequencer Data//////////////////
//
  if(control == 20){ //COPY - Step into Memory
       int z; for(z=0; z<8; z++) {         //6 Tracks, XSteps,                8 PolyNotes
       StepClipBoard[TrackFocus][z]  =  Sequence[TrackFocus][StepFocus[TrackFocus]][z];}} //copy steps of actual page  into clipboard memory

  if(control == 21){ //PASTE - Memory into Step  
       int z; for(z=0; z<8; z++) {
        Sequence[TrackFocus][StepFocus[TrackFocus]][z] = StepClipBoard[TrackFocus][z];} //copy steps of actual page  into clipboard memory
        if((LCD0Page >= 0) && (LCD0Page <= 3)) { MENUE(126, 0);}
        }

  if(control == 22){ //CLEAR - Step
       int z; for(z=0; z<8; z++)  { Sequence[TrackFocus][StepFocus[TrackFocus]][z] = 0;} //fill Page with 0ś
        if((LCD0Page >= 0) && (LCD0Page <= 3)) {MENUE(126, 0);}} //Update Menu
        
  if(control == 23){ //COPY  - Track
       int x; for(x=0; x<8; x++) { int y; for(y=0; y<256; y++)  {TrackClipboard[y][x]  =  Sequence[TrackFocus][y][x];}}} 

  if(control == 24){ //PASTE - Track
       int x; for(x=0; x<8; x++) { int y; for(y=0; y<256; y++)  {Sequence[TrackFocus][y][x] = TrackClipboard[y][x];}} MENUE(126, 0);}
       
  if(control == 25){ //CLEAR - Track
       int x; for(x=0; x<8; x++) { int y; for(y=0; y<256; y++)  {Sequence[TrackFocus][y][x] = 0;}} MENUE(126, 0);}

}//End Sequencer Stuff
}//End DIN-Midi Input
} //End Hook
*/



	
void APP_Tick(void){ // called each mS from main task which also handles DIN, ENC and AIN events. You could add more jobs here - dont do more than 300uS!
	  SDCardCount = SDCardCount + 1; if(SDCardCount > 2000){SDCardCount = 0; StoreLoad(1, 3);} //2s Counter > send Check-SDCard-Commant to StoreLoad-Function
  if(MenueUpdateFlag==0){MenueUpdateCount = MenueUpdateCount + 1; 
     if(MenueUpdateCount > 250){MenueUpdateCount = 0; //update the Menue 0,15seconds after a message like "stored" (1.time!)
		                         MenueUpdateFlag=1; //deactivate Timer
		                         MENUE(126, 0);}} //Update LCD with the currently selected Menu}}

  //Programchange Load when Stopped turning the Wheel (from External Source, internal we use a LOAD-Button for that)

  if(PCLoadFlag==1){PCLoadFlagCount = PCLoadFlagCount + 1; 
     if(PCLoadFlagCount >= 200) {LCD0Page = 0; //Switch to Song Load Display
                                 PCLoadFlagCount = 0; //init counter for the next run
                                 StoreLoad(1, 1); //LOAD Program
                                 PCLoadFlag = 0;  //Kill Flag to avoid double triggering events
                                 MenueUpdateFlag=0;
		                         MENUE(126, 0);}} //Update LCD with the currently selected Menu}}
}

void APP_MIDI_Tick(void){} //@1mS rate, dont do more then 300uS!

void APP_MIDI_NotifyPackage(mios32_midi_port_t port, mios32_midi_package_t midi_package){
  //MIOS32_LCD_CursorSet(0, 0);  MIOS32_LCD_PrintFormattedString("%d %d %d %d", port, midi_package.chn, midi_package.note, midi_package.velocity);

if(midi_package.evnt0 != 248 && midi_package.evnt0 != 187){//avoid that Midiclock DataMess
  //Receive ProgramChange Information
  if(port == PCPortIN && PCReceive == 1) {
    //Set MidiChannel && Programchange-Channel     set Song Nr with Programchange Nr  show Page PC  setLoadProgram     LOAD command   
    if(PCChIN == 0  && midi_package.evnt0 == 192) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;}
    if(PCChIN == 1  && midi_package.evnt0 == 193) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 2  && midi_package.evnt0 == 194) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;}
    if(PCChIN == 3  && midi_package.evnt0 == 195) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 4  && midi_package.evnt0 == 196) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;}
    if(PCChIN == 5  && midi_package.evnt0 == 197) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 6  && midi_package.evnt0 == 198) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 7  && midi_package.evnt0 == 199) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 8  && midi_package.evnt0 == 200) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 9  && midi_package.evnt0 == 201) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 10 && midi_package.evnt0 == 202) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 11 && midi_package.evnt0 == 203) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 12 && midi_package.evnt0 == 204) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 13 && midi_package.evnt0 == 205) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 14 && midi_package.evnt0 == 206) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;} 
    if(PCChIN == 15 && midi_package.evnt0 == 207) {SongNrLoad = midi_package.evnt1; LCD0Page = 0; MENUE(0, 0); PCLoadFlag = 1;}


//MIOS32_LCD_Clear(); //clear screen 
//MIOS32_LCD_CursorSet(0,  0);MIOS32_LCD_PrintFormattedString("%d %d %d", midi_package.evnt0, midi_package.evnt1, midi_package.evnt2);
//MIOS32_LCD_CursorSet(0,  1);MIOS32_LCD_PrintFormattedString("%d %d", SongNrLoad, PCPortIN);

}

  //ReRoute PitchBend, CC & NOTE Inputs          
  if(port==MelPortIn){//MELODY-INPUT-BUFFER////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 

      //Pitchbend s16bit Value Calculation
      u8 channel = 0;
      if(midi_package.evnt0 >= 224 && midi_package.evnt0 <= 239){channel = midi_package.evnt0 - 224; //Offset to MidiChannel 0-15
      u16 msb = midi_package.evnt2 << 7;
      pitchbend[channel] = midi_package.evnt1 + msb;}

      //Melody-Buffer [0]
      if(MelImute[0] == 0) { // MATRIX = Melody HOLD or "Mute Melody Input"
            if(port==MelPortIn)  {         //right Midi-Port?
               if((midi_package.chn==0) && (MelIroute[0][0]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER0(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER0(1, midi_package.note);}
                  if(PitchbendPWR[0]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[0], pitchbend[0]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==1) && (MelIroute[1][0]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER0(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER0(1, midi_package.note);}
                  if(PitchbendPWR[0]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[0], pitchbend[1]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==2) && (MelIroute[2][0]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER0(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER0(1, midi_package.note);}
                  if(PitchbendPWR[0]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[0], pitchbend[2]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==3) && (MelIroute[3][0]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER0(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER0(1, midi_package.note);}
                  if(PitchbendPWR[0]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[0], pitchbend[3]);} // SEND Pitchbend(port,chn, u16 val)
                }


               if((midi_package.chn==4) && (MelIroute[4][0]==1)){  //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER0(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER0(1, midi_package.note);}
                  if(PitchbendPWR[0]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[0], pitchbend[4]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==5) && (MelIroute[5][0]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER0(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER0(1, midi_package.note);}
                  if(PitchbendPWR[0]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[0], pitchbend[5]);} // SEND Pitchbend(port,chn, u16 val)
               }
                          }}

      //Melody-Buffer [1]
      if(MelImute[1] == 0) { // MATRIX = Melody HOLD or "Mute Melody Input"
            if(port==MelPortIn)  {         //right Midi-Port?
               if((midi_package.chn==0) && (MelIroute[0][1]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER1(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER1(1, midi_package.note);}
                  if(PitchbendPWR[1]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[1], pitchbend[0]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==1) && (MelIroute[1][1]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER1(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER1(1, midi_package.note);}
                  if(PitchbendPWR[1]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[1], pitchbend[1]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==2) && (MelIroute[2][1]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER1(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER1(1, midi_package.note);}
                  if(PitchbendPWR[1]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[1], pitchbend[2]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==3) && (MelIroute[3][1]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER1(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER1(1, midi_package.note);}
                  if(PitchbendPWR[1]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[1], pitchbend[3]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==4) && (MelIroute[4][1]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER1(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER1(1, midi_package.note);}
                  if(PitchbendPWR[1]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[1], pitchbend[4]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==5) && (MelIroute[5][1]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
               }

                          }}

      //Melody-Buffer [2]
      if(MelImute[2] == 0) { // MATRIX = Melody HOLD or "Mute Melody Input"
            if(port==MelPortIn)  {         //right Midi-Port?
               if((midi_package.chn==0) && (MelIroute[0][2]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER2(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER2(1, midi_package.note);}
                  if(PitchbendPWR[2]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[2], pitchbend[0]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==1) && (MelIroute[1][2]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER2(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER2(1, midi_package.note);}
                  if(PitchbendPWR[2]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[2], pitchbend[1]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==2) && (MelIroute[2][2]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER2(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER2(1, midi_package.note);}
                  if(PitchbendPWR[2]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[2], pitchbend[2]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==3) && (MelIroute[3][2]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER2(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER2(1, midi_package.note);}
                  if(PitchbendPWR[2]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[2], pitchbend[3]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==4) && (MelIroute[4][2]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER2(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER2(1, midi_package.note);}
                  if(PitchbendPWR[2]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[2], pitchbend[4]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==5) && (MelIroute[5][2]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER2(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER2(1, midi_package.note);}
                  if(PitchbendPWR[2]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[2], pitchbend[5]);} // SEND Pitchbend(port,chn, u16 val)
               }

                          }}

      //Melody-Buffer [3]
      if(MelImute[3] == 0) { // MATRIX = Melody HOLD or "Mute Melody Input"
            if(port==MelPortIn)  {         //right Midi-Port?
               if((midi_package.chn==0) && (MelIroute[0][3]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER3(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER3(1, midi_package.note);}
                  if(PitchbendPWR[3]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[3], pitchbend[0]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==1) && (MelIroute[1][3]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER3(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER3(1, midi_package.note);}
                  if(PitchbendPWR[3]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[3], pitchbend[1]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==2) && (MelIroute[2][3]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER3(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER3(1, midi_package.note);}
                  if(PitchbendPWR[3]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[3], pitchbend[2]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==3) && (MelIroute[3][3]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER3(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER3(1, midi_package.note);}
                  if(PitchbendPWR[3]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[3], pitchbend[3]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==4) && (MelIroute[4][3]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER3(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER3(1, midi_package.note);}
                  if(PitchbendPWR[3]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[3], pitchbend[4]);} // SEND Pitchbend(port,chn, u16 val)
               }

               if((midi_package.chn==5) && (MelIroute[5][3]==1)){ //MATRIX = Melody Routing > which MidiChannel-IN to which MidiChannel-OUT
                  if((midi_package.type==NoteOn && midi_package.velocity<=0) || (midi_package.type == NoteOff)) { MELODY_BUFFER3(0, midi_package.note);}
                  if( midi_package.type==NoteOn && midi_package.velocity>=1)                                    { MELODY_BUFFER3(1, midi_package.note);}
                  if(PitchbendPWR[3]==1) {MIOS32_MIDI_SendPitchBend(MelPortOut, MeloOutCh[3], pitchbend[5]);} // SEND Pitchbend(port,chn, u16 val)
               }

                          }}

} //END MELODY-INPUT-BUFFER//////////////////////////////////////////////////////////////////////////////////////////         
} //END MidiClock-Filter
}


static s32 APP_MIDI_TimeOut(mios32_midi_port_t port) { // parse incoming sysex stream for MIOS32 commands
  return 0; // no error
}


void APP_SRIO_ServicePrepare(void){//called before shift register chain is scanned
////////////////////////////////////////////////////////////////////////////////////////
//    
//    #        #####    ###       ####     #    #    #     ###     
//    #        #        #  #      #   #    #    ##   #    #        
//    #        #####    #   #     ###      #    # #  #    #  ##    
//    #        #        #  #      #  #     #    #  # #    #   #    
//    #####    #####    ###       #   #    #    #   ##     ####    
//
//
////STEP-POSITION INDICATION////

//1. Deactivate all Leds from a Ledring
    int u = 0;
    for(u = 0; u < 16; u++) {MIOS32_DOUT_PinSet(u, OFF);} 
  
//2. Deactivate Common-Pins from all Ledrings
    int v = 0;
    for(v = 0; v < 8; v++) {MIOS32_DOUT_PinSet(v+16, ON);} 

//3. Activate Common PIN
    //Count thru common Pins
    LED_RING_Common = LED_RING_Common + 1; 
    if (LED_RING_Common > 7) {LED_RING_Common = 0;}
    //Activate Common pin
    MIOS32_DOUT_PinSet((LED_RING_Common + 16),     OFF);

//4. Activate LED-PIN for current Step
    MIOS32_DOUT_PinSet(IND[LED_RING_Common],     ON);

/////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////
//    
//    #        #####    ###        
//    #        #        #  #            
//    #        #####    #   #        
//    #        #        #  #       
//    #####    #####    ###        
//
//
//Indicate the PageGroup 0
  if(ActualPage == 0) {
    MIOS32_DOUT_PinSet(40, 0); //Turn OFF PAGE[1] Bank - LEDs
    MIOS32_DOUT_PinSet(41, 0); //Turn OFF PAGE[1] Bank - LEDs
    MIOS32_DOUT_PinSet(42, 0); //Turn OFF PAGE[1] Bank - LEDs                              
    MIOS32_DOUT_PinSet(43, 0); //Turn OFF PAGE[1] Bank - LEDs 

    if(PAGE[ActualPage] == 0)   { MIOS32_DOUT_PinSet(32, 1); //Activate Page[0] Bank - LED
                                  MIOS32_DOUT_PinSet(33, 0);
                                  MIOS32_DOUT_PinSet(34, 0);                         
                                  MIOS32_DOUT_PinSet(35, 0);  }

    if(PAGE[ActualPage] == 1)   { MIOS32_DOUT_PinSet(32, 0);
                                  MIOS32_DOUT_PinSet(33, 1);
                                  MIOS32_DOUT_PinSet(34, 0);                              
                                  MIOS32_DOUT_PinSet(35, 0);  }

    if(PAGE[ActualPage] == 2)   { MIOS32_DOUT_PinSet(32, 0);
                                  MIOS32_DOUT_PinSet(33, 0);
                                  MIOS32_DOUT_PinSet(34, 1);                              
                                  MIOS32_DOUT_PinSet(35, 0);  }

    if(PAGE[ActualPage] == 3)   { MIOS32_DOUT_PinSet(32, 0);
                                  MIOS32_DOUT_PinSet(33, 0);
                                  MIOS32_DOUT_PinSet(34, 0);                              
                                  MIOS32_DOUT_PinSet(35, 1);  }
  }

//Indicate the PageGroup 1
  if(ActualPage == 1) {
    MIOS32_DOUT_PinSet(32, 0); //Turn OFF PAGE[0] Bank - LEDs
    MIOS32_DOUT_PinSet(33, 0); //Turn OFF PAGE[0] Bank - LEDs
    MIOS32_DOUT_PinSet(34, 0); //Turn OFF PAGE[0] Bank - LEDs                              
    MIOS32_DOUT_PinSet(35, 0); //Turn OFF PAGE[0] Bank - LEDs 

    if(PAGE[ActualPage] == 0)   { MIOS32_DOUT_PinSet(40,  1); //Activate Page[0] Bank - LED
                                  MIOS32_DOUT_PinSet(41,  0);
                                  MIOS32_DOUT_PinSet(42, 0);                         
                                  MIOS32_DOUT_PinSet(43, 0);  }

    if(PAGE[ActualPage] == 1)   { MIOS32_DOUT_PinSet(40,  0);
                                  MIOS32_DOUT_PinSet(41,  1);
                                  MIOS32_DOUT_PinSet(42, 0);                              
                                  MIOS32_DOUT_PinSet(43, 0);  }

    if(PAGE[ActualPage] == 2)   { MIOS32_DOUT_PinSet(40,  0);
                                  MIOS32_DOUT_PinSet(41,  0);
                                  MIOS32_DOUT_PinSet(42, 1);                              
                                  MIOS32_DOUT_PinSet(43, 0);  }

    if(PAGE[ActualPage] == 3)   { MIOS32_DOUT_PinSet(40,  0);
                                  MIOS32_DOUT_PinSet(41,  0);
                                  MIOS32_DOUT_PinSet(42, 0);                              
                                  MIOS32_DOUT_PinSet(43, 1);  }
  }

//Show Virtual Pages
if(ActualPage == 0){
  //1st Line of Virtual-LEDs
  MIOS32_DOUT_PinSet( 48, VLED[ActualPage][PAGE[0]][0]);
  MIOS32_DOUT_PinSet( 49, VLED[ActualPage][PAGE[0]][1]); 
  MIOS32_DOUT_PinSet( 50, VLED[ActualPage][PAGE[0]][2]);    
  MIOS32_DOUT_PinSet( 51, VLED[ActualPage][PAGE[0]][3]);     
  MIOS32_DOUT_PinSet( 52, VLED[ActualPage][PAGE[0]][4]); 	       
  MIOS32_DOUT_PinSet( 53, VLED[ActualPage][PAGE[0]][5]);      
  MIOS32_DOUT_PinSet( 54, VLED[ActualPage][PAGE[0]][6]);       
  MIOS32_DOUT_PinSet( 55, VLED[ActualPage][PAGE[0]][7]);
  //2nd Line of Virtual-LEDs
  MIOS32_DOUT_PinSet( 56, VLED[ActualPage][PAGE[0]][8]);
  MIOS32_DOUT_PinSet( 57, VLED[ActualPage][PAGE[0]][9]); 
  MIOS32_DOUT_PinSet( 58, VLED[ActualPage][PAGE[0]][10]);        
  MIOS32_DOUT_PinSet( 59, VLED[ActualPage][PAGE[0]][11]);        
  MIOS32_DOUT_PinSet( 60, VLED[ActualPage][PAGE[0]][12]);        
  MIOS32_DOUT_PinSet( 61, VLED[ActualPage][PAGE[0]][13]); 
  MIOS32_DOUT_PinSet( 62, VLED[ActualPage][PAGE[0]][14]);                                                     
  MIOS32_DOUT_PinSet( 63, VLED[ActualPage][PAGE[0]][15]);}

if( (ActualPage == 1) && (Page == 0) ) {
  //1st Line of Virtual-LEDs
  MIOS32_DOUT_PinSet( 48, MelOmute[0]);
  MIOS32_DOUT_PinSet( 49, MelOmute[1]); 
  MIOS32_DOUT_PinSet( 50, MelOmute[2]);    
  MIOS32_DOUT_PinSet( 51, MelOmute[3]);     
  MIOS32_DOUT_PinSet( 52, MelOmute[4]); 	       
  MIOS32_DOUT_PinSet( 53, MelOmute[5]);      
  MIOS32_DOUT_PinSet( 54, MelOmute[6]);       
  MIOS32_DOUT_PinSet( 55, MelOmute[7]);
  //2nd Line of Virtual-LEDs
  MIOS32_DOUT_PinSet( 56, REC[0]);
  MIOS32_DOUT_PinSet( 57, REC[1]); 
  MIOS32_DOUT_PinSet( 58, REC[2]);        
  MIOS32_DOUT_PinSet( 59, REC[3]);        
  MIOS32_DOUT_PinSet( 60, REC[4]);        
  MIOS32_DOUT_PinSet( 61, REC[5]); 
  MIOS32_DOUT_PinSet( 62, REC[6]);                                                     
  MIOS32_DOUT_PinSet( 63, REC[7]);}

//SEQ-Is-Running-LED
if(SEQ_BPM_IsRunning()==1) {MIOS32_DOUT_PinSet(37,1);}//Play LED Deactive
                      else {MIOS32_DOUT_PinSet(37,0);}//Play LED Active
	} 
    
    
void APP_SRIO_ServiceFinish(void){} //called after shift register chain was scanned



void APP_DIN_NotifyToggle(u32 pin, u32 pin_value){
  //Support Virtual Pages    //VLED[PAGE][LINE][LED]    VLED [PAGE[ActualPage]]  [0]  [0]);
  if(pin == 0 && pin_value == 0){  //Virtual Button-Switch 0  to  switch between Pages of Actual Page 0

    if(ActualPage != 0) {ActualPage = 0;} //only switch to the actual pageś (we have 2 Bank Button
    else {PAGE[ActualPage] = PAGE[ActualPage] +1; //count thru the pages
               if(PAGE[ActualPage] <= 0) {PAGE[ActualPage] = 0;}; //just to be sure...
               if(PAGE[ActualPage] >  3) {PAGE[ActualPage] = 0;};} //step thru the Pages in a cycle
    //Update Menue-LCDs
    MENUE(126, 0);
}
  //Support Virtual Pages    //VLED[PAGE][LINE][LED]    VLED [PAGE[ActualPage]]  [0]  [0]);
  if(pin == 1 && pin_value == 0){  //Virtual Button-Switch 0  to  switch between Pages of Actual Page 0
    if(ActualPage != 1) {ActualPage = 1;} //only switch to the actual pageś (we have 2 Bank Buttons)
    else {PAGE[ActualPage] = PAGE[ActualPage] +1; //count thru the pages
               if(PAGE[ActualPage] <= 0) {PAGE[ActualPage] = 0;}; //just to be sure...
               if(PAGE[ActualPage] >  3) {PAGE[ActualPage] = 0;};} //step thru the Pages in a cycle
    //Update Menue-LCDs
    MENUE(126, 0);
}


  //Route 16x Footboard buttons to Menue-Hook
  if(pin >= 8) { //where the16 footboard switche pins begins
      pin = pin - 8;//offset to put them from 0-16
      MENUE(pin, pin_value);  //forwarding to Function "MENUE"
}
/*  STORE LOAD BUTTONS
	if(LCD0Page == 5){//Store Pattern || Store System Settings
		if(pin == 18 && pin_value == 0) {StoreLoad(1, 2);  MenueUpdateFlag=0;}  //Store
		if(pin == 19 && pin_value == 0) {StoreLoad(1, 1);  MenueUpdateFlag=0;}} //Load
*/ 
}



void APP_ENC_NotifyChange(u32 encoder, s32 incrementer){}

void APP_AIN_NotifyChange(u32 pin, u32 pin_value){} //called when a pot has been moved

static void TASK_SEQ(void *pvParameters){ //called periodically each mS to handle sequencer requests
  portTickType xLastExecutionTime;

  // Initialise the xLastExecutionTime variable on task entry
  xLastExecutionTime = xTaskGetTickCount();

  while( 1 ) {
    vTaskDelayUntil(&xLastExecutionTime, 1 / portTICK_RATE_MS);

    // execute sequencer handler
    SEQ_Handler();

  }
}

static s32 NOTIFY_MIDI_Rx(mios32_midi_port_t port, u8 midi_byte){//filter Midiclock Port and rerout it to SEQUENCER-BPM
if(port == ClockPortIN) { SEQ_BPM_NotifyMIDIRx(midi_byte); }
  return 0; // no error, no filtering
}



//Melody-Out-Generation - via incoming sequence
u8 APP_NOTEPROCESS(u8 port, u8 note, u8 velocity, u16 step, u16 tic){ //do triggering
 //MIOS32_LCD_DeviceSet  (0); MIOS32_LCD_Clear(); MIOS32_LCD_CursorSet(0, 0);  MIOS32_LCD_PrintFormattedString("%d", step);
  
//Random Trigger Gate 4  "MKill"  & "DKill"
  if(tic == 0  || tic == BaseTic*4) { //avoid to much Calculations every 32n step is enough
  if(port == 0){ //sequencer port
    for(TMR0L=0; TMR0L<127; TMR0L++){random_tmp=random_tmp+1;} //Random-Value 4 RandomKill 
    random_tmp = random_seed_l * random_seed_h;
    random_seed_l = TMR0L + (random_tmp & 0xff); //TMR0L = Timer
    random_seed_h = 0x69 + (random_tmp >> 8);
    random_nr = random_tmp % 127; //>> 2;       // divide by 2 > bitshifting - thx2"kpete" 
    if(RandomGateEnc > random_nr) {MeloRandGate = 1;} //Pass Notes   /CCBank4[0][0] = MKill - Melody-Trigger-Kill
    if(RandomGateEnc < random_nr) {MeloRandGate = 0;} //Kill Notes   /CCBank4[0][0] = MKill - Melody-Trigger-Kill
   }}

//RECORD Track 0 a
  if(REC[0] == 1) {//Record Switch Active?
              //Write Notes in currently played Step
              if(tic <= (BaseTic*6) )  {  //12x6=72 (Tic 0-72)
                  int x=0;	for(x=0; x<8; ++x) {
                    Sequence[0] [loop_step_pos[0]] [x] = RecNoteBuff[0][x];} 
              }
              else { //UP-QUANTIZE > Write Notes into Next Step

                      
                    if(loop_step_pos[0] == ((Loop_Length[0]+1) * StepPerPage)) { //when Last step is arrived
                        int x=0;	for(x=0; x<8; ++x) { Sequence[0][0][x] = RecNoteBuff[0][x];}} 
                      
                    if(loop_step_pos[0] <  ((Loop_Length[0]+1) * StepPerPage))  { //If not Last Step, then simply write in next Step:
                        int x=0;	for(x=0; x<8; ++x) { Sequence[0][loop_step_pos[0]+1][x] = RecNoteBuff[0][x];}} 
                   
}
    }

//RECORD Track 0 b
  if(REC[1] == 1) {//Record Switch Active?
              //Write Notes in currently played Step
              if(tic <= (BaseTic*6) )  {  //12x6=72 (Tic 0-72)
                  int x=0;	for(x=0; x<8; ++x) {
                    Sequence[1] [loop_step_pos[1]] [x] = RecNoteBuff[0][x];} 
              }
              else { //UP-QUANTIZE > Write Notes into Next Step

                      
                    if(loop_step_pos[1] == ((Loop_Length[1]+1) * StepPerPage)) { //when Last step is arrived
                        int x=0;	for(x=0; x<8; ++x) { Sequence[1][0][x] = RecNoteBuff[0][x];}} 
                      
                    if(loop_step_pos[1] <  ((Loop_Length[1]+1) * StepPerPage))  { //If not Last Step, then simply write in next Step:
                        int x=0;	for(x=0; x<8; ++x) { Sequence[1][loop_step_pos[1]+1][x] = RecNoteBuff[0][x];}} 
                   
                }
    }

//RECORD Track 1 b
  if(REC[2] == 1) {//Record Switch Active?
              //Write Notes in currently played Step
              if(tic <= (BaseTic*6) )  {  //12x6=72 (Tic 0-72)
                  int x=0;	for(x=0; x<8; ++x) {
                    Sequence[2] [loop_step_pos[2]] [x] = RecNoteBuff[0][x];} 
              }
              else { //UP-QUANTIZE > Write Notes into Next Step

                      
                    if(loop_step_pos[2] == ((Loop_Length[2]+1) * StepPerPage)) { //when Last step is arrived
                        int x=0;	for(x=0; x<8; ++x) { Sequence[2][0][x] = RecNoteBuff[0][x];}} 
                      
                    if(loop_step_pos[2] <  ((Loop_Length[2]+1) * StepPerPage))  { //If not Last Step, then simply write in next Step:
                        int x=0;	for(x=0; x<8; ++x) { Sequence[2][loop_step_pos[2]+1][x] = RecNoteBuff[0][x];}} 
                   
                }
    }

//RECORD Track 1 b
  if(REC[3] == 1) {//Record Switch Active?
              //Write Notes in currently played Step
              if(tic <= (BaseTic*6) )  {  //12x6=72 (Tic 0-72)
                  int x=0;	for(x=0; x<8; ++x) {
                    Sequence[3] [loop_step_pos[3]] [x] = RecNoteBuff[0][x];} 
              }
              else { //UP-QUANTIZE > Write Notes into Next Step

                      
                    if(loop_step_pos[3] == ((Loop_Length[3]+1) * StepPerPage)) { //when Last step is arrived
                        int x=0;	for(x=0; x<8; ++x) { Sequence[3][0][x] = RecNoteBuff[0][x];}} 
                      
                    if(loop_step_pos[3] <  ((Loop_Length[3]+1) * StepPerPage))  { //If not Last Step, then simply write in next Step:
                        int x=0;	for(x=0; x<8; ++x) { Sequence[3][loop_step_pos[3]+1][x] = RecNoteBuff[0][x];}} 
                   
                }
    }




////////////////////////////////////////////////////////////////////                                   
//Trigger0 - Poly-Retrigger   u8 RecMode = 0; //0:Realtime, 1:StepEnter
   if (RecMode == 0) { //@Realtime Record/Play
       if ( (MelOmute[0]==1) &&  //ACTIVATE Button 
           (port == 0)       &&  //InternalStepSequencer Input
              ( ((Sequence[0] [loop_step_pos[0]] [0] > 0) && (MainDelay  == tic))  || //any Note in Sequencer Step? Right Tic?
                ((Sequence[0] [loop_step_pos[0]] [1] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [loop_step_pos[0]] [2] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [loop_step_pos[0]] [3] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [loop_step_pos[0]] [4] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [loop_step_pos[0]] [5] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [loop_step_pos[0]] [6] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [loop_step_pos[0]] [7] > 0) && (MainDelay  == tic))  
                )   )
         {  //send Poly Melody NoteOut
            if(MeloRandGate==1 || RandomActive[0]==0){   //RandomGate || Does Random Gate have Effect?
              Decay = SetDecayTime[0]; //get predefinied Melody-Decay-Time
              int buf=0;	for(buf=0; buf<8; ++buf) {
                               if(Sequence[0][loop_step_pos[0]][buf] > 0) {
                               MIOS32_MIDI_SendNoteOn(MelPortOut, MeloOutCh[0], Sequence[0][loop_step_pos[0]][buf], 127); 
                               MeloDecayTIMER[0][Sequence[0][loop_step_pos[0]][buf]] = Decay;}}
                  }}}
                  
  else if (RecMode == 1) { //@StepEntry Record
       if ( (MelOmute[0]==1) &&  //ACTIVATE Button 
           (port == 0)       &&  //InternalStepSequencer Input
              ( ((Sequence[0] [StepLCD] [0] > 0) && (MainDelay  == tic))  || //any Note in Sequencer loop_step_pos[0]? Right Tic?
                ((Sequence[0] [StepLCD] [1] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [StepLCD] [2] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [StepLCD] [3] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [StepLCD] [4] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [StepLCD] [5] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [StepLCD] [6] > 0) && (MainDelay  == tic))  ||
                ((Sequence[0] [StepLCD] [7] > 0) && (MainDelay  == tic))  
                )   )
         {  //send Poly Melody NoteOut
            if(MeloRandGate==1 || RandomActive[0]==0){   //RandomGate || Does Random Gate have Effect?
              Decay = SetDecayTime[0]; //get predefinied Melody-Decay-Time
              int buf=0;	for(buf=0; buf<8; ++buf) {
                               if(Sequence[0][StepLCD][buf] > 0) {
                               MIOS32_MIDI_SendNoteOn(MelPortOut, MeloOutCh[0], Sequence[0][StepLCD][buf], 127); 
                               MeloDecayTIMER[0][Sequence[0][StepLCD][buf]] = Decay;}}
                  }}}                  


////////////////////////////////////////////////////////////////////                                   
//Trigger[1] - Poly-Retrigger   u8 RecMode = 0; //0:Realtime, 1:StepEnter
   if (RecMode == 0) { //@Realtime Record/Play
       if ( (MelOmute[1]==1) &&  //ACTIVATE Button 
           (port == 0)       &&  //InternalStepSequencer Input
              ( ((Sequence[1] [loop_step_pos[1]] [0] > 0) && (MainDelay  == tic))  || //any Note in Sequencer Step? Right Tic?
                ((Sequence[1] [loop_step_pos[1]] [1] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [loop_step_pos[1]] [2] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [loop_step_pos[1]] [3] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [loop_step_pos[1]] [4] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [loop_step_pos[1]] [5] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [loop_step_pos[1]] [6] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [loop_step_pos[1]] [7] > 0) && (MainDelay  == tic))  
                )   )
         {  //send Poly Melody NoteOut
            if(MeloRandGate==1 || RandomActive[1]==0){   //RandomGate || Does Random Gate have Effect?
              Decay = SetDecayTime[1]; //get predefinied Melody-Decay-Time
              int buf=0;	for(buf=0; buf<8; ++buf) {
                               if(Sequence[1][loop_step_pos[1]][buf] > 0) {
                               MIOS32_MIDI_SendNoteOn(MelPortOut, MeloOutCh[1], Sequence[1][loop_step_pos[1]][buf], 127); 
                               MeloDecayTIMER[1][Sequence[1][loop_step_pos[1]][buf]] = Decay;}}
                  }}}
                  
  else if (RecMode == 1) { //@StepEntry Record
       if ( (MelOmute[1]==1) &&  //ACTIVATE Button 
           (port == 0)       &&  //InternalStepSequencer Input
              ( ((Sequence[1] [StepLCD] [0] > 0) && (MainDelay  == tic))  || //any Note in Sequencer Step? Right Tic?
                ((Sequence[1] [StepLCD] [1] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [StepLCD] [2] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [StepLCD] [3] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [StepLCD] [4] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [StepLCD] [5] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [StepLCD] [6] > 0) && (MainDelay  == tic))  ||
                ((Sequence[1] [StepLCD] [7] > 0) && (MainDelay  == tic))  
                )   )
         {  //send Poly Melody NoteOut
            if(MeloRandGate==1 || RandomActive[1]==0){   //RandomGate || Does Random Gate have Effect?
              Decay = SetDecayTime[1]; //get predefinied Melody-Decay-Time
              int buf=0;	for(buf=0; buf<8; ++buf) {
                               if(Sequence[1][StepLCD][buf] > 0) {
                               MIOS32_MIDI_SendNoteOn(MelPortOut, MeloOutCh[1], Sequence[1][StepLCD][buf], 127); 
                               MeloDecayTIMER[1][Sequence[1][StepLCD][buf]] = Decay;}}
                  }}}                                   

////////////////////////////////////////////////////////////////////                                   
//Trigger[2] - Poly-Retrigger   u8 RecMode = 0; //0:Realtime, 1:StepEnter
   if (RecMode == 0) { //@Realtime Record/Play
       if ( (MelOmute[2]==1) &&  //ACTIVATE Button 
           (port == 0)       &&  //InternalStepSequencer Input
              ( ((Sequence[2] [loop_step_pos[2]] [0] > 0) && (MainDelay  == tic))  || //any Note in Sequencer Step? Right Tic?
                ((Sequence[2] [loop_step_pos[2]] [1] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [loop_step_pos[2]] [2] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [loop_step_pos[2]] [3] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [loop_step_pos[2]] [4] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [loop_step_pos[2]] [5] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [loop_step_pos[2]] [6] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [loop_step_pos[2]] [7] > 0) && (MainDelay  == tic))  
                )   )
         {  //send Poly Melody NoteOut
            if(MeloRandGate==1 || RandomActive[2]==0){   //RandomGate || Does Random Gate have Effect?
              Decay = SetDecayTime[2]; //get predefinied Melody-Decay-Time
              int buf=0;	for(buf=0; buf<8; ++buf) {
                               if(Sequence[2][loop_step_pos[2]][buf] > 0) {
                               MIOS32_MIDI_SendNoteOn(MelPortOut, MeloOutCh[2], Sequence[2][loop_step_pos[2]][buf], 127); 
                               MeloDecayTIMER[2][Sequence[2][loop_step_pos[2]][buf]] = Decay;}}
                  }}}
                  
  else if (RecMode == 1) { //@StepEntry Record
       if ( (MelOmute[2]==1) &&  //ACTIVATE Button 
           (port == 0)       &&  //InternalStepSequencer Input
              ( ((Sequence[2] [StepLCD] [0] > 0) && (MainDelay  == tic))  || //any Note in Sequencer Step? Right Tic?
                ((Sequence[2] [StepLCD] [1] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [StepLCD] [2] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [StepLCD] [3] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [StepLCD] [4] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [StepLCD] [5] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [StepLCD] [6] > 0) && (MainDelay  == tic))  ||
                ((Sequence[2] [StepLCD] [7] > 0) && (MainDelay  == tic))  
                )   )
         {  //send Poly Melody NoteOut
            if(MeloRandGate==1 || RandomActive[2]==0){   //RandomGate || Does Random Gate have Effect?
              Decay = SetDecayTime[2]; //get predefinied Melody-Decay-Time
              int buf=0;	for(buf=0; buf<8; ++buf) {
                               if(Sequence[2][StepLCD][buf] > 0) {
                               MIOS32_MIDI_SendNoteOn(MelPortOut, MeloOutCh[2], Sequence[2][StepLCD][buf], 127); 
                               MeloDecayTIMER[2][Sequence[2][StepLCD][buf]] = Decay;}}
                  }}}  

////////////////////////////////////////////////////////////////////                                   
//Trigger[3] - Poly-Retrigger   u8 RecMode = 0; //0:Realtime, 1:StepEnter
   if (RecMode == 0) { //@Realtime Record/Play
       if ( (MelOmute[3]==1) &&  //ACTIVATE Button 
           (port == 0)       &&  //InternalStepSequencer Input
              ( ((Sequence[3] [loop_step_pos[3]] [0] > 0) && (MainDelay  == tic))  || //any Note in Sequencer Step? Right Tic?
                ((Sequence[3] [loop_step_pos[3]] [1] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [loop_step_pos[3]] [2] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [loop_step_pos[3]] [3] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [loop_step_pos[3]] [4] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [loop_step_pos[3]] [5] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [loop_step_pos[3]] [6] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [loop_step_pos[3]] [7] > 0) && (MainDelay  == tic))  
                )   )
         {  //send Poly Melody NoteOut
            if(MeloRandGate==1 || RandomActive[3]==0){   //RandomGate || Does Random Gate have Effect?
              Decay = SetDecayTime[3]; //get predefinied Melody-Decay-Time
              int buf=0;	for(buf=0; buf<8; ++buf) {
                               if(Sequence[3][loop_step_pos[3]][buf] > 0) {
                               MIOS32_MIDI_SendNoteOn(MelPortOut, MeloOutCh[3], Sequence[3][loop_step_pos[3]][buf], 127); 
                               MeloDecayTIMER[3][Sequence[3][loop_step_pos[3]][buf]] = Decay;}}
                  }}}
                  
  else if (RecMode == 1) { //@StepEntry Record
       if ( (MelOmute[3]==1) &&  //ACTIVATE Button 
           (port == 0)       &&  //InternalStepSequencer Input
              ( ((Sequence[3] [StepLCD] [0] > 0) && (MainDelay  == tic))  || //any Note in Sequencer Step? Right Tic?
                ((Sequence[3] [StepLCD] [1] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [StepLCD] [2] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [StepLCD] [3] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [StepLCD] [4] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [StepLCD] [5] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [StepLCD] [6] > 0) && (MainDelay  == tic))  ||
                ((Sequence[3] [StepLCD] [7] > 0) && (MainDelay  == tic))  
                )   )
         {  //send Poly Melody NoteOut
            if(MeloRandGate==1 || RandomActive[3]==0){   //RandomGate || Does Random Gate have Effect?
              Decay = SetDecayTime[3]; //get predefinied Melody-Decay-Time
              int buf=0;	for(buf=0; buf<8; ++buf) {
                               if(Sequence[3][StepLCD][buf] > 0) {
                               MIOS32_MIDI_SendNoteOn(MelPortOut, MeloOutCh[3], Sequence[3][StepLCD][buf], 127); 
                               MeloDecayTIMER[3][Sequence[3][StepLCD][buf]] = Decay;}}
                  }}}  

//End-drum-trigger
  return 0; // no error,
}

    

static void TASK_Period1mS(void *pvParameters){    //called @1mS rate   used for BLM
  portTickType xLastExecutionTime;
  xLastExecutionTime = xTaskGetTickCount();   // Initialise the xLastExecutionTime variable on task entry

  while( 1 ) {
    vTaskDelayUntil(&xLastExecutionTime, 1 / portTICK_RATE_MS);

    // skip delay gap if we had to wait for more than 5 ticks to avoid 
    // unnecessary repeats until xLastExecutionTime reached xTaskGetTickCount() again
    portTickType xCurrentTickCount = xTaskGetTickCount();
    if( xLastExecutionTime < (xCurrentTickCount-5) )
      xLastExecutionTime = xCurrentTickCount;

  }
}



static void StoreLoad(u8 tick, u16 order){         //Display MENUE, Store+Load Bank & Sys-Config
	//STORE
    if(tick == 1 && order == 2){//Store Data & update the Load NR & Name													
      MUTEX_SDCARD_TAKE;  statusSD = FILE_CheckSDCard();   MUTEX_SDCARD_GIVE;      //SDC-Connected?
      if(statusSD != 0)  {MIOS32_LCD_DeviceSet(0); MIOS32_LCD_Clear(); MIOS32_LCD_PrintFormattedString       ("%s %d", "CANT-SAVE-Status:", statusSD);}
      if(statusSD == 0) {                                                          // YES CARD!  >>> next: check Card-content 
          MUTEX_SDCARD_TAKE; statusDir = FILE_DirExists("mq"); MUTEX_SDCARD_GIVE;   //ask/call file.c: exist a folder "t/" on the CARD?
           if(statusDir == 0) {MIOS32_LCD_DeviceSet(0); MIOS32_LCD_Clear(); MIOS32_LCD_PrintFormattedString ("%s", "Invalid File Structure>Reconnect Card");}
           if(statusDir == 1) {
           //WRITE  Variables in Array

           CC_SD[2]=Page;
           CC_SD[3]=StepPerPage;
           CC_SD[9]=MainDelay;
           CC_SD[10]=PpqnDividerCount;
           CC_SD[11]=BaseTicCount;
           CC_SD[12]=BPM;
           CC_SD[13]=Rythm;
           CC_SD[14]=RythmRepeat;
           CC_SD[15]=RandomGateEnc;
           
             MUTEX_SDCARD_TAKE;  
					sprintf(filepathS, "mq/%d.mq", SongNrStor);        //First Pattern --- will be later copied 511 times!	
						FILE_WriteOpen	(filepathS, 1);                //WRITE!!!!
							FILE_WriteBuffer((u8  *)file_typeBank,  4);  //"MQ01" = 4 Positons
                            FILE_WriteBuffer((u8  *)Sequence,   12288);  //6x256x8  =12288 - Melody Sequence Data
                            FILE_WriteBuffer((u8  *)MelIroute,     36);
                            FILE_WriteBuffer((u8  *)MelImute,       6);
                            FILE_WriteBuffer((u8  *)MelIshift,      6);     
                            FILE_WriteBuffer((u8  *)MelOmute,       6);   
                            FILE_WriteBuffer((u8  *)RandomActive,   6);   
                            FILE_WriteBuffer((u8  *)SetDecayTime,   6);                              
                            FILE_WriteBuffer((u8  *)CC_SD,        128);  // ALL OTHER VARIABLES are saved in this ARRAY (CC_SD[128]) 
                     FILE_WriteClose	();                          //The First File named "0" is locatet @ SDCARD/mq/0.mq, now its time to copy this 511 times, I want 511 Banks/files
             MUTEX_SDCARD_GIVE;                                   //SD-Card is now free 4 access
             MUTEX_LCD_TAKE; MIOS32_LCD_DeviceSet(0); MIOS32_LCD_Clear(); MIOS32_LCD_PrintFormattedString	("%s", "stored!"); MUTEX_LCD_GIVE; MenueUpdateFlag = 0;}}}//menue Updateflag 2 restore after some time the menue

    //LOAD
    if(tick == 1 && order == 1){//Load Data & update the Load NR & Name and also Store Nr & Store Name
      MUTEX_SDCARD_TAKE;  statusDir = FILE_DirExists("mq"); MUTEX_SDCARD_GIVE;
      if(statusDir != 1) {MUTEX_LCD_TAKE; MIOS32_LCD_DeviceSet(0); MIOS32_LCD_Clear(); MIOS32_LCD_PrintFormattedString	("%s %d", "CANT-LOAD-PATCH-Status:", statusDir); MUTEX_LCD_GIVE;}
      if(statusDir == 1) {
		   sprintf(filepathL, "mq/%d.mq", SongNrLoad);
           FILE_ReadOpen  (&midifile_fi, filepathL);
           FILE_ReadBuffer((u8  *)file_typeBank,  4); 	
           FILE_ReadBuffer((u8  *)Sequence,   12288);  //6x256x8  =12288 - Melody Sequence Data
           FILE_ReadBuffer((u8  *)MelIroute,     36);
           FILE_ReadBuffer((u8  *)MelImute,       6);
           FILE_ReadBuffer((u8  *)MelIshift,      6);     
           FILE_ReadBuffer((u8  *)MelOmute,       6);   
           FILE_ReadBuffer((u8  *)RandomActive,   6);   
           FILE_ReadBuffer((u8  *)SetDecayTime,   6);                              
           FILE_ReadBuffer((u8  *)CC_SD,        128);  // ALL OTHER VARIABLES are saved in this ARRAY (CC_SD[128])            
           FILE_ReadClose	(&midifile_fi);

       MUTEX_SDCARD_GIVE
           //Paste  Variables from Array

           Page              = CC_SD[2];
           StepPerPage       = CC_SD[3];
           MainDelay         = CC_SD[9];
           PpqnDividerCount  = CC_SD[10];
           BaseTicCount      = CC_SD[11];
           BPM               = CC_SD[12];
           Rythm             = CC_SD[13];
           RythmRepeat       = CC_SD[14];
           RandomGateEnc     = CC_SD[15];
           
           SEQ_BPM_Set(BPM);
            if(BaseTicCount == 0){BaseTic =  8;}
       else if(BaseTicCount == 1){BaseTic = 12;}
       else if(BaseTicCount == 2){BaseTic = 18;}
       Tic = 32 * BaseTic; //32x12=384
       SyncFlag = 1; //update ClockCounter when ready...

       MIOS32_LCD_DeviceSet(0); MIOS32_LCD_Clear(); MIOS32_LCD_PrintFormattedString	("%s", "SONG loaded!"); 
       SongNrStor = SongNrLoad; //transfiar load nr 2 store nr.

     }
    //Load Routing 0

    
}
    
    //SD-Card Initalize - SONG
  if(tick == 1 && order == 3){//CHECK SD-Card/Initalize Card
    MUTEX_SDCARD_TAKE;					//SD-Card is now only for the following LINES reserved:
      statusSD = FILE_CheckSDCard();
      if (statusSD == 2) {MUTEX_LCD_TAKE;  MIOS32_LCD_DeviceSet(0); MIOS32_LCD_Clear(); MIOS32_LCD_CursorSet(0, 0); MIOS32_LCD_PrintFormattedString	("%s", "NO SD-Card"); MUTEX_LCD_GIVE;  } 
      if (statusSD == 3) {
         if	(!FILE_SDCardAvailable() ) {MUTEX_LCD_TAKE; MIOS32_LCD_DeviceSet(0); MIOS32_LCD_Clear(); MIOS32_LCD_CursorSet(0, 0); MIOS32_LCD_PrintFormattedString	("%s", "NO Cord(song)");MUTEX_LCD_GIVE;} 
         if	(!FILE_VolumeAvailable() ) {MUTEX_LCD_TAKE; MIOS32_LCD_DeviceSet(0); MIOS32_LCD_Clear(); MIOS32_LCD_CursorSet(0, 0); MIOS32_LCD_PrintFormattedString	("%s", "NO FAT(song)"); MUTEX_LCD_GIVE;}} 
      if (statusSD == 1) {// YES CARD!  >>> next: check Card-content
         statusDir = FILE_DirExists("mq");																								//ask file.c: exist a folder "t/" on the CARD?
         if(statusDir == 1){MUTEX_LCD_TAKE; MIOS32_LCD_DeviceSet(0); MIOS32_LCD_Clear(); MIOS32_LCD_CursorSet(0, 0); MIOS32_LCD_PrintFormattedString	("%s", "good SONG file structure");MenueUpdateFlag = 0;MUTEX_LCD_GIVE;} 
         if(statusDir == 0){MUTEX_LCD_TAKE; MIOS32_LCD_DeviceSet(0); MIOS32_LCD_Clear(); MIOS32_LCD_CursorSet(0, 0); MIOS32_LCD_PrintFormattedString	("%s", "create SONG filestructure.1,5 minutes!");MUTEX_LCD_GIVE;

         //WRITE  Variables from Array

           CC_SD[2]=Page;
           CC_SD[3]=StepPerPage;
           CC_SD[9]=MainDelay;
           CC_SD[10]=PpqnDividerCount;
           CC_SD[11]=BaseTicCount;
           CC_SD[12]=BPM;
           CC_SD[13]=Rythm;
           CC_SD[14]=RythmRepeat;
           CC_SD[15]=RandomGateEnc;
                  
            FILE_MakeDir("mq");                        //make a folder "tm" on the SD-Card (root/t/)
            sprintf(filepathS, "mq/0.mq");             //First Pattern --- will be later copied 511 times!	
                            FILE_WriteOpen	(filepathS, 1);               
                            FILE_WriteBuffer((u8  *)file_typeBank,  4); //"MQ01" = 4 Positons
                            FILE_WriteBuffer((u8  *)Sequence,   12288);  //6x256x8  =12288 - Melody Sequence Data
                            FILE_WriteBuffer((u8  *)MelIroute,     36);
                            FILE_WriteBuffer((u8  *)MelImute,       6);
                            FILE_WriteBuffer((u8  *)MelIshift,      6);     
                            FILE_WriteBuffer((u8  *)MelOmute,       6);   
                            FILE_WriteBuffer((u8  *)RandomActive,   6);   
                            FILE_WriteBuffer((u8  *)SetDecayTime,   6);                              
                            FILE_WriteBuffer((u8  *)CC_SD,        128);  // ALL OTHER VARIABLES are saved in this ARRAY (CC_SD[128]) 
            FILE_WriteClose	();                         //The First File named "0" is locatet @ SDCARD/mq/0.mq, now its time to copy this 511 times, I want 511 Banks/files
            s16 BankCreateCounter = 0;                  //declere and set the Bank Counter inital value to 0 
            for(BankCreateCounter=0; BankCreateCounter<512; BankCreateCounter++){  //countes 2 511 and do following commandos 511 times in a loop:
            char copyfilepath[8];
            sprintf(copyfilepath, "mq/%d.mq", BankCreateCounter); //make a new filename depending on the counter value 1.mq, 2.mq, 3.mq...511.mq
            FILE_Copy ((char *)"mq/0.mq", (char *)copyfilepath);} //copy the File 0.tm to all other 511 files...
    MUTEX_SDCARD_GIVE;												//SD-Card is now free 4 access
    MUTEX_LCD_TAKE; MIOS32_LCD_DeviceSet(0); MIOS32_LCD_Clear(); MIOS32_LCD_CursorSet(0, 0); MIOS32_LCD_PrintFormattedString	("%s", "init SDCard > done"); MUTEX_LCD_GIVE;MenueUpdateFlag = 0;}}}
}



static void MELODY_BUFFER0(u8 velocity, u8 note){  //Melody Input buffer

	 NoteCalc[0] = note;
	 if(NoteCalc[0] > 0 && NoteCalc[0] < 127){// Only Note Numbers in Midi-Range 0-127
	    if(velocity >= 1){              // Only Note On Messages
                                
      //CLEAR NOTE BUFFER
         if(NoteCount[0]==0){int i0b;for(i0b=0; i0b<10; ++i0b)RecNoteBuff[0][i0b] = 0;}//set to Note Nr 0, which we filter later out
         
      //FILL  NOTE BUFFER
         if(NoteCount[0] == 0) {RecNoteBuff[0][0]  = NoteCalc[0];}
         if(NoteCount[0] == 1) {RecNoteBuff[0][1]  = NoteCalc[0];}          
         if(NoteCount[0] == 2) {RecNoteBuff[0][2]  = NoteCalc[0];}
         if(NoteCount[0] == 3) {RecNoteBuff[0][3]  = NoteCalc[0];}
         if(NoteCount[0] == 4) {RecNoteBuff[0][4]  = NoteCalc[0];}
         if(NoteCount[0] == 5) {RecNoteBuff[0][5]  = NoteCalc[0];}
         if(NoteCount[0] == 6) {RecNoteBuff[0][6]  = NoteCalc[0];}
         if(NoteCount[0] == 7) {RecNoteBuff[0][7]  = NoteCalc[0];}                                                                                      
                               
      //NOTE COUNTER 
         NoteCount[0]++;}} //(cheap Polyphony) counts from 0-12 >> this numbers route the notes in their buffers, when it reaches 12 it begins from 0 again - note overflow
         //Count Reset
         if(velocity == 0)                  {NoteCount[0] = 0;} 
         if(velocity >= 1 && NoteCount[0] >=10){NoteCount[0] = 0;} //begin again from 0 when reaching max Count of 10 10Fingers on Keyboard = max.10 Polyphone  
}//END HOOK

static void MELODY_BUFFER1(u8 velocity, u8 note){  //Melody Input buffer
	 NoteCalc[1] = note;
	 if(NoteCalc[1] > 0 && NoteCalc[1] < 127){// Only Note Numbers in Midi-Range 0-127
	    if(velocity >= 1){              // Only Note On Messages
                                
      //CLEAR NOTE BUFFER
         if(NoteCount[1]==0){int i0b;for(i0b=0; i0b<10; ++i0b)RecNoteBuff[1][i0b] = 0;}//set to Note Nr 0, which we filter later out
         
      //FILL  NOTE BUFFER
         if(NoteCount[1] == 0) {RecNoteBuff[1][0]  = NoteCalc[1];}
         if(NoteCount[1] == 1) {RecNoteBuff[1][1]  = NoteCalc[1];}          
         if(NoteCount[1] == 2) {RecNoteBuff[1][2]  = NoteCalc[1];}
         if(NoteCount[1] == 3) {RecNoteBuff[1][3]  = NoteCalc[1];}
         if(NoteCount[1] == 4) {RecNoteBuff[1][4]  = NoteCalc[1];}
         if(NoteCount[1] == 5) {RecNoteBuff[1][5]  = NoteCalc[1];}
         if(NoteCount[1] == 6) {RecNoteBuff[1][6]  = NoteCalc[1];}
         if(NoteCount[1] == 7) {RecNoteBuff[1][7]  = NoteCalc[1];}                                                                                      
                               
      //NOTE COUNTER 
         NoteCount[1]++;}} //(cheap Polyphony) counts from 0-12 >> this numbers route the notes in their buffers, when it reaches 12 it begins from 0 again - note overflow
         //Count Reset
         if(velocity == 0)                  {NoteCount[1] = 0;} 
         if(velocity >= 1 && NoteCount[1] >=10){NoteCount[1] = 0;} //begin again from 0 when reaching max Count of 10 10Fingers on Keyboard = max.10 Polyphone  
}//END HOOK

static void MELODY_BUFFER2(u8 velocity, u8 note){  //Melody Input buffer
	 NoteCalc[2] = note;
	 if(NoteCalc[2] > 0 && NoteCalc[2] < 127){// Only Note Numbers in Midi-Range 0-127
	    if(velocity >= 1){              // Only Note On Messages
                                
      //CLEAR NOTE BUFFER
         if(NoteCount[2]==0){int i0b;for(i0b=0; i0b<10; ++i0b)RecNoteBuff[2][i0b] = 0;}//set to Note Nr 0, which we filter later out
         
      //FILL  NOTE BUFFER
         if(NoteCount[2] == 0) {RecNoteBuff[2][0]  = NoteCalc[2];}
         if(NoteCount[2] == 1) {RecNoteBuff[2][1]  = NoteCalc[2];}          
         if(NoteCount[2] == 2) {RecNoteBuff[2][2]  = NoteCalc[2];}
         if(NoteCount[2] == 3) {RecNoteBuff[2][3]  = NoteCalc[2];}
         if(NoteCount[2] == 4) {RecNoteBuff[2][4]  = NoteCalc[2];}
         if(NoteCount[2] == 5) {RecNoteBuff[2][5]  = NoteCalc[2];}
         if(NoteCount[2] == 6) {RecNoteBuff[2][6]  = NoteCalc[2];}
         if(NoteCount[2] == 7) {RecNoteBuff[2][7]  = NoteCalc[2];}                                                                                      
                               
      //NOTE COUNTER 
         NoteCount[2]++;}} //(cheap Polyphony) counts from 0-12 >> this numbers route the notes in their buffers, when it reaches 12 it begins from 0 again - note overflow
         //Count Reset
         if(velocity == 0)                  {NoteCount[2] = 0;} 
         if(velocity >= 1 && NoteCount[2] >=10){NoteCount[2] = 0;} //begin again from 0 when reaching max Count of 10 10Fingers on Keyboard = max.10 Polyphone  
}//END HOOK

static void MELODY_BUFFER3(u8 velocity, u8 note){  //Melody Input buffer
	 NoteCalc[3] = note;
	 if(NoteCalc[3] > 0 && NoteCalc[3] < 127){// Only Note Numbers in Midi-Range 0-127
	    if(velocity >= 1){              // Only Note On Messages
                                
      //CLEAR NOTE BUFFER
         if(NoteCount[3]==0){int i0b;for(i0b=0; i0b<10; ++i0b)RecNoteBuff[3][i0b] = 0;}//set to Note Nr 0, which we filter later out
         
      //FILL  NOTE BUFFER
         if(NoteCount[3] == 0) {RecNoteBuff[3][0]  = NoteCalc[3];}
         if(NoteCount[3] == 1) {RecNoteBuff[3][1]  = NoteCalc[3];}          
         if(NoteCount[3] == 2) {RecNoteBuff[3][2]  = NoteCalc[3];}
         if(NoteCount[3] == 3) {RecNoteBuff[3][3]  = NoteCalc[3];}
         if(NoteCount[3] == 4) {RecNoteBuff[3][4]  = NoteCalc[3];}
         if(NoteCount[3] == 5) {RecNoteBuff[3][5]  = NoteCalc[3];}
         if(NoteCount[3] == 6) {RecNoteBuff[3][6]  = NoteCalc[3];}
         if(NoteCount[3] == 7) {RecNoteBuff[3][7]  = NoteCalc[3];}                                                                                      
                               
      //NOTE COUNTER 
         NoteCount[3]++;}} //(cheap Polyphony) counts from 0-12 >> this numbers route the notes in their buffers, when it reaches 12 it begins from 0 again - note overflow
         //Count Reset
         if(velocity == 0)                  {NoteCount[3] = 0;} 
         if(velocity >= 1 && NoteCount[3] >=10){NoteCount[3] = 0;} //begin again from 0 when reaching max Count of 10 10Fingers on Keyboard = max.10 Polyphone  
}//END HOOK
 
 
static void TASK_Midi_Decay0(void *pvParameters){  //Send Note OFFs after a while
   portTickType xLastExecutionTime;
   xLastExecutionTime = xTaskGetTickCount();       // Initialise the xLastExecutionTime variable on task entry
  while( 1 ) {
    vTaskDelayUntil(&xLastExecutionTime, 1 / portTICK_RATE_MS);
    
   if   (MeloDecayTIMER[0][1] != 0) {
	     MeloDecayTIMER[0][1]--;
	  if(MeloDecayTIMER[0][1] == 0) {
		   MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 1, 0); //Port, Chn, Note, Velocity 
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 1, 0);}}  
   if   (MeloDecayTIMER[0][2] != 0) {
	     MeloDecayTIMER[0][2]--;
	  if(MeloDecayTIMER[0][2] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 2, 0); //Port, Chn, Note, Velocity 
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 2, 0);}}   
   if   (MeloDecayTIMER[0][3] != 0) {
	     MeloDecayTIMER[0][3]--;
	  if(MeloDecayTIMER[0][3] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 3, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 3, 0);}}
   if   (MeloDecayTIMER[0][4] != 0) {
	     MeloDecayTIMER[0][4]--;
	  if(MeloDecayTIMER[0][4] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 4, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 4, 0);}} 
   if   (MeloDecayTIMER[0][5] != 0) {
	     MeloDecayTIMER[0][5]--;
	  if(MeloDecayTIMER[0][5] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 5, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 5, 0);}} 
   if   (MeloDecayTIMER[0][6] != 0) {
	     MeloDecayTIMER[0][6]--;
	  if(MeloDecayTIMER[0][6] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 6, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 6, 0);}} 
   if   (MeloDecayTIMER[0][7] != 0) {
	     MeloDecayTIMER[0][7]--;
	  if(MeloDecayTIMER[0][7] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 7, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 7, 0);}}   
   if   (MeloDecayTIMER[0][8] != 0) {
	     MeloDecayTIMER[0][8]--;
	  if(MeloDecayTIMER[0][8] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 8, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 8, 0);}}   
   if   (MeloDecayTIMER[0][9] != 0) {
	     MeloDecayTIMER[0][9]--;
	  if(MeloDecayTIMER[0][9] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 9, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 9, 0);}}   
   if   (MeloDecayTIMER[0][10] != 0) {
	     MeloDecayTIMER[0][10]--;
	  if(MeloDecayTIMER[0][10] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 10, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 10, 0);}}   
   if   (MeloDecayTIMER[0][11] != 0) {
	     MeloDecayTIMER[0][11]--;
	  if(MeloDecayTIMER[0][11] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 11, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 11, 0);}}   
   if   (MeloDecayTIMER[0][12] != 0) {
	     MeloDecayTIMER[0][12]--;
	  if(MeloDecayTIMER[0][12] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 12, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 12, 0);}}   
   if   (MeloDecayTIMER[0][13] != 0) {
	     MeloDecayTIMER[0][13]--;
	  if(MeloDecayTIMER[0][13] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 13, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 13, 0);}}   
   if   (MeloDecayTIMER[0][14] != 0) {
	     MeloDecayTIMER[0][14]--;
	  if(MeloDecayTIMER[0][14] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 14, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 14, 0);}}   
   if   (MeloDecayTIMER[0][15] != 0) {
	     MeloDecayTIMER[0][15]--;
	  if(MeloDecayTIMER[0][15] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 15, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 15, 0);}}   
   if   (MeloDecayTIMER[0][16] != 0) {
	     MeloDecayTIMER[0][16]--;
	  if(MeloDecayTIMER[0][16] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 16, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 16, 0);}}   
   if   (MeloDecayTIMER[0][17] != 0) {
	     MeloDecayTIMER[0][17]--;
	  if(MeloDecayTIMER[0][17] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 17, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 17, 0);}}   
   if   (MeloDecayTIMER[0][18] != 0) {
	     MeloDecayTIMER[0][18]--;
	  if(MeloDecayTIMER[0][18] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 18, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 18, 0);}}   
   if   (MeloDecayTIMER[0][19] != 0) {
	     MeloDecayTIMER[0][19]--;
	  if(MeloDecayTIMER[0][19] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 19, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 19, 0);}}   
   if   (MeloDecayTIMER[0][20] != 0) {
	     MeloDecayTIMER[0][20]--;
	  if(MeloDecayTIMER[0][20] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 20, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 20, 0);}}   
   if   (MeloDecayTIMER[0][21] != 0) {
	     MeloDecayTIMER[0][21]--;
	  if(MeloDecayTIMER[0][21] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 21, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 21, 0);}}   
   if   (MeloDecayTIMER[0][22] != 0) {
	     MeloDecayTIMER[0][22]--;
	  if(MeloDecayTIMER[0][22] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 22, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 22, 0);}}   
   if   (MeloDecayTIMER[0][23] != 0) {
	     MeloDecayTIMER[0][23]--;
	  if(MeloDecayTIMER[0][23] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 23, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 23, 0);}}   
   if   (MeloDecayTIMER[0][24] != 0) {
	     MeloDecayTIMER[0][24]--;
	  if(MeloDecayTIMER[0][24] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 24, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 24, 0);}}   
   if   (MeloDecayTIMER[0][25] != 0) {
	     MeloDecayTIMER[0][25]--;
	  if(MeloDecayTIMER[0][25] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 25, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 25, 0);}}   
   if   (MeloDecayTIMER[0][26] != 0) {
	     MeloDecayTIMER[0][26]--;
	  if(MeloDecayTIMER[0][26] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 26, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 26, 0);}}   
   if   (MeloDecayTIMER[0][27] != 0) {
	     MeloDecayTIMER[0][27]--;
	  if(MeloDecayTIMER[0][27] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 27, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 27, 0);}}   
   if   (MeloDecayTIMER[0][28] != 0) {
	     MeloDecayTIMER[0][28]--;
	  if(MeloDecayTIMER[0][28] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 28, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 28, 0);}}   
   if   (MeloDecayTIMER[0][29] != 0) {
	     MeloDecayTIMER[0][29]--;
	  if(MeloDecayTIMER[0][29] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 29, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 29, 0);}}   
   if   (MeloDecayTIMER[0][30] != 0) {
	     MeloDecayTIMER[0][30]--;
	  if(MeloDecayTIMER[0][30] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 30, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 30, 0);}}   
   if   (MeloDecayTIMER[0][31] != 0) {
	     MeloDecayTIMER[0][31]--;
	  if(MeloDecayTIMER[0][31] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 31, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 31, 0);}}   
   if   (MeloDecayTIMER[0][32] != 0) {
	     MeloDecayTIMER[0][32]--;
	  if(MeloDecayTIMER[0][32] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 32, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 32, 0);}}   
   if   (MeloDecayTIMER[0][33] != 0) {
	     MeloDecayTIMER[0][33]--;
	  if(MeloDecayTIMER[0][33] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 33, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 33, 0);}}   
   if   (MeloDecayTIMER[0][34] != 0) {
	     MeloDecayTIMER[0][34]--;
	  if(MeloDecayTIMER[0][34] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 34, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 34, 0);}}   
   if   (MeloDecayTIMER[0][35] != 0) {
	     MeloDecayTIMER[0][35]--;
	  if(MeloDecayTIMER[0][35] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 35, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 35, 0);}}   
   if   (MeloDecayTIMER[0][36] != 0) {
	     MeloDecayTIMER[0][36]--;
	  if(MeloDecayTIMER[0][36] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 36, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 36, 0);}}   
   if   (MeloDecayTIMER[0][37] != 0) {
	     MeloDecayTIMER[0][37]--;
	  if(MeloDecayTIMER[0][37] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 37, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 37, 0);}}   
   if   (MeloDecayTIMER[0][38] != 0) {
	     MeloDecayTIMER[0][38]--;
	  if(MeloDecayTIMER[0][38] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 38, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 38, 0);}}         
   if   (MeloDecayTIMER[0][39] != 0) {
	     MeloDecayTIMER[0][39]--;
	  if(MeloDecayTIMER[0][39] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 39, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 39, 0);}}              
   if   (MeloDecayTIMER[0][40] != 0) {
	     MeloDecayTIMER[0][40]--;
	  if(MeloDecayTIMER[0][40] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 40, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 40, 0);}}              
   if   (MeloDecayTIMER[0][41] != 0) {
	     MeloDecayTIMER[0][41]--;
	  if(MeloDecayTIMER[0][41] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 41, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 41, 0);}}              
   if   (MeloDecayTIMER[0][42] != 0) {
	     MeloDecayTIMER[0][42]--;
	  if(MeloDecayTIMER[0][42] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 42, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 42, 0);}}             
   if   (MeloDecayTIMER[0][43] != 0) {
	     MeloDecayTIMER[0][43]--;
	  if(MeloDecayTIMER[0][43] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 43, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 43, 0);}}              
   if   (MeloDecayTIMER[0][44] != 0) {
	     MeloDecayTIMER[0][44]--;
	  if(MeloDecayTIMER[0][44] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 44, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 44, 0);}}              
   if   (MeloDecayTIMER[0][45] != 0) {
	     MeloDecayTIMER[0][45]--;
	  if(MeloDecayTIMER[0][45] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 45, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 45, 0);}}              
   if   (MeloDecayTIMER[0][46] != 0) {
	     MeloDecayTIMER[0][46]--;
	  if(MeloDecayTIMER[0][46] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 46, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 46, 0);}}              
   if   (MeloDecayTIMER[0][47] != 0) {
	     MeloDecayTIMER[0][47]--;
	  if(MeloDecayTIMER[0][47] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 47, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 47, 0);}}              
   if   (MeloDecayTIMER[0][48] != 0) {
	     MeloDecayTIMER[0][48]--;
	  if(MeloDecayTIMER[0][48] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 48, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 48, 0);}}              
   if   (MeloDecayTIMER[0][49] != 0) {
	     MeloDecayTIMER[0][49]--;
	  if(MeloDecayTIMER[0][49] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 49, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 49, 0);}}              
   if   (MeloDecayTIMER[0][50] != 0) {
	     MeloDecayTIMER[0][50]--;
	  if(MeloDecayTIMER[0][50] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 50, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 50, 0);}}              
   if   (MeloDecayTIMER[0][51] != 0) {
	     MeloDecayTIMER[0][51]--;
	  if(MeloDecayTIMER[0][51] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 51, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 51, 0);}}              
   if   (MeloDecayTIMER[0][52] != 0) {
	     MeloDecayTIMER[0][52]--;
	  if(MeloDecayTIMER[0][52] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 52, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 52, 0);}}    
   if   (MeloDecayTIMER[0][53] != 0) {
	     MeloDecayTIMER[0][53]--;
	  if(MeloDecayTIMER[0][53] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 53, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 53, 0);}}              
   if   (MeloDecayTIMER[0][54] != 0) {
	     MeloDecayTIMER[0][54]--;
	  if(MeloDecayTIMER[0][54] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 54, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 54, 0);}}              
   if   (MeloDecayTIMER[0][55] != 0) {
	     MeloDecayTIMER[0][55]--;
	  if(MeloDecayTIMER[0][55] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 55, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 55, 0);}}              
   if   (MeloDecayTIMER[0][56] != 0) {
	     MeloDecayTIMER[0][56]--;
	  if(MeloDecayTIMER[0][56] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 56, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 56, 0);}}              
   if   (MeloDecayTIMER[0][57] != 0) {
	     MeloDecayTIMER[0][57]--;
	  if(MeloDecayTIMER[0][57] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 57, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 57, 0);}}              
   if   (MeloDecayTIMER[0][58] != 0) {
	     MeloDecayTIMER[0][58]--;
	  if(MeloDecayTIMER[0][58] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 58, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 58, 0);}}              
   if   (MeloDecayTIMER[0][59] != 0) {
	     MeloDecayTIMER[0][59]--;
	  if(MeloDecayTIMER[0][59] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 59, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 59, 0);}}              
   if   (MeloDecayTIMER[0][60] != 0) {
	     MeloDecayTIMER[0][60]--;
	  if(MeloDecayTIMER[0][60] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 60, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 60, 0);}}              
   if   (MeloDecayTIMER[0][61] != 0) {
	     MeloDecayTIMER[0][61]--;
	  if(MeloDecayTIMER[0][61] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 61, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 61, 0);}}              
   if   (MeloDecayTIMER[0][62] != 0) {
	     MeloDecayTIMER[0][62]--;
	  if(MeloDecayTIMER[0][62] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 62, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 62, 0);}}              
   if   (MeloDecayTIMER[0][63] != 0) {
	     MeloDecayTIMER[0][63]--;
	  if(MeloDecayTIMER[0][63] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 63, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 63, 0);}}              
   if   (MeloDecayTIMER[0][64] != 0) {
	     MeloDecayTIMER[0][64]--;
	  if(MeloDecayTIMER[0][64] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 64, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 64, 0);}}              
   if   (MeloDecayTIMER[0][65] != 0) {
	     MeloDecayTIMER[0][65]--;
	  if(MeloDecayTIMER[0][65] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 65, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 65, 0);}}              
   if   (MeloDecayTIMER[0][66] != 0) {
	     MeloDecayTIMER[0][66]--;
	  if(MeloDecayTIMER[0][66] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 66, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 66, 0);}}              
   if   (MeloDecayTIMER[0][67] != 0) {
	     MeloDecayTIMER[0][67]--;
	  if(MeloDecayTIMER[0][67] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 67, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 67, 0);}}              
   if   (MeloDecayTIMER[0][68] != 0) {
	     MeloDecayTIMER[0][68]--;
	  if(MeloDecayTIMER[0][68] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 68, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 68, 0);}}              
   if   (MeloDecayTIMER[0][69] != 0) {
	     MeloDecayTIMER[0][69]--;
	  if(MeloDecayTIMER[0][69] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 69, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 69, 0);}}              
   if   (MeloDecayTIMER[0][70] != 0) {
	     MeloDecayTIMER[0][70]--;
	  if(MeloDecayTIMER[0][70] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 70, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 70, 0);}}              
   if   (MeloDecayTIMER[0][71] != 0) {
	     MeloDecayTIMER[0][71]--;
	  if(MeloDecayTIMER[0][71] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 71, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 71, 0);}}              
   if   (MeloDecayTIMER[0][72] != 0) {
	     MeloDecayTIMER[0][72]--;
	  if(MeloDecayTIMER[0][72] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 72, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 72, 0);}}              
   if   (MeloDecayTIMER[0][73] != 0) {
	     MeloDecayTIMER[0][73]--;
	  if(MeloDecayTIMER[0][73] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 73, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 73, 0);}}              
   if   (MeloDecayTIMER[0][74] != 0) {
	     MeloDecayTIMER[0][74]--;
	  if(MeloDecayTIMER[0][74] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 74, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 74, 0);}}              
   if   (MeloDecayTIMER[0][75] != 0) {
	     MeloDecayTIMER[0][75]--;
	  if(MeloDecayTIMER[0][75] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 75, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 75, 0);}}              
   if   (MeloDecayTIMER[0][76] != 0) {
	     MeloDecayTIMER[0][76]--;
	  if(MeloDecayTIMER[0][76] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 76, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 76, 0);}}              
   if   (MeloDecayTIMER[0][77] != 0) {
	     MeloDecayTIMER[0][77]--;
	  if(MeloDecayTIMER[0][77] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 77, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 77, 0);}}              
   if   (MeloDecayTIMER[0][78] != 0) {
	     MeloDecayTIMER[0][78]--;
	  if(MeloDecayTIMER[0][78] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 78, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 78, 0);}}              
   if   (MeloDecayTIMER[0][79] != 0) {
	     MeloDecayTIMER[0][79]--;
	  if(MeloDecayTIMER[0][79] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 79, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 79, 0);}}              
   if   (MeloDecayTIMER[0][80] != 0) {
	     MeloDecayTIMER[0][80]--;
	  if(MeloDecayTIMER[0][80] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 80, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 80, 0);}}              
   if   (MeloDecayTIMER[0][81] != 0) {
	     MeloDecayTIMER[0][81]--;
	  if(MeloDecayTIMER[0][81] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 81, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 81, 0);}}              
   if   (MeloDecayTIMER[0][82] != 0) {
	     MeloDecayTIMER[0][82]--;
	  if(MeloDecayTIMER[0][82] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 82, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 82, 0);}}              
   if   (MeloDecayTIMER[0][83] != 0) {
	     MeloDecayTIMER[0][83]--;
	  if(MeloDecayTIMER[0][83] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 83, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 83, 0);}}              
   if   (MeloDecayTIMER[0][84] != 0) {
	     MeloDecayTIMER[0][84]--;
	  if(MeloDecayTIMER[0][84] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 84, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 84, 0);}}              
   if   (MeloDecayTIMER[0][85] != 0) {
	     MeloDecayTIMER[0][85]--;
	  if(MeloDecayTIMER[0][85] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 85, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 85, 0);}}              
   if   (MeloDecayTIMER[0][86] != 0) {
	     MeloDecayTIMER[0][86]--;
	  if(MeloDecayTIMER[0][86] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 86, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 86, 0);}}              
   if   (MeloDecayTIMER[0][87] != 0) {
	     MeloDecayTIMER[0][87]--;
	  if(MeloDecayTIMER[0][87] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 87, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 87, 0);}}              
   if   (MeloDecayTIMER[0][88] != 0) {
	     MeloDecayTIMER[0][88]--;
	  if(MeloDecayTIMER[0][88] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 88, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 88, 0);}}              
   if   (MeloDecayTIMER[0][89] != 0) {
	     MeloDecayTIMER[0][89]--;
	  if(MeloDecayTIMER[0][89] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 89, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 89, 0);}}                    
   if   (MeloDecayTIMER[0][90] != 0) {
	     MeloDecayTIMER[0][90]--;
	  if(MeloDecayTIMER[0][90] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 90, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 90, 0);}}              
   if   (MeloDecayTIMER[0][91] != 0) {
	     MeloDecayTIMER[0][91]--;
	  if(MeloDecayTIMER[0][91] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 91, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 91, 0);}}              
   if   (MeloDecayTIMER[0][92] != 0) {
	     MeloDecayTIMER[0][92]--;
	  if(MeloDecayTIMER[0][92] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 92, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 92, 0);}}              
   if   (MeloDecayTIMER[0][93] != 0) {
	     MeloDecayTIMER[0][93]--;
	  if(MeloDecayTIMER[0][93] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 93, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 93, 0);}}   
   if   (MeloDecayTIMER[0][94] != 0) {
	     MeloDecayTIMER[0][94]--;
	  if(MeloDecayTIMER[0][94] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 94, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 94, 0);}}               
   if   (MeloDecayTIMER[0][95] != 0) {
	     MeloDecayTIMER[0][95]--;
	  if(MeloDecayTIMER[0][95] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 95, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 95, 0);}}              
   if   (MeloDecayTIMER[0][96] != 0) {
	     MeloDecayTIMER[0][96]--;
	  if(MeloDecayTIMER[0][96] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 96, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 96, 0);}}              
   if   (MeloDecayTIMER[0][97] != 0) {
	     MeloDecayTIMER[0][97]--;
	  if(MeloDecayTIMER[0][97] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 97, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 97, 0);}}              
   if   (MeloDecayTIMER[0][98] != 0) {
	     MeloDecayTIMER[0][98]--;
	  if(MeloDecayTIMER[0][98] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 98, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 98, 0);}}              
   if   (MeloDecayTIMER[0][99] != 0) {
	     MeloDecayTIMER[0][99]--;
	  if(MeloDecayTIMER[0][99] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 99, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 99, 0);}}              
   if   (MeloDecayTIMER[0][100] != 0) {
	     MeloDecayTIMER[0][100]--;
	  if(MeloDecayTIMER[0][100] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 100, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 100, 0);}}              
   if   (MeloDecayTIMER[0][101] != 0) {
	     MeloDecayTIMER[0][101]--;
	  if(MeloDecayTIMER[0][101] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 101, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 101, 0);}}              
   if   (MeloDecayTIMER[0][102] != 0) {
	     MeloDecayTIMER[0][102]--;
	  if(MeloDecayTIMER[0][102] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 102, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 102, 0);}}              
   if   (MeloDecayTIMER[0][103] != 0) {
	     MeloDecayTIMER[0][103]--;
	  if(MeloDecayTIMER[0][103] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 103, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 103, 0);}}              
   if   (MeloDecayTIMER[0][104] != 0) {
	     MeloDecayTIMER[0][104]--;
	  if(MeloDecayTIMER[0][104] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 104, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 104, 0);}}              
   if   (MeloDecayTIMER[0][105] != 0) {
	     MeloDecayTIMER[0][105]--;
	  if(MeloDecayTIMER[0][105] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 105, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 105, 0);}}              
   if   (MeloDecayTIMER[0][106] != 0) {
	     MeloDecayTIMER[0][106]--;
	  if(MeloDecayTIMER[0][106] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 106, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 106, 0);}}              
   if   (MeloDecayTIMER[0][107] != 0) {
	     MeloDecayTIMER[0][107]--;
	  if(MeloDecayTIMER[0][107] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 107, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 107, 0);}}              
   if   (MeloDecayTIMER[0][108] != 0) {
	     MeloDecayTIMER[0][108]--;
	  if(MeloDecayTIMER[0][108] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 108, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 108, 0);}}              
   if   (MeloDecayTIMER[0][109] != 0) {
	     MeloDecayTIMER[0][109]--;
	  if(MeloDecayTIMER[0][109] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 109, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 109, 0);}}              
   if   (MeloDecayTIMER[0][110] != 0) {
	     MeloDecayTIMER[0][110]--;
	  if(MeloDecayTIMER[0][110] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 110, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 110, 0);}}              
   if   (MeloDecayTIMER[0][111] != 0) {
	     MeloDecayTIMER[0][111]--;
	  if(MeloDecayTIMER[0][111] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 111, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 111, 0);}}              
   if   (MeloDecayTIMER[0][112] != 0) {
	     MeloDecayTIMER[0][112]--;
	  if(MeloDecayTIMER[0][112] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 112, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 112, 0);}}              
   if   (MeloDecayTIMER[0][113] != 0) {
	     MeloDecayTIMER[0][113]--;
	  if(MeloDecayTIMER[0][113] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 113, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 113, 0);}}              
   if   (MeloDecayTIMER[0][114] != 0) {
	     MeloDecayTIMER[0][114]--;
	  if(MeloDecayTIMER[0][114] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 114, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 114, 0);}}              
   if   (MeloDecayTIMER[0][115] != 0) {
	     MeloDecayTIMER[0][115]--;
	  if(MeloDecayTIMER[0][115] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 115, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 115, 0);}}              
   if   (MeloDecayTIMER[0][116] != 0) {
	     MeloDecayTIMER[0][116]--;
	  if(MeloDecayTIMER[0][116] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 116, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 116, 0);}}              
   if   (MeloDecayTIMER[0][117] != 0) {
	     MeloDecayTIMER[0][117]--;
	  if(MeloDecayTIMER[0][117] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 117, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 117, 0);}}              
   if   (MeloDecayTIMER[0][118] != 0) {
	     MeloDecayTIMER[0][118]--;
	  if(MeloDecayTIMER[0][118] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 118, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 118, 0);}}              
   if   (MeloDecayTIMER[0][119] != 0) {
	     MeloDecayTIMER[0][119]--;
	  if(MeloDecayTIMER[0][119] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 119, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 119, 0);}}              
   if   (MeloDecayTIMER[0][120] != 0) {
	     MeloDecayTIMER[0][120]--;
	  if(MeloDecayTIMER[0][120] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 120, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 120, 0);}}              
   if   (MeloDecayTIMER[0][121] != 0) {
	     MeloDecayTIMER[0][121]--;
	  if(MeloDecayTIMER[0][121] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 121, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 121, 0);}}              
   if   (MeloDecayTIMER[0][122] != 0) {
	     MeloDecayTIMER[0][122]--;
	  if(MeloDecayTIMER[0][122] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 122, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 122, 0);}}              
   if   (MeloDecayTIMER[0][123] != 0) {
	     MeloDecayTIMER[0][123]--;
	  if(MeloDecayTIMER[0][23] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 123, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 123, 0);}}              
   if   (MeloDecayTIMER[0][124] != 0) {
	     MeloDecayTIMER[0][124]--;
	  if(MeloDecayTIMER[0][124] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 124, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 124, 0);}}              
   if   (MeloDecayTIMER[0][125] != 0) {
	     MeloDecayTIMER[0][125]--;
	  if(MeloDecayTIMER[0][125] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 125, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 125, 0);}}              
   if   (MeloDecayTIMER[0][126] != 0) {
	     MeloDecayTIMER[0][126]--;
	  if(MeloDecayTIMER[0][126] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 126, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 126, 0);}}              
   if   (MeloDecayTIMER[0][127] != 0) {
	     MeloDecayTIMER[0][127]--;
	  if(MeloDecayTIMER[0][127] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, 0, 127, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, 0, 127, 0);}}                                       
}//End While
}//End TICK 0 >>> 1ms Task...
static void TASK_Midi_Decay1(void *pvParameters){  //Send Note OFFs after a while
   portTickType xLastExecutionTime;
   xLastExecutionTime = xTaskGetTickCount();       // Initialise the xLastExecutionTime variable on task entry

  while( 1 ) {
    vTaskDelayUntil(&xLastExecutionTime, 1 / portTICK_RATE_MS);
    
   if   (MeloDecayTIMER[1][1] != 0) {
	     MeloDecayTIMER[1][1]--;
	  if(MeloDecayTIMER[1][1] == 0) {
		   MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 1, 0); //Port, Chn, Note, Velocity 
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 1, 0);}}  
   if   (MeloDecayTIMER[1][2] != 0) {
	     MeloDecayTIMER[1][2]--;
	  if(MeloDecayTIMER[1][2] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 2, 0); //Port, Chn, Note, Velocity 
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 2, 0);}}   
   if   (MeloDecayTIMER[1][3] != 0) {
	     MeloDecayTIMER[1][3]--;
	  if(MeloDecayTIMER[1][3] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 3, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 3, 0);}}
   if   (MeloDecayTIMER[1][4] != 0) {
	     MeloDecayTIMER[1][4]--;
	  if(MeloDecayTIMER[1][4] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 4, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 4, 0);}} 
   if   (MeloDecayTIMER[1][5] != 0) {
	     MeloDecayTIMER[1][5]--;
	  if(MeloDecayTIMER[1][5] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 5, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 5, 0);}} 
   if   (MeloDecayTIMER[1][6] != 0) {
	     MeloDecayTIMER[1][6]--;
	  if(MeloDecayTIMER[1][6] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 6, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 6, 0);}} 
   if   (MeloDecayTIMER[1][7] != 0) {
	     MeloDecayTIMER[1][7]--;
	  if(MeloDecayTIMER[1][7] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 7, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 7, 0);}}   
   if   (MeloDecayTIMER[1][8] != 0) {
	     MeloDecayTIMER[1][8]--;
	  if(MeloDecayTIMER[1][8] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 8, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 8, 0);}}   
   if   (MeloDecayTIMER[1][9] != 0) {
	     MeloDecayTIMER[1][9]--;
	  if(MeloDecayTIMER[1][9] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 9, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 9, 0);}}   
   if   (MeloDecayTIMER[1][10] != 0) {
	     MeloDecayTIMER[1][10]--;
	  if(MeloDecayTIMER[1][10] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 10, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 10, 0);}}   
   if   (MeloDecayTIMER[1][11] != 0) {
	     MeloDecayTIMER[1][11]--;
	  if(MeloDecayTIMER[1][11] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 11, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 11, 0);}}   
   if   (MeloDecayTIMER[1][12] != 0) {
	     MeloDecayTIMER[1][12]--;
	  if(MeloDecayTIMER[1][12] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 12, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 12, 0);}}   
   if   (MeloDecayTIMER[1][13] != 0) {
	     MeloDecayTIMER[1][13]--;
	  if(MeloDecayTIMER[1][13] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 13, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 13, 0);}}   
   if   (MeloDecayTIMER[1][14] != 0) {
	     MeloDecayTIMER[1][14]--;
	  if(MeloDecayTIMER[1][14] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 14, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 14, 0);}}   
   if   (MeloDecayTIMER[1][15] != 0) {
	     MeloDecayTIMER[1][15]--;
	  if(MeloDecayTIMER[1][15] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 15, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 15, 0);}}   
   if   (MeloDecayTIMER[1][16] != 0) {
	     MeloDecayTIMER[1][16]--;
	  if(MeloDecayTIMER[1][16] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 16, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 16, 0);}}   
   if   (MeloDecayTIMER[1][17] != 0) {
	     MeloDecayTIMER[1][17]--;
	  if(MeloDecayTIMER[1][17] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 17, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 17, 0);}}   
   if   (MeloDecayTIMER[1][18] != 0) {
	     MeloDecayTIMER[1][18]--;
	  if(MeloDecayTIMER[1][18] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 18, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 18, 0);}}   
   if   (MeloDecayTIMER[1][19] != 0) {
	     MeloDecayTIMER[1][19]--;
	  if(MeloDecayTIMER[1][19] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 19, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 19, 0);}}   
   if   (MeloDecayTIMER[1][20] != 0) {
	     MeloDecayTIMER[1][20]--;
	  if(MeloDecayTIMER[1][20] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 20, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 20, 0);}}   
   if   (MeloDecayTIMER[1][21] != 0) {
	     MeloDecayTIMER[1][21]--;
	  if(MeloDecayTIMER[1][21] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 21, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 21, 0);}}   
   if   (MeloDecayTIMER[1][22] != 0) {
	     MeloDecayTIMER[1][22]--;
	  if(MeloDecayTIMER[1][22] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 22, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 22, 0);}}   
   if   (MeloDecayTIMER[1][23] != 0) {
	     MeloDecayTIMER[1][23]--;
	  if(MeloDecayTIMER[1][23] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 23, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 23, 0);}}   
   if   (MeloDecayTIMER[1][24] != 0) {
	     MeloDecayTIMER[1][24]--;
	  if(MeloDecayTIMER[1][24] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 24, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 24, 0);}}   
   if   (MeloDecayTIMER[1][25] != 0) {
	     MeloDecayTIMER[1][25]--;
	  if(MeloDecayTIMER[1][25] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 25, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 25, 0);}}   
   if   (MeloDecayTIMER[1][26] != 0) {
	     MeloDecayTIMER[1][26]--;
	  if(MeloDecayTIMER[1][26] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 26, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 26, 0);}}   
   if   (MeloDecayTIMER[1][27] != 0) {
	     MeloDecayTIMER[1][27]--;
	  if(MeloDecayTIMER[1][27] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 27, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 27, 0);}}   
   if   (MeloDecayTIMER[1][28] != 0) {
	     MeloDecayTIMER[1][28]--;
	  if(MeloDecayTIMER[1][28] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 28, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 28, 0);}}   
   if   (MeloDecayTIMER[1][29] != 0) {
	     MeloDecayTIMER[1][29]--;
	  if(MeloDecayTIMER[1][29] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 29, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 29, 0);}}   
   if   (MeloDecayTIMER[1][30] != 0) {
	     MeloDecayTIMER[1][30]--;
	  if(MeloDecayTIMER[1][30] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 30, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 30, 0);}}   
   if   (MeloDecayTIMER[1][31] != 0) {
	     MeloDecayTIMER[1][31]--;
	  if(MeloDecayTIMER[1][31] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 31, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 31, 0);}}   
   if   (MeloDecayTIMER[1][32] != 0) {
	     MeloDecayTIMER[1][32]--;
	  if(MeloDecayTIMER[1][32] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 32, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 32, 0);}}   
   if   (MeloDecayTIMER[1][33] != 0) {
	     MeloDecayTIMER[1][33]--;
	  if(MeloDecayTIMER[1][33] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 33, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 33, 0);}}   
   if   (MeloDecayTIMER[1][34] != 0) {
	     MeloDecayTIMER[1][34]--;
	  if(MeloDecayTIMER[1][34] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 34, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 34, 0);}}   
   if   (MeloDecayTIMER[1][35] != 0) {
	     MeloDecayTIMER[1][35]--;
	  if(MeloDecayTIMER[1][35] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 35, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 35, 0);}}   
   if   (MeloDecayTIMER[1][36] != 0) {
	     MeloDecayTIMER[1][36]--;
	  if(MeloDecayTIMER[1][36] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 36, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 36, 0);}}   
   if   (MeloDecayTIMER[1][37] != 0) {
	     MeloDecayTIMER[1][37]--;
	  if(MeloDecayTIMER[1][37] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 37, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 37, 0);}}   
   if   (MeloDecayTIMER[1][38] != 0) {
	     MeloDecayTIMER[1][38]--;
	  if(MeloDecayTIMER[1][38] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 38, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 38, 0);}}         
   if   (MeloDecayTIMER[1][39] != 0) {
	     MeloDecayTIMER[1][39]--;
	  if(MeloDecayTIMER[1][39] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 39, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 39, 0);}}              
   if   (MeloDecayTIMER[1][40] != 0) {
	     MeloDecayTIMER[1][40]--;
	  if(MeloDecayTIMER[1][40] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 40, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 40, 0);}}              
   if   (MeloDecayTIMER[1][41] != 0) {
	     MeloDecayTIMER[1][41]--;
	  if(MeloDecayTIMER[1][41] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 41, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 41, 0);}}              
   if   (MeloDecayTIMER[1][42] != 0) {
	     MeloDecayTIMER[1][42]--;
	  if(MeloDecayTIMER[1][42] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 42, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 42, 0);}}             
   if   (MeloDecayTIMER[1][43] != 0) {
	     MeloDecayTIMER[1][43]--;
	  if(MeloDecayTIMER[1][43] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 43, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 43, 0);}}              
   if   (MeloDecayTIMER[1][44] != 0) {
	     MeloDecayTIMER[1][44]--;
	  if(MeloDecayTIMER[1][44] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 44, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 44, 0);}}              
   if   (MeloDecayTIMER[1][45] != 0) {
	     MeloDecayTIMER[1][45]--;
	  if(MeloDecayTIMER[1][45] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 45, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 45, 0);}}              
   if   (MeloDecayTIMER[1][46] != 0) {
	     MeloDecayTIMER[1][46]--;
	  if(MeloDecayTIMER[1][46] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 46, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 46, 0);}}              
   if   (MeloDecayTIMER[1][47] != 0) {
	     MeloDecayTIMER[1][47]--;
	  if(MeloDecayTIMER[1][47] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 47, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 47, 0);}}              
   if   (MeloDecayTIMER[1][48] != 0) {
	     MeloDecayTIMER[1][48]--;
	  if(MeloDecayTIMER[1][48] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 48, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 48, 0);}}              
   if   (MeloDecayTIMER[1][49] != 0) {
	     MeloDecayTIMER[1][49]--;
	  if(MeloDecayTIMER[1][49] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 49, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 49, 0);}}              
   if   (MeloDecayTIMER[1][50] != 0) {
	     MeloDecayTIMER[1][50]--;
	  if(MeloDecayTIMER[1][50] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 50, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 50, 0);}}              
   if   (MeloDecayTIMER[1][51] != 0) {
	     MeloDecayTIMER[1][51]--;
	  if(MeloDecayTIMER[1][51] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 51, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 51, 0);}}              
   if   (MeloDecayTIMER[1][52] != 0) {
	     MeloDecayTIMER[1][52]--;
	  if(MeloDecayTIMER[1][52] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 52, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 52, 0);}}    
   if   (MeloDecayTIMER[1][53] != 0) {
	     MeloDecayTIMER[1][53]--;
	  if(MeloDecayTIMER[1][53] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 53, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 53, 0);}}              
   if   (MeloDecayTIMER[1][54] != 0) {
	     MeloDecayTIMER[1][54]--;
	  if(MeloDecayTIMER[1][54] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 54, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 54, 0);}}              
   if   (MeloDecayTIMER[1][55] != 0) {
	     MeloDecayTIMER[1][55]--;
	  if(MeloDecayTIMER[1][55] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 55, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 55, 0);}}              
   if   (MeloDecayTIMER[1][56] != 0) {
	     MeloDecayTIMER[1][56]--;
	  if(MeloDecayTIMER[1][56] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 56, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 56, 0);}}              
   if   (MeloDecayTIMER[1][57] != 0) {
	     MeloDecayTIMER[1][57]--;
	  if(MeloDecayTIMER[1][57] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 57, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 57, 0);}}              
   if   (MeloDecayTIMER[1][58] != 0) {
	     MeloDecayTIMER[1][58]--;
	  if(MeloDecayTIMER[1][58] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 58, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 58, 0);}}              
   if   (MeloDecayTIMER[1][59] != 0) {
	     MeloDecayTIMER[1][59]--;
	  if(MeloDecayTIMER[1][59] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 59, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 59, 0);}}              
   if   (MeloDecayTIMER[1][60] != 0) {
	     MeloDecayTIMER[1][60]--;
	  if(MeloDecayTIMER[1][60] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 60, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 60, 0);}}              
   if   (MeloDecayTIMER[1][61] != 0) {
	     MeloDecayTIMER[1][61]--;
	  if(MeloDecayTIMER[1][61] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 61, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 61, 0);}}              
   if   (MeloDecayTIMER[1][62] != 0) {
	     MeloDecayTIMER[1][62]--;
	  if(MeloDecayTIMER[1][62] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 62, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 62, 0);}}              
   if   (MeloDecayTIMER[1][63] != 0) {
	     MeloDecayTIMER[1][63]--;
	  if(MeloDecayTIMER[1][63] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 63, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 63, 0);}}              
   if   (MeloDecayTIMER[1][64] != 0) {
	     MeloDecayTIMER[1][64]--;
	  if(MeloDecayTIMER[1][64] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 64, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 64, 0);}}              
   if   (MeloDecayTIMER[1][65] != 0) {
	     MeloDecayTIMER[1][65]--;
	  if(MeloDecayTIMER[1][65] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 65, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 65, 0);}}              
   if   (MeloDecayTIMER[1][66] != 0) {
	     MeloDecayTIMER[1][66]--;
	  if(MeloDecayTIMER[1][66] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 66, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 66, 0);}}              
   if   (MeloDecayTIMER[1][67] != 0) {
	     MeloDecayTIMER[1][67]--;
	  if(MeloDecayTIMER[1][67] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 67, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 67, 0);}}              
   if   (MeloDecayTIMER[1][68] != 0) {
	     MeloDecayTIMER[1][68]--;
	  if(MeloDecayTIMER[1][68] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 68, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 68, 0);}}              
   if   (MeloDecayTIMER[1][69] != 0) {
	     MeloDecayTIMER[1][69]--;
	  if(MeloDecayTIMER[1][69] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 69, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 69, 0);}}              
   if   (MeloDecayTIMER[1][70] != 0) {
	     MeloDecayTIMER[1][70]--;
	  if(MeloDecayTIMER[1][70] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 70, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 70, 0);}}              
   if   (MeloDecayTIMER[1][71] != 0) {
	     MeloDecayTIMER[1][71]--;
	  if(MeloDecayTIMER[1][71] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 71, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 71, 0);}}              
   if   (MeloDecayTIMER[1][72] != 0) {
	     MeloDecayTIMER[1][72]--;
	  if(MeloDecayTIMER[1][72] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 72, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 72, 0);}}              
   if   (MeloDecayTIMER[1][73] != 0) {
	     MeloDecayTIMER[1][73]--;
	  if(MeloDecayTIMER[1][73] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 73, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 73, 0);}}              
   if   (MeloDecayTIMER[1][74] != 0) {
	     MeloDecayTIMER[1][74]--;
	  if(MeloDecayTIMER[1][74] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 74, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 74, 0);}}              
   if   (MeloDecayTIMER[1][75] != 0) {
	     MeloDecayTIMER[1][75]--;
	  if(MeloDecayTIMER[1][75] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 75, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 75, 0);}}              
   if   (MeloDecayTIMER[1][76] != 0) {
	     MeloDecayTIMER[1][76]--;
	  if(MeloDecayTIMER[1][76] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 76, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 76, 0);}}              
   if   (MeloDecayTIMER[1][77] != 0) {
	     MeloDecayTIMER[1][77]--;
	  if(MeloDecayTIMER[1][77] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 77, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 77, 0);}}              
   if   (MeloDecayTIMER[1][78] != 0) {
	     MeloDecayTIMER[1][78]--;
	  if(MeloDecayTIMER[1][78] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 78, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 78, 0);}}              
   if   (MeloDecayTIMER[1][79] != 0) {
	     MeloDecayTIMER[1][79]--;
	  if(MeloDecayTIMER[1][79] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 79, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 79, 0);}}              
   if   (MeloDecayTIMER[1][80] != 0) {
	     MeloDecayTIMER[1][80]--;
	  if(MeloDecayTIMER[1][80] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 80, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 80, 0);}}              
   if   (MeloDecayTIMER[1][81] != 0) {
	     MeloDecayTIMER[1][81]--;
	  if(MeloDecayTIMER[1][81] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 81, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 81, 0);}}              
   if   (MeloDecayTIMER[1][82] != 0) {
	     MeloDecayTIMER[1][82]--;
	  if(MeloDecayTIMER[1][82] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 82, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 82, 0);}}              
   if   (MeloDecayTIMER[1][83] != 0) {
	     MeloDecayTIMER[1][83]--;
	  if(MeloDecayTIMER[1][83] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 83, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 83, 0);}}              
   if   (MeloDecayTIMER[1][84] != 0) {
	     MeloDecayTIMER[1][84]--;
	  if(MeloDecayTIMER[1][84] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 84, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 84, 0);}}              
   if   (MeloDecayTIMER[1][85] != 0) {
	     MeloDecayTIMER[1][85]--;
	  if(MeloDecayTIMER[1][85] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 85, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 85, 0);}}              
   if   (MeloDecayTIMER[1][86] != 0) {
	     MeloDecayTIMER[1][86]--;
	  if(MeloDecayTIMER[1][86] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 86, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 86, 0);}}              
   if   (MeloDecayTIMER[1][87] != 0) {
	     MeloDecayTIMER[1][87]--;
	  if(MeloDecayTIMER[1][87] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 87, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 87, 0);}}              
   if   (MeloDecayTIMER[1][88] != 0) {
	     MeloDecayTIMER[1][88]--;
	  if(MeloDecayTIMER[1][88] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 88, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 88, 0);}}              
   if   (MeloDecayTIMER[1][89] != 0) {
	     MeloDecayTIMER[1][89]--;
	  if(MeloDecayTIMER[1][89] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 89, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 89, 0);}}                    
   if   (MeloDecayTIMER[1][90] != 0) {
	     MeloDecayTIMER[1][90]--;
	  if(MeloDecayTIMER[1][90] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 90, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 90, 0);}}              
   if   (MeloDecayTIMER[1][91] != 0) {
	     MeloDecayTIMER[1][91]--;
	  if(MeloDecayTIMER[1][91] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 91, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 91, 0);}}              
   if   (MeloDecayTIMER[1][92] != 0) {
	     MeloDecayTIMER[1][92]--;
	  if(MeloDecayTIMER[1][92] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 92, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 92, 0);}}              
   if   (MeloDecayTIMER[1][93] != 0) {
	     MeloDecayTIMER[1][93]--;
	  if(MeloDecayTIMER[1][93] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 93, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 93, 0);}}   
   if   (MeloDecayTIMER[1][94] != 0) {
	     MeloDecayTIMER[1][94]--;
	  if(MeloDecayTIMER[1][94] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 94, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 94, 0);}}               
   if   (MeloDecayTIMER[1][95] != 0) {
	     MeloDecayTIMER[1][95]--;
	  if(MeloDecayTIMER[1][95] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 95, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 95, 0);}}              
   if   (MeloDecayTIMER[1][96] != 0) {
	     MeloDecayTIMER[1][96]--;
	  if(MeloDecayTIMER[1][96] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 96, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 96, 0);}}              
   if   (MeloDecayTIMER[1][97] != 0) {
	     MeloDecayTIMER[1][97]--;
	  if(MeloDecayTIMER[1][97] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 97, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 97, 0);}}              
   if   (MeloDecayTIMER[1][98] != 0) {
	     MeloDecayTIMER[1][98]--;
	  if(MeloDecayTIMER[1][98] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 98, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 98, 0);}}              
   if   (MeloDecayTIMER[1][99] != 0) {
	     MeloDecayTIMER[1][99]--;
	  if(MeloDecayTIMER[1][99] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 99, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 99, 0);}}              
   if   (MeloDecayTIMER[1][100] != 0) {
	     MeloDecayTIMER[1][100]--;
	  if(MeloDecayTIMER[1][100] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 100, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 100, 0);}}              
   if   (MeloDecayTIMER[1][101] != 0) {
	     MeloDecayTIMER[1][101]--;
	  if(MeloDecayTIMER[1][101] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 101, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 101, 0);}}              
   if   (MeloDecayTIMER[1][102] != 0) {
	     MeloDecayTIMER[1][102]--;
	  if(MeloDecayTIMER[1][102] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 102, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 102, 0);}}              
   if   (MeloDecayTIMER[1][103] != 0) {
	     MeloDecayTIMER[1][103]--;
	  if(MeloDecayTIMER[1][103] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 103, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 103, 0);}}              
   if   (MeloDecayTIMER[1][104] != 0) {
	     MeloDecayTIMER[1][104]--;
	  if(MeloDecayTIMER[1][104] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 104, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 104, 0);}}              
   if   (MeloDecayTIMER[1][105] != 0) {
	     MeloDecayTIMER[1][105]--;
	  if(MeloDecayTIMER[1][105] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 105, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 105, 0);}}              
   if   (MeloDecayTIMER[1][106] != 0) {
	     MeloDecayTIMER[1][106]--;
	  if(MeloDecayTIMER[1][106] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 106, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 106, 0);}}              
   if   (MeloDecayTIMER[1][107] != 0) {
	     MeloDecayTIMER[1][107]--;
	  if(MeloDecayTIMER[1][107] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 107, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 107, 0);}}              
   if   (MeloDecayTIMER[1][108] != 0) {
	     MeloDecayTIMER[1][108]--;
	  if(MeloDecayTIMER[1][108] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 108, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 108, 0);}}              
   if   (MeloDecayTIMER[1][109] != 0) {
	     MeloDecayTIMER[1][109]--;
	  if(MeloDecayTIMER[1][109] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 109, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 109, 0);}}              
   if   (MeloDecayTIMER[1][110] != 0) {
	     MeloDecayTIMER[1][110]--;
	  if(MeloDecayTIMER[1][110] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 110, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 110, 0);}}              
   if   (MeloDecayTIMER[1][111] != 0) {
	     MeloDecayTIMER[1][111]--;
	  if(MeloDecayTIMER[1][111] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 111, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 111, 0);}}              
   if   (MeloDecayTIMER[1][112] != 0) {
	     MeloDecayTIMER[1][112]--;
	  if(MeloDecayTIMER[1][112] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 112, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 112, 0);}}              
   if   (MeloDecayTIMER[1][113] != 0) {
	     MeloDecayTIMER[1][113]--;
	  if(MeloDecayTIMER[1][113] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 113, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 113, 0);}}              
   if   (MeloDecayTIMER[1][114] != 0) {
	     MeloDecayTIMER[1][114]--;
	  if(MeloDecayTIMER[1][114] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 114, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 114, 0);}}              
   if   (MeloDecayTIMER[1][115] != 0) {
	     MeloDecayTIMER[1][115]--;
	  if(MeloDecayTIMER[1][115] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 115, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 115, 0);}}              
   if   (MeloDecayTIMER[1][116] != 0) {
	     MeloDecayTIMER[1][116]--;
	  if(MeloDecayTIMER[1][116] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 116, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 116, 0);}}              
   if   (MeloDecayTIMER[1][117] != 0) {
	     MeloDecayTIMER[1][117]--;
	  if(MeloDecayTIMER[1][117] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 117, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 117, 0);}}              
   if   (MeloDecayTIMER[1][118] != 0) {
	     MeloDecayTIMER[1][118]--;
	  if(MeloDecayTIMER[1][118] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 118, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 118, 0);}}              
   if   (MeloDecayTIMER[1][119] != 0) {
	     MeloDecayTIMER[1][119]--;
	  if(MeloDecayTIMER[1][119] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 119, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 119, 0);}}              
   if   (MeloDecayTIMER[1][120] != 0) {
	     MeloDecayTIMER[1][120]--;
	  if(MeloDecayTIMER[1][120] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 120, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 120, 0);}}              
   if   (MeloDecayTIMER[1][121] != 0) {
	     MeloDecayTIMER[1][121]--;
	  if(MeloDecayTIMER[1][121] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 121, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 121, 0);}}              
   if   (MeloDecayTIMER[1][122] != 0) {
	     MeloDecayTIMER[1][122]--;
	  if(MeloDecayTIMER[1][122] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 122, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 122, 0);}}              
   if   (MeloDecayTIMER[1][123] != 0) {
	     MeloDecayTIMER[1][123]--;
	  if(MeloDecayTIMER[1][23] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 123, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 123, 0);}}              
   if   (MeloDecayTIMER[1][124] != 0) {
	     MeloDecayTIMER[1][124]--;
	  if(MeloDecayTIMER[1][124] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 124, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 124, 0);}}              
   if   (MeloDecayTIMER[1][125] != 0) {
	     MeloDecayTIMER[1][125]--;
	  if(MeloDecayTIMER[1][125] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 125, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 125, 0);}}              
   if   (MeloDecayTIMER[1][126] != 0) {
	     MeloDecayTIMER[1][126]--;
	  if(MeloDecayTIMER[1][126] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 126, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 126, 0);}}              
   if   (MeloDecayTIMER[1][127] != 0) {
	     MeloDecayTIMER[1][127]--;
	  if(MeloDecayTIMER[1][127] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[1], 127, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[1], 127, 0);}}                                       
}//End While
}//End TICK 0 >>> 1ms Task...
static void TASK_Midi_Decay2(void *pvParameters){  //Send Note OFFs after a while
   portTickType xLastExecutionTime;
   xLastExecutionTime = xTaskGetTickCount();       // Initialise the xLastExecutionTime variable on task entry

  while( 1 ) {
    vTaskDelayUntil(&xLastExecutionTime, 1 / portTICK_RATE_MS);
    
   if   (MeloDecayTIMER[2][1] != 0) {
	     MeloDecayTIMER[2][1]--;
	  if(MeloDecayTIMER[2][1] == 0) {
		   MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 1, 0); //Port, Chn, Note, Velocity 
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 1, 0);}}  
   if   (MeloDecayTIMER[2][2] != 0) {
	     MeloDecayTIMER[2][2]--;
	  if(MeloDecayTIMER[2][2] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 2, 0); //Port, Chn, Note, Velocity 
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 2, 0);}}   
   if   (MeloDecayTIMER[2][3] != 0) {
	     MeloDecayTIMER[2][3]--;
	  if(MeloDecayTIMER[2][3] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 3, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 3, 0);}}
   if   (MeloDecayTIMER[2][4] != 0) {
	     MeloDecayTIMER[2][4]--;
	  if(MeloDecayTIMER[2][4] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 4, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 4, 0);}} 
   if   (MeloDecayTIMER[2][5] != 0) {
	     MeloDecayTIMER[2][5]--;
	  if(MeloDecayTIMER[2][5] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 5, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 5, 0);}} 
   if   (MeloDecayTIMER[2][6] != 0) {
	     MeloDecayTIMER[2][6]--;
	  if(MeloDecayTIMER[2][6] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 6, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 6, 0);}} 
   if   (MeloDecayTIMER[2][7] != 0) {
	     MeloDecayTIMER[2][7]--;
	  if(MeloDecayTIMER[2][7] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 7, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 7, 0);}}   
   if   (MeloDecayTIMER[2][8] != 0) {
	     MeloDecayTIMER[2][8]--;
	  if(MeloDecayTIMER[2][8] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 8, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 8, 0);}}   
   if   (MeloDecayTIMER[2][9] != 0) {
	     MeloDecayTIMER[2][9]--;
	  if(MeloDecayTIMER[2][9] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 9, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 9, 0);}}   
   if   (MeloDecayTIMER[2][10] != 0) {
	     MeloDecayTIMER[2][10]--;
	  if(MeloDecayTIMER[2][10] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 10, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 10, 0);}}   
   if   (MeloDecayTIMER[2][11] != 0) {
	     MeloDecayTIMER[2][11]--;
	  if(MeloDecayTIMER[2][11] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 11, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 11, 0);}}   
   if   (MeloDecayTIMER[2][12] != 0) {
	     MeloDecayTIMER[2][12]--;
	  if(MeloDecayTIMER[2][12] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 12, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 12, 0);}}   
   if   (MeloDecayTIMER[2][13] != 0) {
	     MeloDecayTIMER[2][13]--;
	  if(MeloDecayTIMER[2][13] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 13, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 13, 0);}}   
   if   (MeloDecayTIMER[2][14] != 0) {
	     MeloDecayTIMER[2][14]--;
	  if(MeloDecayTIMER[2][14] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 14, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 14, 0);}}   
   if   (MeloDecayTIMER[2][15] != 0) {
	     MeloDecayTIMER[2][15]--;
	  if(MeloDecayTIMER[2][15] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 15, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 15, 0);}}   
   if   (MeloDecayTIMER[2][16] != 0) {
	     MeloDecayTIMER[2][16]--;
	  if(MeloDecayTIMER[2][16] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 16, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 16, 0);}}   
   if   (MeloDecayTIMER[2][17] != 0) {
	     MeloDecayTIMER[2][17]--;
	  if(MeloDecayTIMER[2][17] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 17, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 17, 0);}}   
   if   (MeloDecayTIMER[2][18] != 0) {
	     MeloDecayTIMER[2][18]--;
	  if(MeloDecayTIMER[2][18] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 18, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 18, 0);}}   
   if   (MeloDecayTIMER[2][19] != 0) {
	     MeloDecayTIMER[2][19]--;
	  if(MeloDecayTIMER[2][19] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 19, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 19, 0);}}   
   if   (MeloDecayTIMER[2][20] != 0) {
	     MeloDecayTIMER[2][20]--;
	  if(MeloDecayTIMER[2][20] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 20, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 20, 0);}}   
   if   (MeloDecayTIMER[2][21] != 0) {
	     MeloDecayTIMER[2][21]--;
	  if(MeloDecayTIMER[2][21] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 21, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 21, 0);}}   
   if   (MeloDecayTIMER[2][22] != 0) {
	     MeloDecayTIMER[2][22]--;
	  if(MeloDecayTIMER[2][22] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 22, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 22, 0);}}   
   if   (MeloDecayTIMER[2][23] != 0) {
	     MeloDecayTIMER[2][23]--;
	  if(MeloDecayTIMER[2][23] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 23, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 23, 0);}}   
   if   (MeloDecayTIMER[2][24] != 0) {
	     MeloDecayTIMER[2][24]--;
	  if(MeloDecayTIMER[2][24] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 24, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 24, 0);}}   
   if   (MeloDecayTIMER[2][25] != 0) {
	     MeloDecayTIMER[2][25]--;
	  if(MeloDecayTIMER[2][25] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 25, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 25, 0);}}   
   if   (MeloDecayTIMER[2][26] != 0) {
	     MeloDecayTIMER[2][26]--;
	  if(MeloDecayTIMER[2][26] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 26, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 26, 0);}}   
   if   (MeloDecayTIMER[2][27] != 0) {
	     MeloDecayTIMER[2][27]--;
	  if(MeloDecayTIMER[2][27] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 27, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 27, 0);}}   
   if   (MeloDecayTIMER[2][28] != 0) {
	     MeloDecayTIMER[2][28]--;
	  if(MeloDecayTIMER[2][28] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 28, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 28, 0);}}   
   if   (MeloDecayTIMER[2][29] != 0) {
	     MeloDecayTIMER[2][29]--;
	  if(MeloDecayTIMER[2][29] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 29, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 29, 0);}}   
   if   (MeloDecayTIMER[2][30] != 0) {
	     MeloDecayTIMER[2][30]--;
	  if(MeloDecayTIMER[2][30] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 30, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 30, 0);}}   
   if   (MeloDecayTIMER[2][31] != 0) {
	     MeloDecayTIMER[2][31]--;
	  if(MeloDecayTIMER[2][31] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 31, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 31, 0);}}   
   if   (MeloDecayTIMER[2][32] != 0) {
	     MeloDecayTIMER[2][32]--;
	  if(MeloDecayTIMER[2][32] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 32, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 32, 0);}}   
   if   (MeloDecayTIMER[2][33] != 0) {
	     MeloDecayTIMER[2][33]--;
	  if(MeloDecayTIMER[2][33] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 33, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 33, 0);}}   
   if   (MeloDecayTIMER[2][34] != 0) {
	     MeloDecayTIMER[2][34]--;
	  if(MeloDecayTIMER[2][34] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 34, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 34, 0);}}   
   if   (MeloDecayTIMER[2][35] != 0) {
	     MeloDecayTIMER[2][35]--;
	  if(MeloDecayTIMER[2][35] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 35, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 35, 0);}}   
   if   (MeloDecayTIMER[2][36] != 0) {
	     MeloDecayTIMER[2][36]--;
	  if(MeloDecayTIMER[2][36] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 36, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 36, 0);}}   
   if   (MeloDecayTIMER[2][37] != 0) {
	     MeloDecayTIMER[2][37]--;
	  if(MeloDecayTIMER[2][37] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 37, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 37, 0);}}   
   if   (MeloDecayTIMER[2][38] != 0) {
	     MeloDecayTIMER[2][38]--;
	  if(MeloDecayTIMER[2][38] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 38, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 38, 0);}}         
   if   (MeloDecayTIMER[2][39] != 0) {
	     MeloDecayTIMER[2][39]--;
	  if(MeloDecayTIMER[2][39] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 39, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 39, 0);}}              
   if   (MeloDecayTIMER[2][40] != 0) {
	     MeloDecayTIMER[2][40]--;
	  if(MeloDecayTIMER[2][40] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 40, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 40, 0);}}              
   if   (MeloDecayTIMER[2][41] != 0) {
	     MeloDecayTIMER[2][41]--;
	  if(MeloDecayTIMER[2][41] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 41, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 41, 0);}}              
   if   (MeloDecayTIMER[2][42] != 0) {
	     MeloDecayTIMER[2][42]--;
	  if(MeloDecayTIMER[2][42] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 42, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 42, 0);}}             
   if   (MeloDecayTIMER[2][43] != 0) {
	     MeloDecayTIMER[2][43]--;
	  if(MeloDecayTIMER[2][43] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 43, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 43, 0);}}              
   if   (MeloDecayTIMER[2][44] != 0) {
	     MeloDecayTIMER[2][44]--;
	  if(MeloDecayTIMER[2][44] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 44, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 44, 0);}}              
   if   (MeloDecayTIMER[2][45] != 0) {
	     MeloDecayTIMER[2][45]--;
	  if(MeloDecayTIMER[2][45] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 45, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 45, 0);}}              
   if   (MeloDecayTIMER[2][46] != 0) {
	     MeloDecayTIMER[2][46]--;
	  if(MeloDecayTIMER[2][46] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 46, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 46, 0);}}              
   if   (MeloDecayTIMER[2][47] != 0) {
	     MeloDecayTIMER[2][47]--;
	  if(MeloDecayTIMER[2][47] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 47, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 47, 0);}}              
   if   (MeloDecayTIMER[2][48] != 0) {
	     MeloDecayTIMER[2][48]--;
	  if(MeloDecayTIMER[2][48] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 48, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 48, 0);}}              
   if   (MeloDecayTIMER[2][49] != 0) {
	     MeloDecayTIMER[2][49]--;
	  if(MeloDecayTIMER[2][49] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 49, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 49, 0);}}              
   if   (MeloDecayTIMER[2][50] != 0) {
	     MeloDecayTIMER[2][50]--;
	  if(MeloDecayTIMER[2][50] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 50, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 50, 0);}}              
   if   (MeloDecayTIMER[2][51] != 0) {
	     MeloDecayTIMER[2][51]--;
	  if(MeloDecayTIMER[2][51] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 51, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 51, 0);}}              
   if   (MeloDecayTIMER[2][52] != 0) {
	     MeloDecayTIMER[2][52]--;
	  if(MeloDecayTIMER[2][52] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 52, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 52, 0);}}    
   if   (MeloDecayTIMER[2][53] != 0) {
	     MeloDecayTIMER[2][53]--;
	  if(MeloDecayTIMER[2][53] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 53, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 53, 0);}}              
   if   (MeloDecayTIMER[2][54] != 0) {
	     MeloDecayTIMER[2][54]--;
	  if(MeloDecayTIMER[2][54] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 54, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 54, 0);}}              
   if   (MeloDecayTIMER[2][55] != 0) {
	     MeloDecayTIMER[2][55]--;
	  if(MeloDecayTIMER[2][55] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 55, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 55, 0);}}              
   if   (MeloDecayTIMER[2][56] != 0) {
	     MeloDecayTIMER[2][56]--;
	  if(MeloDecayTIMER[2][56] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 56, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 56, 0);}}              
   if   (MeloDecayTIMER[2][57] != 0) {
	     MeloDecayTIMER[2][57]--;
	  if(MeloDecayTIMER[2][57] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 57, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 57, 0);}}              
   if   (MeloDecayTIMER[2][58] != 0) {
	     MeloDecayTIMER[2][58]--;
	  if(MeloDecayTIMER[2][58] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 58, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 58, 0);}}              
   if   (MeloDecayTIMER[2][59] != 0) {
	     MeloDecayTIMER[2][59]--;
	  if(MeloDecayTIMER[2][59] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 59, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 59, 0);}}              
   if   (MeloDecayTIMER[2][60] != 0) {
	     MeloDecayTIMER[2][60]--;
	  if(MeloDecayTIMER[2][60] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 60, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 60, 0);}}              
   if   (MeloDecayTIMER[2][61] != 0) {
	     MeloDecayTIMER[2][61]--;
	  if(MeloDecayTIMER[2][61] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 61, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 61, 0);}}              
   if   (MeloDecayTIMER[2][62] != 0) {
	     MeloDecayTIMER[2][62]--;
	  if(MeloDecayTIMER[2][62] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 62, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 62, 0);}}              
   if   (MeloDecayTIMER[2][63] != 0) {
	     MeloDecayTIMER[2][63]--;
	  if(MeloDecayTIMER[2][63] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 63, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 63, 0);}}              
   if   (MeloDecayTIMER[2][64] != 0) {
	     MeloDecayTIMER[2][64]--;
	  if(MeloDecayTIMER[2][64] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 64, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 64, 0);}}              
   if   (MeloDecayTIMER[2][65] != 0) {
	     MeloDecayTIMER[2][65]--;
	  if(MeloDecayTIMER[2][65] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 65, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 65, 0);}}              
   if   (MeloDecayTIMER[2][66] != 0) {
	     MeloDecayTIMER[2][66]--;
	  if(MeloDecayTIMER[2][66] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 66, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 66, 0);}}              
   if   (MeloDecayTIMER[2][67] != 0) {
	     MeloDecayTIMER[2][67]--;
	  if(MeloDecayTIMER[2][67] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 67, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 67, 0);}}              
   if   (MeloDecayTIMER[2][68] != 0) {
	     MeloDecayTIMER[2][68]--;
	  if(MeloDecayTIMER[2][68] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 68, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 68, 0);}}              
   if   (MeloDecayTIMER[2][69] != 0) {
	     MeloDecayTIMER[2][69]--;
	  if(MeloDecayTIMER[2][69] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 69, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 69, 0);}}              
   if   (MeloDecayTIMER[2][70] != 0) {
	     MeloDecayTIMER[2][70]--;
	  if(MeloDecayTIMER[2][70] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 70, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 70, 0);}}              
   if   (MeloDecayTIMER[2][71] != 0) {
	     MeloDecayTIMER[2][71]--;
	  if(MeloDecayTIMER[2][71] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 71, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 71, 0);}}              
   if   (MeloDecayTIMER[2][72] != 0) {
	     MeloDecayTIMER[2][72]--;
	  if(MeloDecayTIMER[2][72] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 72, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 72, 0);}}              
   if   (MeloDecayTIMER[2][73] != 0) {
	     MeloDecayTIMER[2][73]--;
	  if(MeloDecayTIMER[2][73] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 73, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 73, 0);}}              
   if   (MeloDecayTIMER[2][74] != 0) {
	     MeloDecayTIMER[2][74]--;
	  if(MeloDecayTIMER[2][74] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 74, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 74, 0);}}              
   if   (MeloDecayTIMER[2][75] != 0) {
	     MeloDecayTIMER[2][75]--;
	  if(MeloDecayTIMER[2][75] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 75, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 75, 0);}}              
   if   (MeloDecayTIMER[2][76] != 0) {
	     MeloDecayTIMER[2][76]--;
	  if(MeloDecayTIMER[2][76] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 76, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 76, 0);}}              
   if   (MeloDecayTIMER[2][77] != 0) {
	     MeloDecayTIMER[2][77]--;
	  if(MeloDecayTIMER[2][77] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 77, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 77, 0);}}              
   if   (MeloDecayTIMER[2][78] != 0) {
	     MeloDecayTIMER[2][78]--;
	  if(MeloDecayTIMER[2][78] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 78, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 78, 0);}}              
   if   (MeloDecayTIMER[2][79] != 0) {
	     MeloDecayTIMER[2][79]--;
	  if(MeloDecayTIMER[2][79] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 79, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 79, 0);}}              
   if   (MeloDecayTIMER[2][80] != 0) {
	     MeloDecayTIMER[2][80]--;
	  if(MeloDecayTIMER[2][80] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 80, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 80, 0);}}              
   if   (MeloDecayTIMER[2][81] != 0) {
	     MeloDecayTIMER[2][81]--;
	  if(MeloDecayTIMER[2][81] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 81, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 81, 0);}}              
   if   (MeloDecayTIMER[2][82] != 0) {
	     MeloDecayTIMER[2][82]--;
	  if(MeloDecayTIMER[2][82] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 82, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 82, 0);}}              
   if   (MeloDecayTIMER[2][83] != 0) {
	     MeloDecayTIMER[2][83]--;
	  if(MeloDecayTIMER[2][83] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 83, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 83, 0);}}              
   if   (MeloDecayTIMER[2][84] != 0) {
	     MeloDecayTIMER[2][84]--;
	  if(MeloDecayTIMER[2][84] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 84, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 84, 0);}}              
   if   (MeloDecayTIMER[2][85] != 0) {
	     MeloDecayTIMER[2][85]--;
	  if(MeloDecayTIMER[2][85] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 85, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 85, 0);}}              
   if   (MeloDecayTIMER[2][86] != 0) {
	     MeloDecayTIMER[2][86]--;
	  if(MeloDecayTIMER[2][86] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 86, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 86, 0);}}              
   if   (MeloDecayTIMER[2][87] != 0) {
	     MeloDecayTIMER[2][87]--;
	  if(MeloDecayTIMER[2][87] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 87, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 87, 0);}}              
   if   (MeloDecayTIMER[2][88] != 0) {
	     MeloDecayTIMER[2][88]--;
	  if(MeloDecayTIMER[2][88] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 88, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 88, 0);}}              
   if   (MeloDecayTIMER[2][89] != 0) {
	     MeloDecayTIMER[2][89]--;
	  if(MeloDecayTIMER[2][89] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 89, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 89, 0);}}                    
   if   (MeloDecayTIMER[2][90] != 0) {
	     MeloDecayTIMER[2][90]--;
	  if(MeloDecayTIMER[2][90] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 90, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 90, 0);}}              
   if   (MeloDecayTIMER[2][91] != 0) {
	     MeloDecayTIMER[2][91]--;
	  if(MeloDecayTIMER[2][91] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 91, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 91, 0);}}              
   if   (MeloDecayTIMER[2][92] != 0) {
	     MeloDecayTIMER[2][92]--;
	  if(MeloDecayTIMER[2][92] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 92, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 92, 0);}}              
   if   (MeloDecayTIMER[2][93] != 0) {
	     MeloDecayTIMER[2][93]--;
	  if(MeloDecayTIMER[2][93] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 93, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 93, 0);}}   
   if   (MeloDecayTIMER[2][94] != 0) {
	     MeloDecayTIMER[2][94]--;
	  if(MeloDecayTIMER[2][94] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 94, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 94, 0);}}               
   if   (MeloDecayTIMER[2][95] != 0) {
	     MeloDecayTIMER[2][95]--;
	  if(MeloDecayTIMER[2][95] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 95, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 95, 0);}}              
   if   (MeloDecayTIMER[2][96] != 0) {
	     MeloDecayTIMER[2][96]--;
	  if(MeloDecayTIMER[2][96] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 96, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 96, 0);}}              
   if   (MeloDecayTIMER[2][97] != 0) {
	     MeloDecayTIMER[2][97]--;
	  if(MeloDecayTIMER[2][97] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 97, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 97, 0);}}              
   if   (MeloDecayTIMER[2][98] != 0) {
	     MeloDecayTIMER[2][98]--;
	  if(MeloDecayTIMER[2][98] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 98, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 98, 0);}}              
   if   (MeloDecayTIMER[2][99] != 0) {
	     MeloDecayTIMER[2][99]--;
	  if(MeloDecayTIMER[2][99] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 99, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 99, 0);}}              
   if   (MeloDecayTIMER[2][100] != 0) {
	     MeloDecayTIMER[2][100]--;
	  if(MeloDecayTIMER[2][100] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 100, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 100, 0);}}              
   if   (MeloDecayTIMER[2][101] != 0) {
	     MeloDecayTIMER[2][101]--;
	  if(MeloDecayTIMER[2][101] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 101, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 101, 0);}}              
   if   (MeloDecayTIMER[2][102] != 0) {
	     MeloDecayTIMER[2][102]--;
	  if(MeloDecayTIMER[2][102] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 102, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 102, 0);}}              
   if   (MeloDecayTIMER[2][103] != 0) {
	     MeloDecayTIMER[2][103]--;
	  if(MeloDecayTIMER[2][103] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 103, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 103, 0);}}              
   if   (MeloDecayTIMER[2][104] != 0) {
	     MeloDecayTIMER[2][104]--;
	  if(MeloDecayTIMER[2][104] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 104, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 104, 0);}}              
   if   (MeloDecayTIMER[2][105] != 0) {
	     MeloDecayTIMER[2][105]--;
	  if(MeloDecayTIMER[2][105] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 105, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 105, 0);}}              
   if   (MeloDecayTIMER[2][106] != 0) {
	     MeloDecayTIMER[2][106]--;
	  if(MeloDecayTIMER[2][106] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 106, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 106, 0);}}              
   if   (MeloDecayTIMER[2][107] != 0) {
	     MeloDecayTIMER[2][107]--;
	  if(MeloDecayTIMER[2][107] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 107, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 107, 0);}}              
   if   (MeloDecayTIMER[2][108] != 0) {
	     MeloDecayTIMER[2][108]--;
	  if(MeloDecayTIMER[2][108] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 108, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 108, 0);}}              
   if   (MeloDecayTIMER[2][109] != 0) {
	     MeloDecayTIMER[2][109]--;
	  if(MeloDecayTIMER[2][109] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 109, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 109, 0);}}              
   if   (MeloDecayTIMER[2][110] != 0) {
	     MeloDecayTIMER[2][110]--;
	  if(MeloDecayTIMER[2][110] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 110, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 110, 0);}}              
   if   (MeloDecayTIMER[2][111] != 0) {
	     MeloDecayTIMER[2][111]--;
	  if(MeloDecayTIMER[2][111] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 111, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 111, 0);}}              
   if   (MeloDecayTIMER[2][112] != 0) {
	     MeloDecayTIMER[2][112]--;
	  if(MeloDecayTIMER[2][112] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 112, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 112, 0);}}              
   if   (MeloDecayTIMER[2][113] != 0) {
	     MeloDecayTIMER[2][113]--;
	  if(MeloDecayTIMER[2][113] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 113, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 113, 0);}}              
   if   (MeloDecayTIMER[2][114] != 0) {
	     MeloDecayTIMER[2][114]--;
	  if(MeloDecayTIMER[2][114] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 114, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 114, 0);}}              
   if   (MeloDecayTIMER[2][115] != 0) {
	     MeloDecayTIMER[2][115]--;
	  if(MeloDecayTIMER[2][115] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 115, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 115, 0);}}              
   if   (MeloDecayTIMER[2][116] != 0) {
	     MeloDecayTIMER[2][116]--;
	  if(MeloDecayTIMER[2][116] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 116, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 116, 0);}}              
   if   (MeloDecayTIMER[2][117] != 0) {
	     MeloDecayTIMER[2][117]--;
	  if(MeloDecayTIMER[2][117] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 117, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 117, 0);}}              
   if   (MeloDecayTIMER[2][118] != 0) {
	     MeloDecayTIMER[2][118]--;
	  if(MeloDecayTIMER[2][118] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 118, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 118, 0);}}              
   if   (MeloDecayTIMER[2][119] != 0) {
	     MeloDecayTIMER[2][119]--;
	  if(MeloDecayTIMER[2][119] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 119, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 119, 0);}}              
   if   (MeloDecayTIMER[2][120] != 0) {
	     MeloDecayTIMER[2][120]--;
	  if(MeloDecayTIMER[2][120] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 120, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 120, 0);}}              
   if   (MeloDecayTIMER[2][121] != 0) {
	     MeloDecayTIMER[2][121]--;
	  if(MeloDecayTIMER[2][121] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 121, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 121, 0);}}              
   if   (MeloDecayTIMER[2][122] != 0) {
	     MeloDecayTIMER[2][122]--;
	  if(MeloDecayTIMER[2][122] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 122, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 122, 0);}}              
   if   (MeloDecayTIMER[2][123] != 0) {
	     MeloDecayTIMER[2][123]--;
	  if(MeloDecayTIMER[2][23] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 123, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 123, 0);}}              
   if   (MeloDecayTIMER[2][124] != 0) {
	     MeloDecayTIMER[2][124]--;
	  if(MeloDecayTIMER[2][124] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 124, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 124, 0);}}              
   if   (MeloDecayTIMER[2][125] != 0) {
	     MeloDecayTIMER[2][125]--;
	  if(MeloDecayTIMER[2][125] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 125, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 125, 0);}}              
   if   (MeloDecayTIMER[2][126] != 0) {
	     MeloDecayTIMER[2][126]--;
	  if(MeloDecayTIMER[2][126] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 126, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 126, 0);}}              
   if   (MeloDecayTIMER[2][127] != 0) {
	     MeloDecayTIMER[2][127]--;
	  if(MeloDecayTIMER[2][127] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[2], 127, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[2], 127, 0);}}                                       
}//End While
}//End TICK 0 >>> 1ms Task...
static void TASK_Midi_Decay3(void *pvParameters){  //Send Note OFFs after a while
   portTickType xLastExecutionTime;
   xLastExecutionTime = xTaskGetTickCount();       // Initialise the xLastExecutionTime variable on task entry

  while( 1 ) {
    vTaskDelayUntil(&xLastExecutionTime, 1 / portTICK_RATE_MS);
    
   if   (MeloDecayTIMER[3][1] != 0) {
	     MeloDecayTIMER[3][1]--;
	  if(MeloDecayTIMER[3][1] == 0) {
		   MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 1, 0); //Port, Chn, Note, Velocity 
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 1, 0);}}  
   if   (MeloDecayTIMER[3][2] != 0) {
	     MeloDecayTIMER[3][2]--;
	  if(MeloDecayTIMER[3][2] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 2, 0); //Port, Chn, Note, Velocity 
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 2, 0);}}   
   if   (MeloDecayTIMER[3][3] != 0) {
	     MeloDecayTIMER[3][3]--;
	  if(MeloDecayTIMER[3][3] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 3, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 3, 0);}}
   if   (MeloDecayTIMER[3][4] != 0) {
	     MeloDecayTIMER[3][4]--;
	  if(MeloDecayTIMER[3][4] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 4, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 4, 0);}} 
   if   (MeloDecayTIMER[3][5] != 0) {
	     MeloDecayTIMER[3][5]--;
	  if(MeloDecayTIMER[3][5] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 5, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 5, 0);}} 
   if   (MeloDecayTIMER[3][6] != 0) {
	     MeloDecayTIMER[3][6]--;
	  if(MeloDecayTIMER[3][6] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 6, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 6, 0);}} 
   if   (MeloDecayTIMER[3][7] != 0) {
	     MeloDecayTIMER[3][7]--;
	  if(MeloDecayTIMER[3][7] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 7, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 7, 0);}}   
   if   (MeloDecayTIMER[3][8] != 0) {
	     MeloDecayTIMER[3][8]--;
	  if(MeloDecayTIMER[3][8] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 8, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 8, 0);}}   
   if   (MeloDecayTIMER[3][9] != 0) {
	     MeloDecayTIMER[3][9]--;
	  if(MeloDecayTIMER[3][9] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 9, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 9, 0);}}   
   if   (MeloDecayTIMER[3][10] != 0) {
	     MeloDecayTIMER[3][10]--;
	  if(MeloDecayTIMER[3][10] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 10, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 10, 0);}}   
   if   (MeloDecayTIMER[3][11] != 0) {
	     MeloDecayTIMER[3][11]--;
	  if(MeloDecayTIMER[3][11] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 11, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 11, 0);}}   
   if   (MeloDecayTIMER[3][12] != 0) {
	     MeloDecayTIMER[3][12]--;
	  if(MeloDecayTIMER[3][12] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 12, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 12, 0);}}   
   if   (MeloDecayTIMER[3][13] != 0) {
	     MeloDecayTIMER[3][13]--;
	  if(MeloDecayTIMER[3][13] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 13, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 13, 0);}}   
   if   (MeloDecayTIMER[3][14] != 0) {
	     MeloDecayTIMER[3][14]--;
	  if(MeloDecayTIMER[3][14] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 14, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 14, 0);}}   
   if   (MeloDecayTIMER[3][15] != 0) {
	     MeloDecayTIMER[3][15]--;
	  if(MeloDecayTIMER[3][15] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 15, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 15, 0);}}   
   if   (MeloDecayTIMER[3][16] != 0) {
	     MeloDecayTIMER[3][16]--;
	  if(MeloDecayTIMER[3][16] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 16, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 16, 0);}}   
   if   (MeloDecayTIMER[3][17] != 0) {
	     MeloDecayTIMER[3][17]--;
	  if(MeloDecayTIMER[3][17] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 17, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 17, 0);}}   
   if   (MeloDecayTIMER[3][18] != 0) {
	     MeloDecayTIMER[3][18]--;
	  if(MeloDecayTIMER[3][18] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 18, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 18, 0);}}   
   if   (MeloDecayTIMER[3][19] != 0) {
	     MeloDecayTIMER[3][19]--;
	  if(MeloDecayTIMER[3][19] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 19, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 19, 0);}}   
   if   (MeloDecayTIMER[3][20] != 0) {
	     MeloDecayTIMER[3][20]--;
	  if(MeloDecayTIMER[3][20] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 20, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 20, 0);}}   
   if   (MeloDecayTIMER[3][21] != 0) {
	     MeloDecayTIMER[3][21]--;
	  if(MeloDecayTIMER[3][21] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 21, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 21, 0);}}   
   if   (MeloDecayTIMER[3][22] != 0) {
	     MeloDecayTIMER[3][22]--;
	  if(MeloDecayTIMER[3][22] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 22, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 22, 0);}}   
   if   (MeloDecayTIMER[3][23] != 0) {
	     MeloDecayTIMER[3][23]--;
	  if(MeloDecayTIMER[3][23] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 23, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 23, 0);}}   
   if   (MeloDecayTIMER[3][24] != 0) {
	     MeloDecayTIMER[3][24]--;
	  if(MeloDecayTIMER[3][24] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 24, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 24, 0);}}   
   if   (MeloDecayTIMER[3][25] != 0) {
	     MeloDecayTIMER[3][25]--;
	  if(MeloDecayTIMER[3][25] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 25, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 25, 0);}}   
   if   (MeloDecayTIMER[3][26] != 0) {
	     MeloDecayTIMER[3][26]--;
	  if(MeloDecayTIMER[3][26] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 26, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 26, 0);}}   
   if   (MeloDecayTIMER[3][27] != 0) {
	     MeloDecayTIMER[3][27]--;
	  if(MeloDecayTIMER[3][27] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 27, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 27, 0);}}   
   if   (MeloDecayTIMER[3][28] != 0) {
	     MeloDecayTIMER[3][28]--;
	  if(MeloDecayTIMER[3][28] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 28, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 28, 0);}}   
   if   (MeloDecayTIMER[3][29] != 0) {
	     MeloDecayTIMER[3][29]--;
	  if(MeloDecayTIMER[3][29] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 29, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 29, 0);}}   
   if   (MeloDecayTIMER[3][30] != 0) {
	     MeloDecayTIMER[3][30]--;
	  if(MeloDecayTIMER[3][30] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 30, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 30, 0);}}   
   if   (MeloDecayTIMER[3][31] != 0) {
	     MeloDecayTIMER[3][31]--;
	  if(MeloDecayTIMER[3][31] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 31, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 31, 0);}}   
   if   (MeloDecayTIMER[3][32] != 0) {
	     MeloDecayTIMER[3][32]--;
	  if(MeloDecayTIMER[3][32] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 32, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 32, 0);}}   
   if   (MeloDecayTIMER[3][33] != 0) {
	     MeloDecayTIMER[3][33]--;
	  if(MeloDecayTIMER[3][33] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 33, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 33, 0);}}   
   if   (MeloDecayTIMER[3][34] != 0) {
	     MeloDecayTIMER[3][34]--;
	  if(MeloDecayTIMER[3][34] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 34, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 34, 0);}}   
   if   (MeloDecayTIMER[3][35] != 0) {
	     MeloDecayTIMER[3][35]--;
	  if(MeloDecayTIMER[3][35] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 35, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 35, 0);}}   
   if   (MeloDecayTIMER[3][36] != 0) {
	     MeloDecayTIMER[3][36]--;
	  if(MeloDecayTIMER[3][36] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 36, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 36, 0);}}   
   if   (MeloDecayTIMER[3][37] != 0) {
	     MeloDecayTIMER[3][37]--;
	  if(MeloDecayTIMER[3][37] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 37, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 37, 0);}}   
   if   (MeloDecayTIMER[3][38] != 0) {
	     MeloDecayTIMER[3][38]--;
	  if(MeloDecayTIMER[3][38] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 38, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 38, 0);}}         
   if   (MeloDecayTIMER[3][39] != 0) {
	     MeloDecayTIMER[3][39]--;
	  if(MeloDecayTIMER[3][39] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 39, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 39, 0);}}              
   if   (MeloDecayTIMER[3][40] != 0) {
	     MeloDecayTIMER[3][40]--;
	  if(MeloDecayTIMER[3][40] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 40, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 40, 0);}}              
   if   (MeloDecayTIMER[3][41] != 0) {
	     MeloDecayTIMER[3][41]--;
	  if(MeloDecayTIMER[3][41] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 41, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 41, 0);}}              
   if   (MeloDecayTIMER[3][42] != 0) {
	     MeloDecayTIMER[3][42]--;
	  if(MeloDecayTIMER[3][42] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 42, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 42, 0);}}             
   if   (MeloDecayTIMER[3][43] != 0) {
	     MeloDecayTIMER[3][43]--;
	  if(MeloDecayTIMER[3][43] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 43, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 43, 0);}}              
   if   (MeloDecayTIMER[3][44] != 0) {
	     MeloDecayTIMER[3][44]--;
	  if(MeloDecayTIMER[3][44] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 44, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 44, 0);}}              
   if   (MeloDecayTIMER[3][45] != 0) {
	     MeloDecayTIMER[3][45]--;
	  if(MeloDecayTIMER[3][45] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 45, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 45, 0);}}              
   if   (MeloDecayTIMER[3][46] != 0) {
	     MeloDecayTIMER[3][46]--;
	  if(MeloDecayTIMER[3][46] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 46, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 46, 0);}}              
   if   (MeloDecayTIMER[3][47] != 0) {
	     MeloDecayTIMER[3][47]--;
	  if(MeloDecayTIMER[3][47] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 47, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 47, 0);}}              
   if   (MeloDecayTIMER[3][48] != 0) {
	     MeloDecayTIMER[3][48]--;
	  if(MeloDecayTIMER[3][48] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 48, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 48, 0);}}              
   if   (MeloDecayTIMER[3][49] != 0) {
	     MeloDecayTIMER[3][49]--;
	  if(MeloDecayTIMER[3][49] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 49, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 49, 0);}}              
   if   (MeloDecayTIMER[3][50] != 0) {
	     MeloDecayTIMER[3][50]--;
	  if(MeloDecayTIMER[3][50] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 50, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 50, 0);}}              
   if   (MeloDecayTIMER[3][51] != 0) {
	     MeloDecayTIMER[3][51]--;
	  if(MeloDecayTIMER[3][51] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 51, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 51, 0);}}              
   if   (MeloDecayTIMER[3][52] != 0) {
	     MeloDecayTIMER[3][52]--;
	  if(MeloDecayTIMER[3][52] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 52, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 52, 0);}}    
   if   (MeloDecayTIMER[3][53] != 0) {
	     MeloDecayTIMER[3][53]--;
	  if(MeloDecayTIMER[3][53] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 53, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 53, 0);}}              
   if   (MeloDecayTIMER[3][54] != 0) {
	     MeloDecayTIMER[3][54]--;
	  if(MeloDecayTIMER[3][54] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 54, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 54, 0);}}              
   if   (MeloDecayTIMER[3][55] != 0) {
	     MeloDecayTIMER[3][55]--;
	  if(MeloDecayTIMER[3][55] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 55, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 55, 0);}}              
   if   (MeloDecayTIMER[3][56] != 0) {
	     MeloDecayTIMER[3][56]--;
	  if(MeloDecayTIMER[3][56] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 56, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 56, 0);}}              
   if   (MeloDecayTIMER[3][57] != 0) {
	     MeloDecayTIMER[3][57]--;
	  if(MeloDecayTIMER[3][57] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 57, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 57, 0);}}              
   if   (MeloDecayTIMER[3][58] != 0) {
	     MeloDecayTIMER[3][58]--;
	  if(MeloDecayTIMER[3][58] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 58, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 58, 0);}}              
   if   (MeloDecayTIMER[3][59] != 0) {
	     MeloDecayTIMER[3][59]--;
	  if(MeloDecayTIMER[3][59] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 59, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 59, 0);}}              
   if   (MeloDecayTIMER[3][60] != 0) {
	     MeloDecayTIMER[3][60]--;
	  if(MeloDecayTIMER[3][60] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 60, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 60, 0);}}              
   if   (MeloDecayTIMER[3][61] != 0) {
	     MeloDecayTIMER[3][61]--;
	  if(MeloDecayTIMER[3][61] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 61, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 61, 0);}}              
   if   (MeloDecayTIMER[3][62] != 0) {
	     MeloDecayTIMER[3][62]--;
	  if(MeloDecayTIMER[3][62] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 62, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 62, 0);}}              
   if   (MeloDecayTIMER[3][63] != 0) {
	     MeloDecayTIMER[3][63]--;
	  if(MeloDecayTIMER[3][63] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 63, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 63, 0);}}              
   if   (MeloDecayTIMER[3][64] != 0) {
	     MeloDecayTIMER[3][64]--;
	  if(MeloDecayTIMER[3][64] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 64, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 64, 0);}}              
   if   (MeloDecayTIMER[3][65] != 0) {
	     MeloDecayTIMER[3][65]--;
	  if(MeloDecayTIMER[3][65] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 65, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 65, 0);}}              
   if   (MeloDecayTIMER[3][66] != 0) {
	     MeloDecayTIMER[3][66]--;
	  if(MeloDecayTIMER[3][66] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 66, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 66, 0);}}              
   if   (MeloDecayTIMER[3][67] != 0) {
	     MeloDecayTIMER[3][67]--;
	  if(MeloDecayTIMER[3][67] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 67, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 67, 0);}}              
   if   (MeloDecayTIMER[3][68] != 0) {
	     MeloDecayTIMER[3][68]--;
	  if(MeloDecayTIMER[3][68] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 68, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 68, 0);}}              
   if   (MeloDecayTIMER[3][69] != 0) {
	     MeloDecayTIMER[3][69]--;
	  if(MeloDecayTIMER[3][69] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 69, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 69, 0);}}              
   if   (MeloDecayTIMER[3][70] != 0) {
	     MeloDecayTIMER[3][70]--;
	  if(MeloDecayTIMER[3][70] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 70, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 70, 0);}}              
   if   (MeloDecayTIMER[3][71] != 0) {
	     MeloDecayTIMER[3][71]--;
	  if(MeloDecayTIMER[3][71] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 71, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 71, 0);}}              
   if   (MeloDecayTIMER[3][72] != 0) {
	     MeloDecayTIMER[3][72]--;
	  if(MeloDecayTIMER[3][72] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 72, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 72, 0);}}              
   if   (MeloDecayTIMER[3][73] != 0) {
	     MeloDecayTIMER[3][73]--;
	  if(MeloDecayTIMER[3][73] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 73, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 73, 0);}}              
   if   (MeloDecayTIMER[3][74] != 0) {
	     MeloDecayTIMER[3][74]--;
	  if(MeloDecayTIMER[3][74] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 74, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 74, 0);}}              
   if   (MeloDecayTIMER[3][75] != 0) {
	     MeloDecayTIMER[3][75]--;
	  if(MeloDecayTIMER[3][75] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 75, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 75, 0);}}              
   if   (MeloDecayTIMER[3][76] != 0) {
	     MeloDecayTIMER[3][76]--;
	  if(MeloDecayTIMER[3][76] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 76, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 76, 0);}}              
   if   (MeloDecayTIMER[3][77] != 0) {
	     MeloDecayTIMER[3][77]--;
	  if(MeloDecayTIMER[3][77] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 77, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 77, 0);}}              
   if   (MeloDecayTIMER[3][78] != 0) {
	     MeloDecayTIMER[3][78]--;
	  if(MeloDecayTIMER[3][78] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 78, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 78, 0);}}              
   if   (MeloDecayTIMER[3][79] != 0) {
	     MeloDecayTIMER[3][79]--;
	  if(MeloDecayTIMER[3][79] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 79, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 79, 0);}}              
   if   (MeloDecayTIMER[3][80] != 0) {
	     MeloDecayTIMER[3][80]--;
	  if(MeloDecayTIMER[3][80] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 80, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 80, 0);}}              
   if   (MeloDecayTIMER[3][81] != 0) {
	     MeloDecayTIMER[3][81]--;
	  if(MeloDecayTIMER[3][81] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 81, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 81, 0);}}              
   if   (MeloDecayTIMER[3][82] != 0) {
	     MeloDecayTIMER[3][82]--;
	  if(MeloDecayTIMER[3][82] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 82, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 82, 0);}}              
   if   (MeloDecayTIMER[3][83] != 0) {
	     MeloDecayTIMER[3][83]--;
	  if(MeloDecayTIMER[3][83] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 83, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 83, 0);}}              
   if   (MeloDecayTIMER[3][84] != 0) {
	     MeloDecayTIMER[3][84]--;
	  if(MeloDecayTIMER[3][84] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 84, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 84, 0);}}              
   if   (MeloDecayTIMER[3][85] != 0) {
	     MeloDecayTIMER[3][85]--;
	  if(MeloDecayTIMER[3][85] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 85, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 85, 0);}}              
   if   (MeloDecayTIMER[3][86] != 0) {
	     MeloDecayTIMER[3][86]--;
	  if(MeloDecayTIMER[3][86] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 86, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 86, 0);}}              
   if   (MeloDecayTIMER[3][87] != 0) {
	     MeloDecayTIMER[3][87]--;
	  if(MeloDecayTIMER[3][87] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 87, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 87, 0);}}              
   if   (MeloDecayTIMER[3][88] != 0) {
	     MeloDecayTIMER[3][88]--;
	  if(MeloDecayTIMER[3][88] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 88, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 88, 0);}}              
   if   (MeloDecayTIMER[3][89] != 0) {
	     MeloDecayTIMER[3][89]--;
	  if(MeloDecayTIMER[3][89] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 89, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 89, 0);}}                    
   if   (MeloDecayTIMER[3][90] != 0) {
	     MeloDecayTIMER[3][90]--;
	  if(MeloDecayTIMER[3][90] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 90, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 90, 0);}}              
   if   (MeloDecayTIMER[3][91] != 0) {
	     MeloDecayTIMER[3][91]--;
	  if(MeloDecayTIMER[3][91] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 91, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 91, 0);}}              
   if   (MeloDecayTIMER[3][92] != 0) {
	     MeloDecayTIMER[3][92]--;
	  if(MeloDecayTIMER[3][92] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 92, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 92, 0);}}              
   if   (MeloDecayTIMER[3][93] != 0) {
	     MeloDecayTIMER[3][93]--;
	  if(MeloDecayTIMER[3][93] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 93, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 93, 0);}}   
   if   (MeloDecayTIMER[3][94] != 0) {
	     MeloDecayTIMER[3][94]--;
	  if(MeloDecayTIMER[3][94] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 94, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 94, 0);}}               
   if   (MeloDecayTIMER[3][95] != 0) {
	     MeloDecayTIMER[3][95]--;
	  if(MeloDecayTIMER[3][95] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 95, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 95, 0);}}              
   if   (MeloDecayTIMER[3][96] != 0) {
	     MeloDecayTIMER[3][96]--;
	  if(MeloDecayTIMER[3][96] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 96, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 96, 0);}}              
   if   (MeloDecayTIMER[3][97] != 0) {
	     MeloDecayTIMER[3][97]--;
	  if(MeloDecayTIMER[3][97] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 97, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 97, 0);}}              
   if   (MeloDecayTIMER[3][98] != 0) {
	     MeloDecayTIMER[3][98]--;
	  if(MeloDecayTIMER[3][98] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 98, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 98, 0);}}              
   if   (MeloDecayTIMER[3][99] != 0) {
	     MeloDecayTIMER[3][99]--;
	  if(MeloDecayTIMER[3][99] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 99, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 99, 0);}}              
   if   (MeloDecayTIMER[3][100] != 0) {
	     MeloDecayTIMER[3][100]--;
	  if(MeloDecayTIMER[3][100] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 100, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 100, 0);}}              
   if   (MeloDecayTIMER[3][101] != 0) {
	     MeloDecayTIMER[3][101]--;
	  if(MeloDecayTIMER[3][101] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 101, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 101, 0);}}              
   if   (MeloDecayTIMER[3][102] != 0) {
	     MeloDecayTIMER[3][102]--;
	  if(MeloDecayTIMER[3][102] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 102, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 102, 0);}}              
   if   (MeloDecayTIMER[3][103] != 0) {
	     MeloDecayTIMER[3][103]--;
	  if(MeloDecayTIMER[3][103] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 103, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 103, 0);}}              
   if   (MeloDecayTIMER[3][104] != 0) {
	     MeloDecayTIMER[3][104]--;
	  if(MeloDecayTIMER[3][104] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 104, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 104, 0);}}              
   if   (MeloDecayTIMER[3][105] != 0) {
	     MeloDecayTIMER[3][105]--;
	  if(MeloDecayTIMER[3][105] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 105, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 105, 0);}}              
   if   (MeloDecayTIMER[3][106] != 0) {
	     MeloDecayTIMER[3][106]--;
	  if(MeloDecayTIMER[3][106] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 106, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 106, 0);}}              
   if   (MeloDecayTIMER[3][107] != 0) {
	     MeloDecayTIMER[3][107]--;
	  if(MeloDecayTIMER[3][107] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 107, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 107, 0);}}              
   if   (MeloDecayTIMER[3][108] != 0) {
	     MeloDecayTIMER[3][108]--;
	  if(MeloDecayTIMER[3][108] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 108, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 108, 0);}}              
   if   (MeloDecayTIMER[3][109] != 0) {
	     MeloDecayTIMER[3][109]--;
	  if(MeloDecayTIMER[3][109] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 109, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 109, 0);}}              
   if   (MeloDecayTIMER[3][110] != 0) {
	     MeloDecayTIMER[3][110]--;
	  if(MeloDecayTIMER[3][110] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 110, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 110, 0);}}              
   if   (MeloDecayTIMER[3][111] != 0) {
	     MeloDecayTIMER[3][111]--;
	  if(MeloDecayTIMER[3][111] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 111, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 111, 0);}}              
   if   (MeloDecayTIMER[3][112] != 0) {
	     MeloDecayTIMER[3][112]--;
	  if(MeloDecayTIMER[3][112] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 112, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 112, 0);}}              
   if   (MeloDecayTIMER[3][113] != 0) {
	     MeloDecayTIMER[3][113]--;
	  if(MeloDecayTIMER[3][113] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 113, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 113, 0);}}              
   if   (MeloDecayTIMER[3][114] != 0) {
	     MeloDecayTIMER[3][114]--;
	  if(MeloDecayTIMER[3][114] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 114, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 114, 0);}}              
   if   (MeloDecayTIMER[3][115] != 0) {
	     MeloDecayTIMER[3][115]--;
	  if(MeloDecayTIMER[3][115] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 115, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 115, 0);}}              
   if   (MeloDecayTIMER[3][116] != 0) {
	     MeloDecayTIMER[3][116]--;
	  if(MeloDecayTIMER[3][116] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 116, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 116, 0);}}              
   if   (MeloDecayTIMER[3][117] != 0) {
	     MeloDecayTIMER[3][117]--;
	  if(MeloDecayTIMER[3][117] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 117, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 117, 0);}}              
   if   (MeloDecayTIMER[3][118] != 0) {
	     MeloDecayTIMER[3][118]--;
	  if(MeloDecayTIMER[3][118] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 118, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 118, 0);}}              
   if   (MeloDecayTIMER[3][119] != 0) {
	     MeloDecayTIMER[3][119]--;
	  if(MeloDecayTIMER[3][119] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 119, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 119, 0);}}              
   if   (MeloDecayTIMER[3][120] != 0) {
	     MeloDecayTIMER[3][120]--;
	  if(MeloDecayTIMER[3][120] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 120, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 120, 0);}}              
   if   (MeloDecayTIMER[3][121] != 0) {
	     MeloDecayTIMER[3][121]--;
	  if(MeloDecayTIMER[3][121] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 121, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 121, 0);}}              
   if   (MeloDecayTIMER[3][122] != 0) {
	     MeloDecayTIMER[3][122]--;
	  if(MeloDecayTIMER[3][122] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 122, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 122, 0);}}              
   if   (MeloDecayTIMER[3][123] != 0) {
	     MeloDecayTIMER[3][123]--;
	  if(MeloDecayTIMER[3][23] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 123, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 123, 0);}}              
   if   (MeloDecayTIMER[3][124] != 0) {
	     MeloDecayTIMER[3][124]--;
	  if(MeloDecayTIMER[3][124] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 124, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 124, 0);}}              
   if   (MeloDecayTIMER[3][125] != 0) {
	     MeloDecayTIMER[3][125]--;
	  if(MeloDecayTIMER[3][125] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 125, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 125, 0);}}              
   if   (MeloDecayTIMER[3][126] != 0) {
	     MeloDecayTIMER[3][126]--;
	  if(MeloDecayTIMER[3][126] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 126, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 126, 0);}}              
   if   (MeloDecayTIMER[3][127] != 0) {
	     MeloDecayTIMER[3][127]--;
	  if(MeloDecayTIMER[3][127] == 0) {
           MIOS32_MIDI_SendNoteOff(MelPortOut, MeloOutCh[3], 127, 0);
           MIOS32_MIDI_SendNoteOn (MelPortOut, MeloOutCh[3], 127, 0);}}                                       
}//End While
}//End TICK 0 >>> 1ms Task...
